/* Output from p2c, the Pascal-to-C translator */
/* From input file "kriber.pas" */


#include "p2c.h"


/*#### BK ut */
/*#### BK loadatinput */
/*#### BK ew */
/******************************************************************************/
/*                                                                            */
/*   Program  KRIBER              Version 1.0       (January 1991)            */
/*   ===============                                                          */
/*                                                                            */
/*   Author:   Roland Bialek, Institut fuer Kristallografie und Petrografie   */
/*             ETH-Zentrum, CH-8092 Zuerich, Switzerland                      */
/*                                                                            */
/*   an interactive PASCAL program to                                         */
/*    - calculate distances and angles                                        */
/*    - generate input files for the programs DLS-76 and LOADAT (XRS-82)      */
/*    - calculate coordination sequences and loop configurations              */
/*                                                                            */
/*   Program Installation and Modification                                    */
/*   -------------------------------------                                    */
/*  The program is written in standard PASCAL and has been implemented and    */
/*  fully tested on a CYBER 855 running under NOS/VE 1.5.2. It has also been  */
/*  used successfully on a MicroVax II machine under VMS 5.2                  */
/*                                                                            */
/*  There is only one extension to the standard PASCAL used: The identifiers  */
/*  contain the underscore (_) and the dollar sign ($). If your PASCAL        */
/*  does not have this extension, you can remove these two signs with your    */
/*  editor.                                                                   */
/*                                                                            */
/*  Input:  The program runs interactively.                                   */
/*                                                                            */
/*  Output: The output is written to the terminal screen (longest line is     */
/*          80 columns wide).                                                 */
/*                                                                            */
/*  Files:  SYMDAT, STRUDAT, DISTDAT and COSEQ are input files, DLSINPUT,     */
/*          LOADATINPUT, LIST and STRUDATNEW are output files.                */
/*                                                                            */
/*  Arrays: Following constants can be changed in the constant declaration    */
/*          part:                                                             */
/*          maxanzahlatomlagen    :   maximum number of atom positions        */
/*          maxanzahlbindungen    :   maximum number of bonds per atom        */
/*          maxanzahlsymeqkarten  :   maximum number of symeq cards           */
/*          maxindepwinkel        :   maximum number of independent angles    */
/*          maxindepdistanzen     :   maximum number of independent distances */
/*          maxanzahlgegwinkel    :   maximum number of prescribed angles     */
/*          maxanzahlgegdistanzen :   maximum number of prescribed distances  */
/*          maxanzahldistanzen    :   maximum number of calculated distances  */
/*                                                                            */
/*  Modified by Bernt Karasch, modifications marked with '### BK'             */
/*                                                                            */
/******************************************************************************/

#define maxanzahlatomlagen  100
#define maxanzahlbindungen  8
#define maxanzahlsymeqkarten  100
#define maxindepwinkel  100
#define maxindepdistanzen  100
#define maxanzahlgegwinkel  50
#define maxanzahlgegdistanzen  50
#define maxanzahldistanzen  9


typedef double vektor[3];
typedef long gittervektor[3];
typedef double tensor[3][3];
typedef long rotationsmatrix[3][3];

typedef struct symmetrieoperator {
  rotationsmatrix r;
  vektor t;
} symmetrieoperator;

typedef symmetrieoperator st_symop_array[48];
typedef Char st_kristallsystem[4];
typedef Char st_raumgruppename[30];

typedef struct st_raumgruppe {
  long raumgruppenr;
  st_raumgruppename raumgruppename;
  st_kristallsystem kristallsystem;
  long zaehligkeit;
  symmetrieoperator *symop[192];
  long anz_rotmat, anztranslationen;
  vektor translation[4];
  boolean zentrosymm;
  long multtab[48][48];
} st_raumgruppe;

typedef Char abkbezeichnung[10];
typedef Char string80[80];

typedef struct zellparameter {
  double laenge[3], winkel[3];
} zellparameter;

typedef Char kt_atomlagename[6];

typedef struct kt_atom {
  struct kt_atomart *dessen_atomart;
  gittervektor elementarzelle;
} kt_atom;

typedef struct kt_atomart {
  struct kt_atomlage *deren_atomlage;
  long atomartnr;
  vektor pkoord;
  symmetrieoperator trilinpol;
  kt_atom bindung[maxanzahlbindungen];
} kt_atomart;

typedef Char kt_element[2];

typedef struct kt_atomlage {
  long atomlagenr;
  vektor atomlagekoord;
  symmetrieoperator *symmetrielage;
  kt_atomlagename aname;
  kt_element element;
  long koordinationszahl, anzatome;
  kt_atomart *atom[192];
  long intifa[192];
  struct distanzen_tabellen_eintrag *dist_entry;
  kt_atom grundatom;
} kt_atomlage;

typedef struct kt_strukturdaten {
  string80 name, lit;
  abkbezeichnung abk;
  zellparameter zellpar;
  st_raumgruppename raumgruppe;
  st_kristallsystem kristallsystem;
  long anzatomlagen;
  kt_atomlage *atomlage[maxanzahlatomlagen];
} kt_strukturdaten;

typedef struct distanzen_tabellen_eintrag {
  double laenge;
  kt_atom endatom;
  struct distanzen_tabellen_eintrag *naechste_distanz;
} distanzen_tabellen_eintrag;

typedef long tt_koord_sequenz[10];
typedef enum {
  struktur, distanzen, bindungen
} berechenbare_dinge;
typedef Char mt_befehl[8];


Static _TEXT strudat, symdat;
/*#### BK ew */
Static _TEXT strudatn, distdat;
/*#### BK ut */
Static _TEXT dlsinp;
/*#### BK loadatinput */
Static _TEXT loadainp, list, coseq;
Static mt_befehl befehl;
Static uchar berechnet;
Static boolean strudatnew_offen;
Static tensor st_metrik_tensor;
Static kt_strukturdaten strukturdaten;
Static st_raumgruppe rginfo;


Static double skalar_produkt(a, b)
double *a, *b;
{
  double x;
  long i, j;

  x = 0.0;
  for (i = 0; i <= 2; i++) {
    for (j = 0; j <= 2; j++)
      x += a[i] * st_metrik_tensor[i][j] * b[j];
  }
  return x;
}


Static Void load_zellparameter(r1, r2, r3, w1, w2, w3)
double r1, r2, r3, w1, w2, w3;
{
  st_metrik_tensor[0][0] = r1 * r1;
  st_metrik_tensor[1][1] = r2 * r2;
  st_metrik_tensor[2][2] = r3 * r3;
  st_metrik_tensor[0][1] = r1 * r2 * cos(w3 * 0.0174533);
  st_metrik_tensor[1][2] = r2 * r3 * cos(w1 * 0.0174533);
  st_metrik_tensor[2][0] = r3 * r1 * cos(w2 * 0.0174533);
  st_metrik_tensor[1][0] = r1 * r2 * cos(w3 * 0.0174533);
  st_metrik_tensor[2][1] = r2 * r3 * cos(w1 * 0.0174533);
  st_metrik_tensor[0][2] = r3 * r1 * cos(w2 * 0.0174533);
}


Static boolean gleiche_vektoren(v1, v2)
double *v1, *v2;
{
  return (fabs(v1[0] - v2[0]) < 0.001 && fabs(v1[1] - v2[1]) < 0.001 &&
	  fabs(v1[2] - v2[2]) < 0.001);
}


Static boolean gleiche_rotationsmatrizen(x, y)
long (*x)[3], (*y)[3];
{
  return (x[0][0] == y[0][0] && x[0][1] == y[0][1] && x[0][2] == y[0][2] &&
	  x[1][0] == y[1][0] && x[1][1] == y[1][1] && x[1][2] == y[1][2] &&
	  x[2][0] == y[2][0] && x[2][1] == y[2][1] && x[2][2] == y[2][2]);
}


Static boolean gleiche_symmetrieoperatoren(x, y)
symmetrieoperator x, y;
{
  return (gleiche_rotationsmatrizen(x.r, y.r) & gleiche_vektoren(x.t, y.t));
}


Static Void frac_vektor(va, vb)
double *va, *vb;
{
  long i;

  for (i = 0; i <= 2; i++) {
    if (va[i] + 0.0000001 < 0)
      vb[i] = va[i] - (long)(va[i] + 0.0000001) + 1;
    else
      vb[i] = va[i] - (long)(va[i] + 0.0000001);
  }
}


Static Void int_vektor(v, g)
double *v;
long *g;
{
  long i;

  for (i = 0; i <= 2; i++) {
    if (v[i] + 0.0000001 < 0)
      g[i] = (long)(v[i] + 0.0000001) - 1;
    else
      g[i] = (long)(v[i] + 0.0000001);
  }
}


Static Void write_vektor(f, v)
_TEXT *f;
double *v;
{
  fprintf(f->f, " (%7.4f%8.4f%8.4f)", v[0], v[1], v[2]);
}


Static Void set_einheits_symop(s)
symmetrieoperator *s;
{
  long i, j;

  for (i = 0; i <= 2; i++)
    s->t[i] = 0.0;
  for (i = 0; i <= 2; i++) {
    for (j = 0; j <= 2; j++)
      s->r[i][j] = 0;
  }
  for (i = 0; i <= 2; i++)
    s->r[i][i] = 1;
}


Static Void mult_rotmat_mit_vektor(r, v, vn)
long (*r)[3];
double *v, *vn;
{
  vn[0] = r[0][0] * v[0] + r[0][1] * v[1] + r[0][2] * v[2];
  vn[1] = r[1][0] * v[0] + r[1][1] * v[1] + r[1][2] * v[2];
  vn[2] = r[2][0] * v[0] + r[2][1] * v[1] + r[2][2] * v[2];
}


Static Void mult_symop_mit_vektor(s, v, vn)
symmetrieoperator s;
double *v, *vn;
{
  vn[0] = s.t[0] + s.r[0][0] * v[0] + s.r[0][1] * v[1] + s.r[0][2] * v[2];
  vn[1] = s.t[1] + s.r[1][0] * v[0] + s.r[1][1] * v[1] + s.r[1][2] * v[2];
  vn[2] = s.t[2] + s.r[2][0] * v[0] + s.r[2][1] * v[1] + s.r[2][2] * v[2];
}


Static Void mult_rotationsmatrizen(a, b, c)
long (*a)[3], (*b)[3], (*c)[3];
{
  long i, j;

  for (i = 0; i <= 2; i++) {
    for (j = 0; j <= 2; j++)
      c[i][j] = a[i][0] * b[0][j] + a[i][1] * b[1][j] + a[i][2] * b[2][j];
  }
}


Static Void mult_symmetrieoperatoren(a, b, c)
symmetrieoperator a, b, *c;
{
  mult_rotationsmatrizen(a.r, b.r, c->r);
  mult_symop_mit_vektor(a, b.t, c->t);
}


Static Void subtrahiere_symmetrieoperatoren(x, y, z)
symmetrieoperator x, y, *z;
{
  long i, j;

  for (i = 0; i <= 2; i++) {
    for (j = 0; j <= 2; j++)
      z->r[i][j] = x.r[i][j] - y.r[i][j];
  }
  for (i = 0; i <= 2; i++)
    z->t[i] = x.t[i] - y.t[i];
}


Static Void write_symmetrieoperator(t, x)
_TEXT *t;
symmetrieoperator x;
{
  long i, j, nn;
  _TEXT tt;
  boolean ersterterm;

  tt.f = NULL;
  *tt.name = '\0';
  if (*tt.name != '\0') {
    if (tt.f != NULL)
      tt.f = freopen(tt.name, "w", tt.f);
    else
      tt.f = fopen(tt.name, "w");
  } else {
    if (tt.f != NULL)
      rewind(tt.f);
    else
      tt.f = tmpfile();
  }
  if (tt.f == NULL)
    _EscIO(FileNotFound);
  SETUPBUF(tt.f, Char);
  for (i = 0; i <= 2; i++) {
    ersterterm = true;
    for (j = 1; j <= 3; j++) {
      if (x.r[i][j - 1] != 0) {
	if (x.r[i][j - 1] < 0)
	  putc('-', tt.f);
	else if (!ersterterm)
	  putc('+', tt.f);
	if (labs(x.r[i][j - 1]) != 1)
	  fprintf(tt.f, "%ld", labs(x.r[i][j - 1]));
	if (j == 1)
	  putc('X', tt.f);
	if (j == 2)
	  putc('Y', tt.f);
	if (j == 3)
	  putc('Z', tt.f);
	ersterterm = false;
      }
    }
    if (fabs(x.t[i]) > 0.001) {
      if (x.t[i] < 0)
	putc('-', tt.f);
      else
	putc('+', tt.f);
      nn = 0;
      do {
	nn++;
      } while (fabs(x.t[i] * nn - (long)floor(x.t[i] * nn + 0.5)) >= 0.001);
      fprintf(tt.f, "%ld", (long)floor(fabs(x.t[i] * nn) + 0.5));
      if (nn > 1)
	fprintf(tt.f, "/%ld", nn);
      ersterterm = false;
    } else if (ersterterm)
      putc('0', tt.f);
    if (i + 1 < 3)
      putc(',', tt.f);
  }
  for (i = 1; i <= 36; i++)
    putc(' ', tt.f);
  if (*tt.name != '\0') {
    if (tt.f != NULL)
      tt.f = freopen(tt.name, "r", tt.f);
    else
      tt.f = fopen(tt.name, "r");
  } else
    rewind(tt.f);
  if (tt.f == NULL)
    _EscIO(FileNotFound);
  RESETBUF(tt.f, Char);
  for (i = 1; i <= 22; i++) {
    putc(P_peek(tt.f), t->f);
    getc(tt.f);
  }
  if (tt.f != NULL)
    fclose(tt.f);
}


Static Void write_st_raumgruppe(t, rginfo)
_TEXT *t;
st_raumgruppe rginfo;
{
  long i, j;

  fprintf(t->f, " Symmetry information from file SYMDAT\n");
  fprintf(t->f, " space group %.30s\n\n", rginfo.raumgruppename);
  if (rginfo.anztranslationen > 0) {
    fprintf(t->f, "        ");
    for (i = 0; i < rginfo.anztranslationen; i++) {
      putc('(', t->f);
      for (j = 0; j <= 2; j++) {
	if (fabs(rginfo.translation[i][j]) < 0.0001)
	  fprintf(t->f, " 0");
	if (fabs(rginfo.translation[i][j] - 0.5) < 0.0001)
	  fprintf(t->f, " 1/2");
	if (fabs(rginfo.translation[i][j] - 1.0 / 3) < 0.0001)
	  fprintf(t->f, " 1/3");
	if (fabs(rginfo.translation[i][j] - 2.0 / 3) < 0.0001)
	  fprintf(t->f, " 2/3");
	if (j + 1 != 3)
	  putc(',', t->f);
      }
      fprintf(t->f, ")+   ");
    }
    fprintf(t->f, "\n\n");
  }
  for (i = 1; i <= rginfo.anz_rotmat; i++) {
    fprintf(t->f, " (%3ld) ", i);
    write_symmetrieoperator(t, *rginfo.symop[i - 1]);
    putc('\n', t->f);
  }
  putc('\n', t->f);
}


Static Void read_symmetrieoperator(f, symop)
_TEXT *f;
symmetrieoperator *symop;
{
  double i1, i2;
  long vorz, i, j, k;
  double rr;

  while (P_peek(f->f) == ' ')
    getc(f->f);
  for (i = 0; i <= 2; i++) {
    for (j = 0; j <= 2; j++)
      symop->r[i][j] = 0;
  }
  for (i = 0; i <= 2; i++)
    symop->t[i] = 0.0;
  k = 1;
  do {
    vorz = 1;
    if (P_peek(f->f) == ',') {
      k++;
      getc(f->f);
    }
    if (P_peek(f->f) == '-') {
      vorz = -1;
      getc(f->f);
    }
    if (P_peek(f->f) == '+')
      getc(f->f);
    if (isdigit(P_peek(f->f))) {
      fscanf(f->f, "%lg", &i1);
      if (!P_eoln(f->f)) {
	if (P_peek(f->f) == '/') {
	  getc(f->f);
	  fscanf(f->f, "%lg", &i2);
	  rr = i1 / i2;
	} else
	  rr = i1;
      } else
	rr = i1;
    } else if (P_peek(f->f) == 'Z' || P_peek(f->f) == 'Y' ||
	       P_peek(f->f) == 'X')
      rr = 1.0;
    rr *= vorz;
    if (P_peek(f->f) == 'X')
      symop->r[k - 1][0] = (long)floor(rr + 0.5);
    else if (P_peek(f->f) == 'Y')
      symop->r[k - 1][1] = (long)floor(rr + 0.5);
    else if (P_peek(f->f) == 'Z')
      symop->r[k - 1][2] = (long)floor(rr + 0.5);
    else
      symop->t[k - 1] = rr;
    if (P_peek(f->f) == 'Z' || P_peek(f->f) == 'Y' || P_peek(f->f) == 'X')
      getc(f->f);
    if (P_peek(f->f) == ')')
      getc(f->f);
  } while (!P_eoln(f->f));
}


Static Void readln_raumgruppename(t, rn)
_TEXT *t;
Char *rn;
{
  long i;

  for (i = 0; i <= 29; i++)
    rn[i] = ' ';
  while (P_peek(t->f) == ' ')
    getc(t->f);
  i = 0;
  while (!P_eoln(t->f)) {
    i++;
    rn[i - 1] = getc(t->f);
    if (rn[i - 1] == '\n')
      rn[i - 1] = ' ';
    if (islower(rn[i - 1]))
      rn[i - 1] = _toupper(rn[i - 1]);
  }
  fscanf(t->f, "%*[^\n]");
  getc(t->f);
}


Static Void suche_raumgruppe_entry(t, rn_, tra)
_TEXT *t;
Char *rn_;
symmetrieoperator *tra;
{
  st_raumgruppename rn;
  long i, j;
  st_raumgruppename rnd;
  symmetrieoperator tran;
  _TEXT tt;
  boolean b;

  memcpy(rn, rn_, sizeof(st_raumgruppename));
  tt.f = NULL;
  *tt.name = '\0';
  j = 0;
  for (i = 1; i <= 30; i++) {
    if (rn[i - 1] == '(')
      j = i;
  }
  if (j > 0) {
    rn[j - 1] = ' ';
    if (*tt.name != '\0') {
      if (tt.f != NULL)
	tt.f = freopen(tt.name, "w", tt.f);
      else
	tt.f = fopen(tt.name, "w");
    } else {
      if (tt.f != NULL)
	rewind(tt.f);
      else
	tt.f = tmpfile();
    }
    if (tt.f == NULL)
      _EscIO(FileNotFound);
    SETUPBUF(tt.f, Char);
    i = j;
    while (rn[i - 1] != ')') {
      i++;
      putc(rn[i - 1], tt.f);
    }
    putc('\n', tt.f);
    if (*tt.name != '\0') {
      if (tt.f != NULL)
	tt.f = freopen(tt.name, "r", tt.f);
      else
	tt.f = fopen(tt.name, "r");
    } else
      rewind(tt.f);
    if (tt.f == NULL)
      _EscIO(FileNotFound);
    RESETBUF(tt.f, Char);
    read_symmetrieoperator(&tt, tra);
    fscanf(tt.f, "%*[^\n]");
    getc(tt.f);
    for (i = j - 1; i <= 29; i++)
      rn[i] = ' ';
  } else
    set_einheits_symop(tra);
  if (*t->name != '\0') {
    if (t->f != NULL)
      t->f = freopen(t->name, "r", t->f);
    else
      t->f = fopen(t->name, "r");
  } else
    rewind(t->f);
  if (t->f == NULL)
    _EscIO(FileNotFound);
  RESETBUF(t->f, Char);
  j = 0;
  for (i = 0; i <= 29; i++) {
    if (rn[i] != ' ') {
      j++;
      rn[j - 1] = rn[i];
    }
  }
  for (i = j; i <= 29; i++)
    rn[i] = ' ';
  for (i = 0; i <= 29; i++)
    rnd[i] = ' ';
  do {
    b = false;
    do {
      if (BUFEOF(t->f))
	b = true;
      else if (P_peek(t->f) == '*')
	b = true;
      else {
	fscanf(t->f, "%*[^\n]");
	getc(t->f);
      }
    } while (!b);
    if (BUFEOF(t->f))
      b = false;
    else {
      getc(t->f);
      for (j = 0; j <= 15; j++) {
	rnd[j] = getc(t->f);
	if (rnd[j] == '\n')
	  rnd[j] = ' ';
      }
      if (!strncmp(rnd, "END                           ",
		   sizeof(st_raumgruppename)))
	b = false;
    }
  } while (strncmp(rnd, rn, sizeof(st_raumgruppename)) && b);
  if (!b) {
    printf(" ERROR  space group %.30s not found => P1\n", rn);
    memcpy(rn, "P1                            ", sizeof(st_raumgruppename));
    suche_raumgruppe_entry(t, rn, tra);
  }
  if (P_peek(t->f) == '=') {
    getc(t->f);
    getc(t->f);
    getc(t->f);
    readln_raumgruppename(t, rn);
    suche_raumgruppe_entry(t, rn, &tran);
    mult_symmetrieoperatoren(*tra, tran, tra);
  }
  if (tt.f != NULL)
    fclose(tt.f);
}


Static Void erstelle_multtab(rginfo)
st_raumgruppe *rginfo;
{
  long i, j;
  rotationsmatrix r;
  boolean b;
  long FORLIM, FORLIM1;

  FORLIM = rginfo->anz_rotmat;
  for (i = 0; i < FORLIM; i++) {
    FORLIM1 = rginfo->anz_rotmat;
    for (j = 0; j < FORLIM1; j++) {
      mult_rotationsmatrizen(rginfo->symop[i]->r, rginfo->symop[j]->r, r);
      b = false;
      rginfo->multtab[i][j] = rginfo->anz_rotmat + 1;
      do {
	rginfo->multtab[i][j]--;
	if (rginfo->multtab[i][j] == 0) {
	  printf(" ERROR  space group is wrong (not complete?)\n");
	  b = true;
	} else if (gleiche_rotationsmatrizen(r,
		     rginfo->symop[rginfo->multtab[i][j] - 1]->r))
	  b = true;
      } while (!b);
    }
  }
}


typedef double augm_matrix[4][4];


Local Void austauschschritt(s, x, y)
double (*s)[4];
long x, y;
{
  long i, j;

  for (i = 0; i <= 3; i++) {
    for (j = 0; j <= 3; j++) {
      if (i + 1 != x && j + 1 != y)
	s[i][j] -= s[x - 1][j] * s[i][y - 1] / s[x - 1][y - 1];
    }
  }
  for (i = 0; i <= 3; i++) {
    if (i + 1 != x)
      s[i][y - 1] /= s[x - 1][y - 1];
  }
  for (j = 0; j <= 3; j++) {
    if (j + 1 != y)
      s[x - 1][j] = -(s[x - 1][j] / s[x - 1][y - 1]);
  }
  s[x - 1][y - 1] = 1 / s[x - 1][y - 1];
}


Static Void inverser_symmetrieoperator(sa, sb)
symmetrieoperator sa, *sb;
{
  augm_matrix s, ss;
  long i, j, n;
  long z[4], k[4];
  boolean b;

  for (i = 0; i <= 2; i++) {
    for (j = 0; j <= 2; j++)
      s[i][j] = sa.r[i][j];
  }
  for (i = 0; i <= 2; i++)
    s[i][3] = sa.t[i];
  for (j = 0; j <= 2; j++)
    s[3][j] = 0.0;
  s[3][3] = 1.0;
  for (j = 1; j <= 4; j++)
    z[j - 1] = j;
  for (i = 1; i <= 4; i++)
    k[i - 1] = -i;
  b = false;
  do {
    i = 0;
    do {
      i++;
    } while (k[i - 1] >= 0);
    j = 0;
    do {
      j++;
    } while (z[j - 1] <= 0 || s[i - 1][j - 1] == 0);
    austauschschritt(s, i, j);
    n = k[i - 1];
    k[i - 1] = z[j - 1];
    z[j - 1] = n;
    if (z[0] < 0 && z[1] < 0 && z[2] < 0 && z[3] < 0)
      b = true;
  } while (!b);
  for (i = 0; i <= 3; i++) {
    for (j = 0; j <= 3; j++)
      ss[k[i] - 1][-z[j] - 1] = s[i][j];
  }
  for (i = 0; i <= 2; i++) {
    for (j = 0; j <= 2; j++)
      sb->r[i][j] = (long)floor(ss[i][j] + 0.5);
  }
  for (i = 0; i <= 2; i++)
    sb->t[i] = ss[i][3];
}


Local Void berechne_bravaistyp(rginfo)
st_raumgruppe *rginfo;
{
  rginfo->anztranslationen = 0;
  if (rginfo->raumgruppename[0] == 'P') {
    rginfo->anztranslationen = 0;
    return;
  }
  if (rginfo->raumgruppename[0] == 'A') {
    rginfo->anztranslationen = 1;
    rginfo->translation[0][0] = 0.0;
    rginfo->translation[0][1] = 1.0 / 2;
    rginfo->translation[0][2] = 1.0 / 2;
    return;
  }
  if (rginfo->raumgruppename[0] == 'B') {
    rginfo->anztranslationen = 1;
    rginfo->translation[0][0] = 1.0 / 2;
    rginfo->translation[0][1] = 0.0;
    rginfo->translation[0][2] = 1.0 / 2;
    return;
  }
  if (rginfo->raumgruppename[0] == 'C') {
    rginfo->anztranslationen = 1;
    rginfo->translation[0][0] = 1.0 / 2;
    rginfo->translation[0][1] = 1.0 / 2;
    rginfo->translation[0][2] = 0.0;
    return;
  }
  if (rginfo->raumgruppename[0] == 'F') {
    rginfo->anztranslationen = 3;
    rginfo->translation[0][0] = 0.0;
    rginfo->translation[0][1] = 1.0 / 2;
    rginfo->translation[0][2] = 1.0 / 2;
    rginfo->translation[1][0] = 1.0 / 2;
    rginfo->translation[1][1] = 0.0;
    rginfo->translation[1][2] = 1.0 / 2;
    rginfo->translation[2][0] = 1.0 / 2;
    rginfo->translation[2][1] = 1.0 / 2;
    rginfo->translation[2][2] = 0.0;
    return;
  }
  if (rginfo->raumgruppename[0] == 'I') {
    rginfo->anztranslationen = 1;
    rginfo->translation[0][0] = 1.0 / 2;
    rginfo->translation[0][1] = 1.0 / 2;
    rginfo->translation[0][2] = 1.0 / 2;
    return;
  }
  if (rginfo->raumgruppename[0] != 'R') {
    printf(" ERROR  space group %c has unknown centering\n",
	   rginfo->raumgruppename[0]);
    return;
  }
  if (!strncmp(rginfo->kristallsystem, "RHO ", sizeof(st_kristallsystem)))
    return;
  rginfo->anztranslationen = 2;
  rginfo->translation[0][0] = 1.0 / 3;
  rginfo->translation[0][1] = 2.0 / 3;
  rginfo->translation[0][2] = 2.0 / 3;
  rginfo->translation[1][0] = 2.0 / 3;
  rginfo->translation[1][1] = 1.0 / 3;
  rginfo->translation[1][2] = 1.0 / 3;
}

Local Void spieglesymopansymzent(rginfo)
st_raumgruppe *rginfo;
{
  long i;
  symmetrieoperator inv;
  long FORLIM;

  set_einheits_symop(&inv);
  for (i = 0; i <= 2; i++)
    inv.r[i][i] = -1;
  FORLIM = rginfo->zaehligkeit;
  for (i = 0; i < FORLIM; i++) {
    rginfo->symop[rginfo->zaehligkeit + i] =
      (symmetrieoperator *)Malloc(sizeof(symmetrieoperator));
    mult_symmetrieoperatoren(inv, *rginfo->symop[i],
			     rginfo->symop[rginfo->zaehligkeit + i]);
  }
  rginfo->zaehligkeit *= 2;
}

Local Void bestimme_setting(rginfo, kristallsystem)
st_raumgruppe rginfo;
Char *kristallsystem;
{
  boolean n[3];
  long rr[3];
  long i, j;
  symmetrieoperator *WITH;

  for (i = 0; i <= 2; i++)
    n[i] = false;
  for (i = 0; i < rginfo.zaehligkeit; i++) {
    WITH = rginfo.symop[i];
    for (j = 0; j <= 2; j++)
      rr[j] = WITH->r[0][j] + WITH->r[1][j] + WITH->r[2][j];
    if (rr[0] < 0 && rr[1] < 0 && rr[2] > 0)
      n[2] = true;
    if (rr[0] > 0 && rr[1] > 0 && rr[2] < 0)
      n[2] = true;
    if (rr[0] < 0 && rr[1] > 0 && rr[2] < 0)
      n[1] = true;
    if (rr[0] > 0 && rr[1] < 0 && rr[2] > 0)
      n[1] = true;
    if (rr[0] > 0 && rr[1] < 0 && rr[2] < 0)
      n[0] = true;
    if (rr[0] < 0 && rr[1] > 0 && rr[2] > 0)
      n[0] = true;
  }
  if (n[0] == false && n[1] == false && n[2] == true)
    memcpy(kristallsystem, "MON1", sizeof(st_kristallsystem));
  if (n[0] == false && n[1] == true && n[2] == false)
    memcpy(kristallsystem, "MON2", sizeof(st_kristallsystem));
  if (n[0] == true && n[1] == false && n[2] == false)
    memcpy(kristallsystem, "MON3", sizeof(st_kristallsystem));
}


Static Void read_raumgruppe_entry(symdat, rn, transf, rginfo)
_TEXT *symdat;
Char *rn;
symmetrieoperator transf;
st_raumgruppe *rginfo;
{
  long i, j, k;
  symmetrieoperator inv_transf;
  long FORLIM, FORLIM1;
  symmetrieoperator *WITH1;

  memcpy(rginfo->raumgruppename, rn, sizeof(st_raumgruppename));
  fscanf(symdat->f, "%ld", &rginfo->raumgruppenr);
  while (P_peek(symdat->f) == ' ')
    getc(symdat->f);
  for (i = 0; i <= 3; i++) {
    rginfo->kristallsystem[i] = getc(symdat->f);
    if (rginfo->kristallsystem[i] == '\n')
      rginfo->kristallsystem[i] = ' ';
  }
  fscanf(symdat->f, "%ld%*[^\n]", &i);
  getc(symdat->f);
  if (i == -1)
    rginfo->zentrosymm = true;
  else
    rginfo->zentrosymm = false;
  rginfo->anz_rotmat = 0;
  while (P_peek(symdat->f) == '*') {
    fscanf(symdat->f, "%*[^\n]");
    getc(symdat->f);
  }
  while (P_peek(symdat->f) != '=') {
    rginfo->anz_rotmat++;
    rginfo->symop[rginfo->anz_rotmat - 1] =
      (symmetrieoperator *)Malloc(sizeof(symmetrieoperator));
    read_symmetrieoperator(symdat, rginfo->symop[rginfo->anz_rotmat - 1]);
    fscanf(symdat->f, "%*[^\n]");
    getc(symdat->f);
  }
  rginfo->zaehligkeit = rginfo->anz_rotmat;
  if (rginfo->zentrosymm)
    spieglesymopansymzent(rginfo);
  inverser_symmetrieoperator(transf, &inv_transf);
  FORLIM = rginfo->zaehligkeit;
  for (i = 0; i < FORLIM; i++) {
    mult_symmetrieoperatoren(transf, *rginfo->symop[i], rginfo->symop[i]);
    mult_symmetrieoperatoren(*rginfo->symop[i], inv_transf, rginfo->symop[i]);
    frac_vektor(rginfo->symop[i]->t, rginfo->symop[i]->t);
  }
  berechne_bravaistyp(rginfo);
  rginfo->anz_rotmat = rginfo->zaehligkeit;
  FORLIM = rginfo->anztranslationen;
  for (i = 1; i <= FORLIM; i++) {
    FORLIM1 = rginfo->zaehligkeit;
    for (j = 0; j < FORLIM1; j++) {
      rginfo->symop[i * rginfo->zaehligkeit + j] =
	(symmetrieoperator *)Malloc(sizeof(symmetrieoperator));
      *rginfo->symop[i * rginfo->zaehligkeit + j] = *rginfo->symop[j];
      WITH1 = rginfo->symop[i * rginfo->zaehligkeit + j];
      for (k = 0; k <= 2; k++)
	WITH1->t[k] += rginfo->translation[i - 1][k];
      WITH1 = rginfo->symop[i * rginfo->zaehligkeit + j];
      frac_vektor(WITH1->t, WITH1->t);
    }
  }
  rginfo->zaehligkeit *= rginfo->anztranslationen + 1;
  erstelle_multtab(rginfo);
  if (!strncmp(rginfo->kristallsystem, "MON ", sizeof(st_kristallsystem)))
    bestimme_setting(*rginfo, rginfo->kristallsystem);
}


Static Void read_raumgruppe(t, rn, rg)
_TEXT *t;
Char *rn;
st_raumgruppe *rg;
{
  symmetrieoperator tra;

  suche_raumgruppe_entry(t, rn, &tra);
  read_raumgruppe_entry(t, rn, tra, rg);
}


Static boolean polare_richtung(v, rginfo)
double *v;
st_raumgruppe rginfo;
{
  boolean b;
  long i;
  vektor v2;

  b = true;
  for (i = 0; i < rginfo.zaehligkeit; i++) {
    mult_rotmat_mit_vektor(rginfo.symop[i]->r, v, v2);
    if (!gleiche_vektoren(v2, v))
      b = false;
  }
  return b;
}


Static boolean gleiche_atome(x, y)
kt_atom x, y;
{
  return (x.dessen_atomart == y.dessen_atomart &&
	  x.elementarzelle[0] == y.elementarzelle[0] &&
	  x.elementarzelle[1] == y.elementarzelle[1] &&
	  x.elementarzelle[2] == y.elementarzelle[2]);
}


Static Void berechne_atom_trilinpol(a, ssy)
kt_atom a;
symmetrieoperator *ssy;
{
  long i;

  *ssy = a.dessen_atomart->trilinpol;
  for (i = 0; i <= 2; i++)
    ssy->t[i] += a.elementarzelle[i];
}


typedef double sympar[193][4];


typedef long reihe[3];
typedef double matrix[3][4];


Local Void austauschschritt_(s, x, y)
double (*s)[4];
long x, y;
{
  long i, j;

  for (i = 1; i <= 192; i++) {
    for (j = 0; j <= 3; j++) {
      if (i != x && j + 1 != y)
	s[i][j] -= s[x][j] * s[i][y - 1] / s[x][y - 1];
    }
  }
  for (j = 0; j <= 3; j++) {
    if (j + 1 != y)
      s[x][j] = -(s[x][j] / s[x][y - 1]);
  }
  for (i = 1; i <= 192; i++)
    s[i][y - 1] = 0.0;
}

Local Void sym(sy, n, yy)
double (*sy)[4];
long *n;
double (*yy)[4];
{
  long i, j, m;
  long pa[3];

  for (i = 0; i <= 3; i++)
    sy[0][i] = 0.0;
  for (i = 0; i <= 2; i++)
    pa[i] = 0;
  for (i = 0; i <= 2; i++) {
    m = 192;
    do {
      m--;
    } while ((fabs(sy[m][n[i] - 1]) <= 0.00001 || m == pa[0] || m == pa[1] ||
	      m == pa[2]) && m != 0);
    if (m != 0) {
      austauschschritt_(sy, m, n[i]);
      pa[n[i] - 1] = m;
    }
  }
  for (i = 0; i <= 2; i++) {
    if (pa[i] != 0) {
      for (j = 0; j <= 3; j++)
	yy[i][j] = sy[pa[i]][j];
    } else {
      for (j = 0; j <= 3; j++)
	yy[i][j] = 0.0;
      yy[i][i] = 1.0;
    }
  }
}

Local Void simplex(sy, y)
double (*sy)[4];
symmetrieoperator *y;
{
  matrix yy;
  reihe n;
  sympar ssy;
  long i, j;
  boolean loesung_io;

  memcpy(ssy, sy, sizeof(sympar));
  n[0] = 1;
  n[1] = 2;
  n[2] = 3;
  sym(ssy, n, yy);
  loesung_io = true;
  for (i = 0; i <= 2; i++) {
    for (j = 0; j <= 2; j++) {
      if (fabs((long)floor(yy[i][j] + 0.5) - yy[i][j]) > 0.00001)
	loesung_io = false;
    }
  }
  for (i = 0; i <= 2; i++) {
    if (fabs(yy[i][i]) > 0.00001 && fabs(yy[i][3]) > 0.00001)
      loesung_io = false;
  }
  if (!loesung_io) {
    memcpy(ssy, sy, sizeof(sympar));
    n[0] = 2;
    n[1] = 1;
    n[2] = 3;
    sym(ssy, n, yy);
  }
  for (i = 0; i <= 2; i++) {
    for (j = 0; j <= 2; j++)
      y->r[i][j] = (long)floor(yy[i][j] + 0.5);
  }
  for (i = 0; i <= 2; i++)
    y->t[i] = yy[i][3];
}


Static Void berechne_symmetrielage(rginfo, v, s)
st_raumgruppe rginfo;
double *v;
symmetrieoperator *s;
{
  long i, j, k, n;
  vektor w;
  gittervektor g;
  sympar a;
  symmetrieoperator ss;

  n = 0;
  for (i = 1; i <= 192; i++) {
    for (j = 0; j <= 3; j++)
      a[i][j] = 0.0;
  }
  for (i = 1; i < rginfo.zaehligkeit; i++) {
    mult_symop_mit_vektor(*rginfo.symop[i], v, w);
    int_vektor(w, g);
    frac_vektor(w, w);
    if (gleiche_vektoren(v, w)) {
      ss = *rginfo.symop[i];
      for (j = 0; j <= 2; j++)
	ss.t[j] -= g[j];
      subtrahiere_symmetrieoperatoren(*rginfo.symop[0], ss, &ss);
      n++;
      for (j = 1; j <= 3; j++) {
	for (k = 0; k <= 2; k++)
	  a[n * 3 + j - 3][k] = ss.r[j - 1][k];
	a[n * 3 + j - 3][3] = ss.t[j - 1];
      }
    }
  }
  if (n > 0)
    simplex(a, s);
  else
    *s = *rginfo.symop[0];
}


Static Void berechne_atomlagekoordinaten(rginfo, atomlage)
st_raumgruppe rginfo;
kt_atomlage *atomlage;
{
  long i, j;
  gittervektor g;
  symmetrieoperator s;
  long FORLIM1;
  kt_atomart *WITH1;

  frac_vektor(atomlage->atomlagekoord, atomlage->atomlagekoord);
  atomlage->symmetrielage = (symmetrieoperator *)Malloc(sizeof(symmetrieoperator));
  berechne_symmetrielage(rginfo, atomlage->atomlagekoord,
			 atomlage->symmetrielage);
  int_vektor(atomlage->symmetrielage->t, g);
  frac_vektor(atomlage->symmetrielage->t, atomlage->symmetrielage->t);
  for (i = 0; i <= 2; i++)
    atomlage->atomlagekoord[i] -= g[i];
  atomlage->anzatome = 0;
  for (i = 0; i < rginfo.zaehligkeit; i++) {
    mult_symmetrieoperatoren(*rginfo.symop[i], *atomlage->symmetrielage, &s);
    frac_vektor(s.t, s.t);
    atomlage->intifa[i] = atomlage->anzatome + 1;
    FORLIM1 = atomlage->anzatome;
    for (j = 1; j <= FORLIM1; j++) {
      if (gleiche_symmetrieoperatoren(s, atomlage->atom[j - 1]->trilinpol))
	atomlage->intifa[i] = j;
    }
    if (atomlage->intifa[i] > atomlage->anzatome) {
      atomlage->anzatome++;
      atomlage->atom[atomlage->anzatome - 1] =
	(kt_atomart *)Malloc(sizeof(kt_atomart));
      WITH1 = atomlage->atom[atomlage->anzatome - 1];
      WITH1->trilinpol = s;
      WITH1->atomartnr = atomlage->anzatome;
      WITH1->deren_atomlage = atomlage;
      for (j = 0; j < maxanzahlbindungen; j++)
	WITH1->bindung[j].dessen_atomart = NULL;
    }
  }
  FORLIM1 = atomlage->anzatome;
  for (i = 0; i < FORLIM1; i++) {
    WITH1 = atomlage->atom[i];
    mult_symop_mit_vektor(WITH1->trilinpol, atomlage->atomlagekoord,
			  WITH1->pkoord);
    int_vektor(WITH1->pkoord, g);
    frac_vektor(WITH1->pkoord, WITH1->pkoord);
    for (j = 0; j <= 2; j++)
      WITH1->trilinpol.t[j] -= g[j];
  }
  memcpy(atomlage->atomlagekoord, atomlage->atom[0]->pkoord, sizeof(vektor));
}


Static Void write_atom_trilinpol(t, a)
_TEXT *t;
kt_atom a;
{
  symmetrieoperator ssy;

  berechne_atom_trilinpol(a, &ssy);
  write_symmetrieoperator(t, ssy);
}


Static Void alg_symmetrieoperation(s, a, b)
symmetrieoperator s;
kt_atom a, *b;
{
  long i;
  kt_atomlage *p;
  symmetrieoperator sa, sr;
  gittervektor ga, gr;

  p = a.dessen_atomart->deren_atomlage;
  berechne_atom_trilinpol(a, &sa);
  mult_symmetrieoperatoren(s, sa, &sa);
  int_vektor(sa.t, ga);
  frac_vektor(sa.t, sa.t);
  i = 0;
  do {
    i++;
    sr = p->atom[i - 1]->trilinpol;
    int_vektor(sr.t, gr);
    frac_vektor(sr.t, sr.t);
  } while (!gleiche_symmetrieoperatoren(sr, sa));
  b->dessen_atomart = p->atom[i - 1];
  for (i = 0; i <= 2; i++)
    b->elementarzelle[i] = ga[i] - gr[i];
}


Static Void berechne_atom_koordinaten(a, v)
kt_atom a;
double *v;
{
  long i;

  for (i = 0; i <= 2; i++)
    v[i] = a.elementarzelle[i] + a.dessen_atomart->pkoord[i];
}


Static Void write_atomart_name(f, a)
_TEXT *f;
kt_atomart a;
{
  long i;

  putc(' ', f->f);
  for (i = 0; i <= 3; i++)
    putc(a.deren_atomlage->aname[i], f->f);
  fprintf(f->f, "(%3ld)", a.atomartnr);
}


Static Void write_atom_name(f, a)
_TEXT *f;
kt_atom a;
{
  write_atomart_name(f, *a.dessen_atomart);
  fprintf(f->f, "(%2ld%2ld%2ld)",
	  a.elementarzelle[0], a.elementarzelle[1], a.elementarzelle[2]);
}


Static Void write_atom_koordinaten(t, a)
_TEXT *t;
kt_atom a;
{
  vektor v;

  berechne_atom_koordinaten(a, v);
  write_vektor(t, v);
}


Static Void write_strukturdaten(t, strukturdaten)
_TEXT *t;
kt_strukturdaten strukturdaten;
{
  long i;
  kt_atomlage *WITH;

  fprintf(t->f, "\n Name: %.80s\n", strukturdaten.name);
  fprintf(t->f, " Lit: %.80s\n\n", strukturdaten.lit);
  fprintf(t->f, " Cell parameters:  a = %6.3f  b = %6.3f  c = %6.3f\n",
	  strukturdaten.zellpar.laenge[0], strukturdaten.zellpar.laenge[1],
	  strukturdaten.zellpar.laenge[2]);
  fprintf(t->f,
	  "                   alpha = %6.3f  beta = %6.3f  gamma = %6.3f\n\n",
	  strukturdaten.zellpar.winkel[0], strukturdaten.zellpar.winkel[1],
	  strukturdaten.zellpar.winkel[2]);
  fprintf(t->f, " Space group: %.30s\n\n", strukturdaten.raumgruppe);
  fprintf(t->f, " Atoms with coordination number and atoms per unit cell  \n");
  for (i = 1; i <= strukturdaten.anzatomlagen; i++) {
    WITH = strukturdaten.atomlage[i - 1];
    fprintf(t->f, " (%2ld)   %.6s", i, WITH->aname);
    write_atom_koordinaten(t, WITH->grundatom);
    fprintf(t->f, "%4ld%4ld   ", WITH->koordinationszahl, WITH->anzatome);
    write_atom_trilinpol(t, WITH->grundatom);
    putc('\n', t->f);
  }
}


Static Void write_strukturdaten_karte(t, struda)
_TEXT *t;
kt_strukturdaten struda;
{
  long i;
  kt_atomlage *WITH;

  fprintf(t->f, "*XXX\n");
  fprintf(t->f, "%.80s\n", struda.name);
  fprintf(t->f, "%.80s\n", struda.lit);
  fprintf(t->f, "%.30s\n", struda.raumgruppe);
  fprintf(t->f, "%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f\n",
	  struda.zellpar.laenge[0], struda.zellpar.laenge[1],
	  struda.zellpar.laenge[2], struda.zellpar.winkel[0],
	  struda.zellpar.winkel[1], struda.zellpar.winkel[2]);
  for (i = 0; i < struda.anzatomlagen; i++) {
    WITH = struda.atomlage[i];
    fprintf(t->f, "%.6s%9.5f%9.5f%9.5f",
	    WITH->aname, WITH->atomlagekoord[0], WITH->atomlagekoord[1],
	    WITH->atomlagekoord[2]);
    if (WITH->koordinationszahl > 0)
      fprintf(t->f, "%8ld", WITH->koordinationszahl);
    putc('\n', t->f);
  }
  fprintf(t->f, "------------------------------\n");
}


Static Void write_atomlage_koordinaten(f, atomlage)
_TEXT *f;
kt_atomlage atomlage;
{
  long i;
  kt_atomart *WITH;

  fprintf(f->f, "\n Coordinates for atom %.6s\n", atomlage.aname);
  for (i = 0; i < atomlage.anzatome; i++) {
    WITH = atomlage.atom[i];
    write_atomart_name(f, *atomlage.atom[i]);
    fprintf(f->f, "  ");
    write_vektor(f, WITH->pkoord);
    fprintf(f->f, "    ");
    write_symmetrieoperator(f, WITH->trilinpol);
    putc('\n', f->f);
  }
}


Static Void read_string80(t, s)
_TEXT *t;
Char *s;
{
  long i;

  for (i = 0; i <= 79; i++)
    s[i] = ' ';
  i = 0;
  while (!P_eoln(t->f)) {
    i++;
    s[i - 1] = getc(t->f);
    if (s[i - 1] == '\n')
      s[i - 1] = ' ';
  }
  fscanf(t->f, "%*[^\n]");
  getc(t->f);
}


Static Void set_zellparameter(r1, r2, r3, r4, r5, r6, z)
double r1, r2, r3, r4, r5, r6;
zellparameter *z;
{
  z->laenge[0] = r1;
  z->laenge[1] = r2;
  z->laenge[2] = r3;
  z->winkel[0] = r4;
  z->winkel[1] = r5;
  z->winkel[2] = r6;
  load_zellparameter(r1, r2, r3, r4, r5, r6);
}


Static Void read_zellparameter(t, zellpar, kristallsystem)
_TEXT *t;
zellparameter *zellpar;
Char *kristallsystem;
{
  long i, ii;
  double r[6];

  if (!(P_peek(t->f) == ' ' || P_peek(t->f) == '+' || P_peek(t->f) == '-' ||
	isdigit(P_peek(t->f)))) {
    for (i = 0; i <= 3; i++) {
      kristallsystem[i] = getc(t->f);
      if (kristallsystem[i] == '\n')
	kristallsystem[i] = ' ';
    }
  }
  ii = 0;
  while (!P_eoln(t->f)) {
    ii++;
    fscanf(t->f, "%lg", &r[ii - 1]);
    while ((!P_eoln(t->f)) & (P_peek(t->f) == ' '))
      getc(t->f);
  }
  fscanf(t->f, "%*[^\n]");
  getc(t->f);
  if (ii == 6) {
    set_zellparameter(r[0], r[1], r[2], r[3], r[4], r[5], zellpar);
    return;
  }
  if (!strncmp(kristallsystem, "CUB ", sizeof(st_kristallsystem)) && ii == 1) {
    set_zellparameter(r[0], r[0], r[0], 90.0, 90.0, 90.0, zellpar);
    return;
  }
  if (!strncmp(kristallsystem, "TET ", sizeof(st_kristallsystem)) && ii == 2) {
    set_zellparameter(r[0], r[0], r[1], 90.0, 90.0, 90.0, zellpar);
    return;
  }
  if (!strncmp(kristallsystem, "ORT ", sizeof(st_kristallsystem)) && ii == 3) {
    set_zellparameter(r[0], r[1], r[2], 90.0, 90.0, 90.0, zellpar);
    return;
  }
  if (!strncmp(kristallsystem, "TRIG", sizeof(st_kristallsystem)) && ii == 2) {
    set_zellparameter(r[0], r[0], r[1], 90.0, 90.0, 120.0, zellpar);
    return;
  }
  if (!strncmp(kristallsystem, "HEX ", sizeof(st_kristallsystem)) && ii == 2) {
    set_zellparameter(r[0], r[0], r[1], 90.0, 90.0, 120.0, zellpar);
    return;
  }
  if (!strncmp(kristallsystem, "MON3", sizeof(st_kristallsystem)) && ii == 4) {
    set_zellparameter(r[0], r[1], r[2], r[3], 90.0, 90.0, zellpar);
    return;
  }
  if (!strncmp(kristallsystem, "MON1", sizeof(st_kristallsystem)) && ii == 4) {
    set_zellparameter(r[0], r[1], r[2], 90.0, 90.0, r[3], zellpar);
    return;
  }
  if (!strncmp(kristallsystem, "MON2", sizeof(st_kristallsystem)) && ii == 4) {
    set_zellparameter(r[0], r[1], r[2], 90.0, r[1], 90.0, zellpar);
    return;
  }
  if (!strncmp(kristallsystem, "RHO ", sizeof(st_kristallsystem)) && ii == 2)
    set_zellparameter(r[0], r[0], r[0], r[1], r[1], r[1], zellpar);
  else
    printf(" ERROR  cannot read the cell parameters\n");
}


Static Void suche_naechsten_eintrag(t, abk, b)
_TEXT *t;
Char *abk;
boolean *b;
{
  long i;

  *b = false;
  for (i = 0; i <= 9; i++)
    abk[i] = ' ';
  do {
    if (BUFEOF(t->f))
      *b = true;
    else if (P_peek(t->f) == '*')
      *b = true;
    else {
      fscanf(t->f, "%*[^\n]");
      getc(t->f);
    }
  } while (!*b);
  if (BUFEOF(t->f)) {
    *b = false;
    return;
  }
  getc(t->f);
  i = 0;
  while (!P_eoln(t->f) && i < 10) {
    i++;
    abk[i - 1] = getc(t->f);
    if (abk[i - 1] == '\n')
      abk[i - 1] = ' ';
  }
  fscanf(t->f, "%*[^\n]");
  getc(t->f);
  if (abk[9] != ' ')
    printf(" WARNING  entry name on file STRUDAT too long\n");
  if (!strncmp(abk, "END       ", sizeof(abkbezeichnung)) ||
      !strncmp(abk, "end       ", sizeof(abkbezeichnung)))
    *b = false;
}


Static Void suche_strukturdaten_eintrag(t, gegabk, b)
_TEXT *t;
Char *gegabk;
boolean *b;
{
  abkbezeichnung abk;

  do {
    suche_naechsten_eintrag(t, abk, b);
  } while (strncmp(abk, gegabk, sizeof(abkbezeichnung)) && *b);
  if (*b)
    printf(" ... entry %.10s found\n", abk);
  else
    printf(" ERROR  entry %.10s not found on file STRUDAT\n", gegabk);
}


Static Void erzeuge_atomlage(e, n, koord, kz, a)
Char *e;
long n;
double *koord;
long kz;
kt_atomlage **a;
{
  kt_atomlage *WITH;

  *a = (kt_atomlage *)Malloc(sizeof(kt_atomlage));
  WITH = *a;
  memcpy(WITH->atomlagekoord, koord, sizeof(vektor));
  memcpy(WITH->element, e, sizeof(kt_element));
  memcpy(WITH->aname, "      ", sizeof(kt_atomlagename));
  WITH->aname[0] = e[0];
  WITH->aname[1] = e[1];
  WITH->koordinationszahl = kz;
  WITH->dist_entry = NULL;
  if (n > 0) {
    if (n < 10)
      WITH->aname[2] = (Char)(n - 1 + '1');
    else {
      WITH->aname[2] = (Char)(n / 10 - 1 + '1');
      WITH->aname[3] = (Char)(n % 10 - 1 + '1');
/* p2c: kriber.pas, line 1071: Note: Using % for possibly-negative arguments [317]
 * Note: Using % for possibly-negative arguments [317] */
    }
  }
  if (WITH->aname[1] == ' ' && WITH->aname[2] != ' ') {
    WITH->aname[1] = WITH->aname[2];
    WITH->aname[2] = WITH->aname[3];
    WITH->aname[3] = ' ';
  }
  berechne_atomlagekoordinaten(rginfo, *a);
  (*a)->grundatom.elementarzelle[0] = 0;
  (*a)->grundatom.elementarzelle[1] = 0;
  (*a)->grundatom.elementarzelle[2] = 0;
  (*a)->grundatom.dessen_atomart = (*a)->atom[0];
}


Static Void lade_atomlage(neue_atomlage, struda, b)
kt_atomlage **neue_atomlage;
kt_strukturdaten *struda;
boolean *b;
{
  long i, j;
  kt_atomlage *WITH1;
  long FORLIM, FORLIM1;
  _TEXT TEMP;

  if (struda->anzatomlagen < maxanzahlatomlagen) {
    *b = true;
    WITH1 = *neue_atomlage;
    FORLIM = struda->anzatomlagen;
    for (i = 0; i < FORLIM; i++) {
      FORLIM1 = struda->atomlage[i]->anzatome;
      for (j = 0; j < FORLIM1; j++) {
	if (gleiche_vektoren(WITH1->atomlagekoord,
			     struda->atomlage[i]->atom[j]->pkoord)) {
	  printf(" ERROR  atom position %.6s has the same coordinates as atom",
		 WITH1->aname);
	  TEMP.f = stdout;
	  *TEMP.name = '\0';
	  write_atomart_name(&TEMP, *struda->atomlage[i]->atom[j]);
	  printf("\n        and therefore has not been loaded\n");
	  *b = false;
	}
      }
      if (!strncmp(WITH1->aname, struda->atomlage[i]->aname,
		   sizeof(kt_atomlagename)))
	printf(
	  " WARNING  atom position %.6s has same name as another atom position\n",
	  WITH1->aname);
    }
    if (!*b)
      return;
    struda->anzatomlagen++;
    struda->atomlage[struda->anzatomlagen - 1] = *neue_atomlage;
    struda->atomlage[struda->anzatomlagen - 1]->atomlagenr = struda->anzatomlagen;
    return;
  }
  *b = false;
  printf(" ERROR  too many atom positions. New atom position cannot be loaded.\n");
  printf("        The program is compiled for a maximum of %ld atom positions.\n",
	 (long)maxanzahlatomlagen);
  printf("        Increase constant \"maxanzahlatomlagen\" and compile again.\n");
}


Static Void read_atomlage(t, atomlage)
_TEXT *t;
kt_atomlage **atomlage;
{
  kt_element element;
  long atom_label, koordinationszahl;
  vektor koord;

  if (isalpha(P_peek(t->f))) {
    *element = getc(t->f);
    if (element[0] == '\n')
      element[0] = ' ';
  } else
    element[0] = ' ';
  if (isalpha(P_peek(t->f))) {
    element[1] = getc(t->f);
    if (element[1] == '\n')
      element[1] = ' ';
  } else
    element[1] = ' ';
  if (P_peek(t->f) == '0' || P_peek(t->f) >= '1' && P_peek(t->f) <= '9')
    fscanf(t->f, "%ld", &atom_label);
  else
    atom_label = 0;
  fscanf(t->f, "%lg%lg%lg", koord, &koord[1], &koord[2]);
  while ((!P_eoln(t->f)) & (P_peek(t->f) == ' '))
    getc(t->f);
  if (!P_eoln(t->f))
    fscanf(t->f, "%ld", &koordinationszahl);
  else {
    koordinationszahl = 0;
    if (!strncmp(element, "SI", sizeof(kt_element)) ||
	!strncmp(element, "Si", sizeof(kt_element)))
      koordinationszahl = 4;
    if (!strncmp(element, "AL", sizeof(kt_element)) ||
	!strncmp(element, "Al", sizeof(kt_element)))
      koordinationszahl = 4;
    if (!strncmp(element, "P ", sizeof(kt_element)))
      koordinationszahl = 4;
    if (!strncmp(element, "T ", sizeof(kt_element)))
      koordinationszahl = 4;
    if (!strncmp(element, "O ", sizeof(kt_element)))
      koordinationszahl = 2;
  }
  fscanf(t->f, "%*[^\n]");
  getc(t->f);
  erzeuge_atomlage(element, atom_label, koord, koordinationszahl, atomlage);
}


Static Void read_strukturdaten(t, strukturdaten, frage)
_TEXT *t;
kt_strukturdaten *strukturdaten;
boolean frage;
{
  kt_atomlage *neue_atomlage;
  boolean b;

  if (frage)
    printf(" Name: ");
  read_string80(t, strukturdaten->name);
  if (frage)
    printf(" Lit:  ");
  read_string80(t, strukturdaten->lit);
  if (frage)
    printf(" Space group: ");
  readln_raumgruppename(t, strukturdaten->raumgruppe);
  read_raumgruppe(&symdat, strukturdaten->raumgruppe, &rginfo);
  memcpy(strukturdaten->kristallsystem, rginfo.kristallsystem,
	 sizeof(st_kristallsystem));
  if (frage)
    printf(" Cell parameters: ");
  read_zellparameter(t, &strukturdaten->zellpar,
		     strukturdaten->kristallsystem);
  if (frage) {
    printf(" Add atom position,  end with '*' \n");
    printf(" label (element symbol + number)   coordinates   coordination number\n");
  }
  strukturdaten->anzatomlagen = 0;
  if (frage)
    printf(" Atom%3ld: ", strukturdaten->anzatomlagen + 1);
  while (isalpha(P_peek(t->f))) {
    read_atomlage(t, &neue_atomlage);
    frac_vektor(neue_atomlage->atomlagekoord, neue_atomlage->atomlagekoord);
    lade_atomlage(&neue_atomlage, strukturdaten, &b);
    if (frage)
      printf(" Atom%3ld: ", strukturdaten->anzatomlagen + 1);
  }
  if (frage) {
    fscanf(t->f, "%*[^\n]");
    getc(t->f);
  }
}


Static boolean atom_hat_gleiches_element(a, e)
kt_atom a;
Char *e;
{
  kt_atomlage *WITH;

  WITH = a.dessen_atomart->deren_atomlage;
  return (WITH->element[0] == e[0] && WITH->element[1] == e[1]);
}


Static boolean symmetrieaequivalente_atome(a, b)
kt_atom a, b;
{
  return (a.dessen_atomart->deren_atomlage == b.dessen_atomart->deren_atomlage);
}


Static Void welchsymop(a, b, rginfo, anzsymop, sr)
kt_atom a, b;
st_raumgruppe rginfo;
long *anzsymop;
symmetrieoperator *sr;
{
  long i, j;
  symmetrieoperator sa, sb, sc;
  gittervektor gb, gc;

  *anzsymop = 0;
  if (!symmetrieaequivalente_atome(a, b))
    return;
  berechne_atom_trilinpol(a, &sa);
  berechne_atom_trilinpol(b, &sb);
  int_vektor(sb.t, gb);
  frac_vektor(sb.t, sb.t);
  for (i = 0; i < rginfo.zaehligkeit; i++) {
    mult_symmetrieoperatoren(*rginfo.symop[i], sa, &sc);
    int_vektor(sc.t, gc);
    frac_vektor(sc.t, sc.t);
    if (gleiche_symmetrieoperatoren(sc, sb)) {
      (*anzsymop)++;
      sr[*anzsymop - 1] = *rginfo.symop[i];
      for (j = 0; j <= 2; j++)
	sr[*anzsymop - 1].t[j] += gb[j] - gc[j];
    }
  }
}


Static boolean sym_aequival_distanzen(a1, a2, a3, a4)
kt_atom a1, a2, a3, a4;
{
  st_symop_array symop;
  long anzsymop, i;
  kt_atom a3n;
  boolean b;

  b = false;
  welchsymop(a2, a4, rginfo, &anzsymop, symop);
  for (i = 0; i < anzsymop; i++) {
    alg_symmetrieoperation(symop[i], a1, &a3n);
    if (gleiche_atome(a3n, a3))
      b = true;
  }
  return b;
}


Static boolean sym_aequival_winkel(a1, a2, a3, a4, a5, a6)
kt_atom a1, a2, a3, a4, a5, a6;
{
  st_symop_array symop;
  long anzsymop, i;
  kt_atom a4n, a6n;
  boolean b;

  b = false;
  welchsymop(a2, a5, rginfo, &anzsymop, symop);
  for (i = 0; i < anzsymop; i++) {
    alg_symmetrieoperation(symop[i], a1, &a4n);
    alg_symmetrieoperation(symop[i], a3, &a6n);
    if (gleiche_atome(a4n, a4) & gleiche_atome(a6n, a6))
      b = true;
  }
  return b;
}


Static Void delete_atomlage(atomlage, struda)
kt_atomlage *atomlage;
kt_strukturdaten *struda;
{
  long j, n, FORLIM;

  n = atomlage->atomlagenr;
  atomlage->atomlagenr = 0;
  struda->anzatomlagen--;
  FORLIM = struda->anzatomlagen;
  for (j = n; j <= FORLIM; j++) {
    struda->atomlage[j - 1] = struda->atomlage[j];
    struda->atomlage[j - 1]->atomlagenr = j;
  }
}


Static Void read_atomlage_bez(f, pala)
_TEXT *f;
kt_atomlage **pala;
{
  long i;
  Char nn[6];
  long FORLIM;

  *pala = NULL;
  while ((!P_eoln(f->f)) & (P_peek(f->f) == ' '))
    getc(f->f);
  if (!P_eoln(f->f)) {
    for (i = 0; i <= 5; i++)
      nn[i] = ' ';
    i = 0;
    while ((P_peek(f->f) != ' ') & (P_peek(f->f) != '(') & (!P_eoln(f->f))) {
      i++;
      nn[i - 1] = getchar();
      if (nn[i - 1] == '\n')
	nn[i - 1] = ' ';
    }
    FORLIM = strukturdaten.anzatomlagen;
    for (i = 0; i < FORLIM; i++) {
      if (!strncmp(nn, strukturdaten.atomlage[i]->aname, 6)) {
/* p2c: kriber.pas, line 1284: Warning: Incompatible array sizes [164] */
	*pala = strukturdaten.atomlage[i];
      }
    }
  }
  if (*pala == NULL)
    printf(" ERROR  atom position %.6s not found\n", nn);
}


Static Void readln_atom(t, a)
_TEXT *t;
kt_atom *a;
{
  symmetrieoperator ss, ssy;
  long i, j, n;
  kt_atomlage *pala;
  gittervektor v;
  boolean b;
  long FORLIM;
  _TEXT TEMP;

  a->dessen_atomart = NULL;
  read_atomlage_bez(t, &pala);
  if (pala != NULL) {
    a->dessen_atomart = pala->atom[0];
    a->elementarzelle[0] = 0;
    a->elementarzelle[1] = 0;
    a->elementarzelle[2] = 0;
    while ((!P_eoln(t->f)) & (P_peek(t->f) == ' '))
      getc(t->f);
    if (!P_eoln(t->f)) {
      if (P_peek(t->f) == '(') {
	while ((!P_eoln(t->f)) & ((P_peek(t->f) == ' ') | (P_peek(t->f) ==
				    ')') | (P_peek(t->f) == '(')))
	  getc(t->f);
	if (!P_eoln(t->f))
	  scanf("%ld", &n);
	if (n > 0 && n <= a->dessen_atomart->deren_atomlage->anzatome) {
	  a->dessen_atomart = pala->atom[n - 1];
	  while ((!P_eoln(t->f)) & ((P_peek(t->f) == ' ') | (P_peek(t->f) ==
				      ')') | (P_peek(t->f) == '(')))
	    getc(t->f);
	  if (!P_eoln(t->f))
	    fscanf(t->f, "%ld%ld%ld", a->elementarzelle,
		   &a->elementarzelle[1], &a->elementarzelle[2]);
	} else {
	  printf(" ERROR  there is no atom ");
	  printf("%.6s (%ld)\n", a->dessen_atomart->deren_atomlage->aname, n);
	  a->dessen_atomart = NULL;
	}
      } else {
	read_symmetrieoperator(t, &ss);
	int_vektor(ss.t, a->elementarzelle);
	frac_vektor(ss.t, ss.t);
	b = true;
	FORLIM = pala->anzatome;
	for (i = 0; i < FORLIM; i++) {
	  ssy = pala->atom[i]->trilinpol;
	  int_vektor(ssy.t, v);
	  frac_vektor(ssy.t, ssy.t);
	  if (gleiche_symmetrieoperatoren(ss, ssy) && b) {
	    a->dessen_atomart = pala->atom[i];
	    for (j = 0; j <= 2; j++)
	      a->elementarzelle[j] -= v[j];
	    b = false;
	  }
	}
	if (b) {
	  printf(" ERROR  atom %.6s with symmetry operator ",
		 a->dessen_atomart->deren_atomlage->aname);
	  TEMP.f = stdout;
	  *TEMP.name = '\0';
	  write_symmetrieoperator(&TEMP, ss);
	  printf(" not found\n");
	  a->dessen_atomart = NULL;
	}
      }
    }
  }
  if (a->dessen_atomart == NULL)
    printf(" ERROR  atom not found\n");
  fscanf(t->f, "%*[^\n]");
  getc(t->f);
}


Static long koordinationszahl_des_atoms(a)
kt_atom a;
{
  return (a.dessen_atomart->deren_atomlage->koordinationszahl);
}


Static Void verschiebe_atome_leicht(atomlage)
kt_atomlage *atomlage;
{
  long i;
  double ran;
  long FORLIM;

  ran = 0.23456789;
  ran = fabs(ran + atomlage->atomlagekoord[0]) -
	(long)fabs(ran + atomlage->atomlagekoord[0]);
  atomlage->atomlagekoord[0] += (2 * ran - 1) / 1000;
  ran = fabs(ran + atomlage->atomlagekoord[1]) -
	(long)fabs(ran + atomlage->atomlagekoord[1]);
  atomlage->atomlagekoord[1] += (2 * ran - 1) / 1000;
  ran = fabs(ran + atomlage->atomlagekoord[2]) -
	(long)fabs(ran + atomlage->atomlagekoord[2]);
  atomlage->atomlagekoord[2] += (2 * ran - 1) / 1000;
  mult_symop_mit_vektor(*atomlage->symmetrielage, atomlage->atomlagekoord,
			atomlage->atomlagekoord);
  FORLIM = atomlage->anzatome;
  for (i = 0; i < FORLIM; i++)
    mult_symop_mit_vektor(atomlage->atom[i]->trilinpol,
			  atomlage->atomlagekoord, atomlage->atom[i]->pkoord);
}


Static double volumen(a, b, c, aa, bb, cc)
double a, b, c, aa, bb, cc;
{
  double acos, bcos, ccos, v;

  acos = cos(aa * 0.0174533);
  if (fabs(acos) < 0.00000001)
    acos = 0.0;
  bcos = cos(bb * 0.0174533);
  if (fabs(bcos) < 0.00000001)
    bcos = 0.0;
  ccos = cos(cc * 0.0174533);
  if (fabs(ccos) < 0.00000001)
    ccos = 0.0;
  ccos = cos(cc * 0.0174533);
  if (fabs(ccos) < 0.00000001)
    ccos = 0.0;
  v = 1 - acos * acos - bcos * bcos - ccos * ccos + 2 * acos * bcos * ccos;
  if (fabs(v) < 0.00000001)
    v = 0.0;
  return (a * b * c * sqrt(v));
}


Static Void berechne_distanzen_algebraisch(rginfo)
st_raumgruppe rginfo;
{
  long i;
  symmetrieoperator s;
  _TEXT TEMP;

  printf("  Atom - Atom      Distance\n");
  for (i = 2; i <= rginfo.zaehligkeit; i++) {
    subtrahiere_symmetrieoperatoren(*rginfo.symop[0], *rginfo.symop[i - 1],
				    &s);
    printf("   1     %2ld     ", i);
    TEMP.f = stdout;
    *TEMP.name = '\0';
    write_symmetrieoperator(&TEMP, s);
    putchar('\n');
  }
}


Static Void berechne_distanzen(zatom, strukturdaten, max_anzahl, max_laenge)
kt_atom zatom;
kt_strukturdaten strukturdaten;
long max_anzahl;
double max_laenge;
{
  long ii, i, j, k, m, n, o, me, ne, oe;
  vektor vv;
  boolean b;
  distanzen_tabellen_eintrag *hh, *neue_distanz, *neue_dist_tab;
  double vol, TEMP;
  kt_atomlage *WITH;
  long FORLIM1;
  kt_atom *WITH1;

  vol = volumen(strukturdaten.zellpar.laenge[0],
      strukturdaten.zellpar.laenge[1], strukturdaten.zellpar.laenge[2],
      strukturdaten.zellpar.winkel[0], strukturdaten.zellpar.winkel[1],
      strukturdaten.zellpar.winkel[2]);
  TEMP = cos(strukturdaten.zellpar.winkel[0] * 0.0174533);
  me = (long)(max_laenge * strukturdaten.zellpar.laenge[1] *
	 strukturdaten.zellpar.laenge[2] * sqrt(1 - TEMP * TEMP) / vol) + 1;
  TEMP = cos(strukturdaten.zellpar.winkel[1] * 0.0174533);
  ne = (long)(max_laenge * strukturdaten.zellpar.laenge[0] *
	 strukturdaten.zellpar.laenge[2] * sqrt(1 - TEMP * TEMP) / vol) + 1;
  TEMP = cos(strukturdaten.zellpar.winkel[2] * 0.0174533);
  oe = (long)(max_laenge * strukturdaten.zellpar.laenge[0] *
	 strukturdaten.zellpar.laenge[1] * sqrt(1 - TEMP * TEMP) / vol) + 1;
  neue_dist_tab = NULL;
  neue_distanz = (distanzen_tabellen_eintrag *)
		 Malloc(sizeof(distanzen_tabellen_eintrag));
  max_laenge *= max_laenge;
  ii = 0;
  for (j = 0; j < strukturdaten.anzatomlagen; j++) {
    WITH = strukturdaten.atomlage[j];
    FORLIM1 = WITH->anzatome;
    for (k = 0; k < FORLIM1; k++) {
      for (m = -me; m <= me; m++) {
	for (n = -ne; n <= ne; n++) {
	  for (o = -oe; o <= oe; o++) {
	    vv[0] = WITH->atom[k]->pkoord[0] + m - zatom.dessen_atomart->pkoord[0];
	    vv[1] = WITH->atom[k]->pkoord[1] + n - zatom.dessen_atomart->pkoord[1];
	    vv[2] = WITH->atom[k]->pkoord[2] + o - zatom.dessen_atomart->pkoord[2];
	    neue_distanz->laenge = skalar_produkt(vv, vv);
	    WITH1 = &neue_distanz->endatom;
	    WITH1->elementarzelle[0] = m;
	    WITH1->elementarzelle[1] = n;
	    WITH1->elementarzelle[2] = o;
	    WITH1->dessen_atomart = WITH->atom[k];
	    if (!gleiche_atome(zatom, neue_distanz->endatom) &&
		neue_distanz->laenge <= max_laenge) {
	      b = false;
	      if (neue_dist_tab != NULL) {
		if (neue_dist_tab->laenge < neue_distanz->laenge) {
		  hh = neue_dist_tab;
		  do {
		    if (hh->naechste_distanz == NULL)
		      b = true;
		    else if (hh->naechste_distanz->laenge > neue_distanz->laenge)
		      b = true;
		    else
		      hh = hh->naechste_distanz;
		  } while (!b);
		  neue_distanz->naechste_distanz = hh->naechste_distanz;
		  hh->naechste_distanz = neue_distanz;
		}
	      }
	      if (b == false) {
		neue_distanz->naechste_distanz = neue_dist_tab;
		neue_dist_tab = neue_distanz;
	      }
	      ii++;
	      if (ii > max_anzahl) {
		hh = neue_dist_tab;
		for (i = 2; i <= max_anzahl; i++)
		  hh = hh->naechste_distanz;
		neue_distanz = hh->naechste_distanz;
		hh->naechste_distanz = NULL;
	      } else
		neue_distanz = (distanzen_tabellen_eintrag *)
			       Malloc(sizeof(distanzen_tabellen_eintrag));
	    }
	  }
	}
      }
    }
  }
  hh = neue_dist_tab;
  while (hh != NULL) {
    hh->laenge = sqrt(hh->laenge);
    hh = hh->naechste_distanz;
  }
  zatom.dessen_atomart->deren_atomlage->dist_entry = neue_dist_tab;
}


Static Void write_distanzen(t, a)
_TEXT *t;
kt_atom a;
{
  distanzen_tabellen_eintrag *hh;
  long i, anzs;
  st_symop_array symop;
  kt_atom aa;
  distanzen_tabellen_eintrag *WITH;

  fprintf(t->f, "\n Distances from atom ");
  write_atom_name(t, a);
  fprintf(t->f, "  to\n");
  welchsymop(a.dessen_atomart->deren_atomlage->grundatom, a, rginfo, &anzs,
	     symop);
  hh = a.dessen_atomart->deren_atomlage->dist_entry;
  i = 0;
  while (hh != NULL) {
    WITH = hh;
    i++;
    fprintf(t->f, " (%ld)   -", i);
    if (anzs > 0)
      alg_symmetrieoperation(symop[0], WITH->endatom, &aa);
    else
      aa = WITH->endatom;
    write_atom_name(t, aa);
    fprintf(t->f, "%11.4f\n", WITH->laenge);
    hh = WITH->naechste_distanz;
  }
}


Static Void berechne_nte_distanz(a, n, d, b)
kt_atom a;
long n;
double *d;
kt_atom *b;
{
  distanzen_tabellen_eintrag *hh;
  long i;

  b->dessen_atomart = NULL;
  *d = 0.0;
  hh = a.dessen_atomart->deren_atomlage->dist_entry;
  i = 0;
  while (hh != NULL && i < n) {
    i++;
    *b = hh->endatom;
    *d = hh->laenge;
    hh = hh->naechste_distanz;
  }
  if (i != n)
    b->dessen_atomart = NULL;
  if (b->dessen_atomart == NULL)
    printf(" ERROR  distance no. %ld has not been calculated\n", n);
}


Static double distanz_zwischen_atome(a, b)
kt_atom a, b;
{
  vektor v, va;
  long i;

  berechne_atom_koordinaten(a, va);
  berechne_atom_koordinaten(b, v);
  for (i = 0; i <= 2; i++)
    v[i] -= va[i];
  return sqrt(skalar_produkt(v, v));
}


Static double winkel_zwischen_atome(a, b, c)
kt_atom a, b, c;
{
  vektor va, vb, vc;
  long i;
  double cn, cz, cosw, sin2w;

  berechne_atom_koordinaten(a, va);
  berechne_atom_koordinaten(b, vb);
  berechne_atom_koordinaten(c, vc);
  for (i = 0; i <= 2; i++)
    va[i] -= vb[i];
  for (i = 0; i <= 2; i++)
    vc[i] -= vb[i];
  cn = skalar_produkt(va, vc);
  cz = sqrt(skalar_produkt(va, va) * skalar_produkt(vc, vc));
  if (cz == 0)
    cosw = 1.0;
  else
    cosw = cn / cz;
  if (cosw > 1.0)
    cosw = 1.0;
  if (fabs(cosw) < 0.0000000001)
    return 90.0;
  else {
    sin2w = 1 - cosw * cosw;
    if (sin2w < 0)
      sin2w = 0.0;
    if (cosw > 0)
      return (atan(fabs(sqrt(sin2w) / cosw)) * 57.29577951);
    else
      return (180 - atan(fabs(sqrt(sin2w) / cosw)) * 57.29577951);
  }
}


Static Void write_distanzen_und_winkel(t, a)
_TEXT *t;
kt_atom a;
{
  distanzen_tabellen_eintrag *hh1, *hh2;
  long i;
  double w;

  fprintf(t->f, " Distances and angles for atom ");
  write_atom_name(t, a);
  fprintf(t->f,
    "\n                                       (1)      (2)      (3)      (4)      (5)\n");
  i = 0;
  hh1 = a.dessen_atomart->deren_atomlage->dist_entry;
  while (hh1 != NULL && i < 6) {
    i++;
    fprintf(t->f, " (%ld)   -", i);
    write_atom_name(t, hh1->endatom);
    fprintf(t->f, "%8.4f", hh1->laenge);
    hh2 = a.dessen_atomart->deren_atomlage->dist_entry;
    while (hh2 != hh1) {
      w = winkel_zwischen_atome(hh1->endatom, a, hh2->endatom);
      fprintf(t->f, "%9.3f", w);
      hh2 = hh2->naechste_distanz;
    }
    putc('\n', t->f);
    hh1 = hh1->naechste_distanz;
  }
  putc('\n', t->f);
}


Static Void write_distanzen_und_vektoren(t, a)
_TEXT *t;
kt_atom a;
{
  distanzen_tabellen_eintrag *hh;
  long i, j;
  vektor v, va;

  fprintf(t->f, " Distances and interatomic vectors for atom ");
  write_atom_name(t, a);
  putc('\n', t->f);
  i = 0;
  hh = a.dessen_atomart->deren_atomlage->dist_entry;
  while (hh != NULL) {
    i++;
    fprintf(t->f, " (%ld)   -", i);
    write_atom_name(t, hh->endatom);
    fprintf(t->f, "%8.4f", hh->laenge);
    berechne_atom_koordinaten(a, va);
    berechne_atom_koordinaten(hh->endatom, v);
    for (j = 0; j <= 2; j++)
      v[j] -= va[j];
    fprintf(t->f, "     ");
    write_vektor(t, v);
    putc('\n', t->f);
    hh = hh->naechste_distanz;
  }
  putc('\n', t->f);
}


Static Void berechne_nte_bindung(x, n, y)
kt_atom *x;
long n;
kt_atom *y;
{
  *y = x->dessen_atomart->bindung[n - 1];
  y->elementarzelle[0] += x->elementarzelle[0];
  y->elementarzelle[1] += x->elementarzelle[1];
  y->elementarzelle[2] += x->elementarzelle[2];
}


Static long anzahl_bindungen_des_atoms(a)
kt_atom a;
{
  long j;
  kt_atomart *WITH;

  WITH = a.dessen_atomart;
  if (WITH->bindung[maxanzahlbindungen - 1].dessen_atomart != NULL) {
    j = maxanzahlbindungen;
    return j;
  }
  j = 0;
  do {
    j++;
  } while (WITH->bindung[j - 1].dessen_atomart != NULL);
  j--;
  return j;
}


Static boolean atome_miteinander_verbunden(a1, a2)
kt_atom a1, a2;
{
  long i;
  boolean b;
  kt_atom a3;
  long FORLIM;

  b = false;
  FORLIM = anzahl_bindungen_des_atoms(a1);
  for (i = 1; i <= FORLIM; i++) {
    berechne_nte_bindung(&a1, i, &a3);
    if (gleiche_atome(a2, a3))
      b = true;
  }
  return b;
}


Static Void erzeuge_bindung(a1, a2)
kt_atom a1, a2;
{
  long i, j, j_a1, j_a2, k;
  kt_atom b1, b2;
  boolean zuwenig_bindungen1, zuwenig_bindungen2;
  _TEXT TEMP;
  long FORLIM;
  kt_atomart *WITH;
  kt_atomlage *WITH1;

  if (atome_miteinander_verbunden(a1, a2)) {
    printf(" WARNING  there is already a bond ");
    TEMP.f = stdout;
    *TEMP.name = '\0';
    write_atom_name(&TEMP, a1);
    printf("  -- ");
    TEMP.f = stdout;
    *TEMP.name = '\0';
    write_atom_name(&TEMP, a2);
    putchar('\n');
    return;
  }
  zuwenig_bindungen1 = false;
  zuwenig_bindungen2 = false;
  j_a1 = anzahl_bindungen_des_atoms(a1);
  j_a2 = anzahl_bindungen_des_atoms(a2);
  FORLIM = rginfo.zaehligkeit;
  for (i = 0; i < FORLIM; i++) {
    alg_symmetrieoperation(*rginfo.symop[i], a1, &b1);
    alg_symmetrieoperation(*rginfo.symop[i], a2, &b2);
    for (k = 0; k <= 2; k++)
      b2.elementarzelle[k] -= b1.elementarzelle[k];
    j = j_a1;
    WITH = b1.dessen_atomart;
    do {
      j++;
      if (WITH->bindung[j - 1].dessen_atomart == NULL)
	WITH->bindung[j - 1] = b2;
    } while (!(gleiche_atome(WITH->bindung[j - 1], b2) ||
	       j == maxanzahlbindungen));
    if (!gleiche_atome(WITH->bindung[j - 1], b2))
      zuwenig_bindungen1 = true;
    for (k = 0; k <= 2; k++)
      b1.elementarzelle[k] = -b2.elementarzelle[k];
    j = j_a2;
    WITH = b2.dessen_atomart;
    do {
      j++;
      if (WITH->bindung[j - 1].dessen_atomart == NULL)
	WITH->bindung[j - 1] = b1;
    } while (!(gleiche_atome(WITH->bindung[j - 1], b1) ||
	       j == maxanzahlbindungen));
    if (!gleiche_atome(WITH->bindung[j - 1], b1))
      zuwenig_bindungen2 = true;
  }
  if (zuwenig_bindungen1) {
    printf(" ERROR  too many bonds for atom ");
    TEMP.f = stdout;
    *TEMP.name = '\0';
    write_atom_name(&TEMP, a1);
    printf("\n        The program is compiled for a maximum of %ld bonds.\n",
	   (long)maxanzahlbindungen);
    printf("        Increase constant \"maxanzahlbindungen\" and compile again.\n");
  }
  if (zuwenig_bindungen2) {
    printf(" ERROR  too many bonds for ");
    TEMP.f = stdout;
    *TEMP.name = '\0';
    write_atom_name(&TEMP, a2);
    printf("\n        The program is compiled for a maximum of %ld bonds.\n",
	   (long)maxanzahlbindungen);
    printf("        Increase constant \"maxanzahlbindungen\" and compile again.\n");
  }
  if (zuwenig_bindungen1 || zuwenig_bindungen2) {
    printf(" ERROR  bond ");
    TEMP.f = stdout;
    *TEMP.name = '\0';
    write_atom_name(&TEMP, a1);
    printf("  -- ");
    TEMP.f = stdout;
    *TEMP.name = '\0';
    write_atom_name(&TEMP, a2);
    printf(" not created \n");
    WITH1 = a1.dessen_atomart->deren_atomlage;
    FORLIM = WITH1->anzatome;
    for (j = 0; j < FORLIM; j++) {
      for (k = j_a1; k < maxanzahlbindungen; k++)
	WITH1->atom[j]->bindung[k].dessen_atomart = NULL;
    }
    WITH1 = a2.dessen_atomart->deren_atomlage;
    FORLIM = WITH1->anzatome;
    for (j = 0; j < FORLIM; j++) {
      for (k = j_a2; k < maxanzahlbindungen; k++)
	WITH1->atom[j]->bindung[k].dessen_atomart = NULL;
    }
  } else {
    printf(" ... new bond ");
    TEMP.f = stdout;
    *TEMP.name = '\0';
    write_atom_name(&TEMP, a1);
    printf("  -- ");
    TEMP.f = stdout;
    *TEMP.name = '\0';
    write_atom_name(&TEMP, a2);
    printf(" created \n");
  }
  if (anzahl_bindungen_des_atoms(a1) > koordinationszahl_des_atoms(a1))
    printf(" WARNING  too many bonds for %.6s\n",
	   a1.dessen_atomart->deren_atomlage->aname);
  if (anzahl_bindungen_des_atoms(a2) > koordinationszahl_des_atoms(a2)) {
    if (a1.dessen_atomart->deren_atomlage != a2.dessen_atomart->deren_atomlage)
      printf(" WARNING  too many bonds for %.6s\n",
	     a2.dessen_atomart->deren_atomlage->aname);
  }
}


Static Void loesche_bindung(a1, a2)
kt_atom a1, a2;
{
  kt_atom aa;
  boolean b1[maxanzahlbindungen], b2[maxanzahlbindungen];
  long i, j, k, FORLIM;
  kt_atomlage *WITH;
  long FORLIM1;
  kt_atomart *WITH1;
  _TEXT TEMP;

  if (atome_miteinander_verbunden(a1, a2)) {
    for (i = 0; i < maxanzahlbindungen; i++) {
      berechne_nte_bindung(&a1, i + 1, &aa);
      b1[i] = false;
      if (aa.dessen_atomart != NULL) {
	if ((!sym_aequival_distanzen(a1, a2, a1, aa)) &
	    (!sym_aequival_distanzen(a2, a1, a1, aa)))
	  b1[i] = true;
      }
      berechne_nte_bindung(&a2, i + 1, &aa);
      b2[i] = false;
      if (aa.dessen_atomart != NULL) {
	if ((!sym_aequival_distanzen(a1, a2, a2, aa)) &
	    (!sym_aequival_distanzen(a2, a1, a2, aa)))
	  b2[i] = true;
      }
    }
    k = 0;
    WITH = a1.dessen_atomart->deren_atomlage;
    for (i = 0; i < maxanzahlbindungen; i++) {
      if (b1[i]) {
	k++;
	FORLIM1 = WITH->anzatome;
	for (j = 0; j < FORLIM1; j++) {
	  WITH1 = WITH->atom[j];
	  WITH1->bindung[k - 1] = WITH1->bindung[i];
	}
      }
    }
    for (i = k; i < maxanzahlbindungen; i++) {
      FORLIM = WITH->anzatome;
      for (j = 0; j < FORLIM; j++) {
	WITH1 = WITH->atom[j];
	WITH1->bindung[i].dessen_atomart = NULL;
      }
    }
    if (a1.dessen_atomart->deren_atomlage != a2.dessen_atomart->deren_atomlage) {
      k = 0;
      WITH = a2.dessen_atomart->deren_atomlage;
      for (i = 0; i < maxanzahlbindungen; i++) {
	if (b2[i]) {
	  k++;
	  FORLIM = WITH->anzatome;
	  for (j = 0; j < FORLIM; j++) {
	    WITH1 = WITH->atom[j];
	    WITH1->bindung[k - 1] = WITH1->bindung[i];
	  }
	}
      }
      for (i = k; i < maxanzahlbindungen; i++) {
	FORLIM = WITH->anzatome;
	for (j = 0; j < FORLIM; j++) {
	  WITH1 = WITH->atom[j];
	  WITH1->bindung[i].dessen_atomart = NULL;
	}
      }
    }
    printf(" ... old bond ");
    TEMP.f = stdout;
    *TEMP.name = '\0';
    write_atom_name(&TEMP, a1);
    printf("  -- ");
    TEMP.f = stdout;
    *TEMP.name = '\0';
    write_atom_name(&TEMP, a2);
    printf(" broken \n");
    return;
  }
  printf(" WARNING  there is no bond ");
  TEMP.f = stdout;
  *TEMP.name = '\0';
  write_atom_name(&TEMP, a1);
  printf("  -- ");
  TEMP.f = stdout;
  *TEMP.name = '\0';
  write_atom_name(&TEMP, a2);
  putchar('\n');
}


Static Void berechne_alle_bindungen(struda)
kt_strukturdaten *struda;
{
  long i, j;
  distanzen_tabellen_eintrag *hh;
  long FORLIM;
  kt_atomlage *WITH1;
  long FORLIM1;
  _TEXT TEMP;

  FORLIM = struda->anzatomlagen;
  for (j = 0; j < FORLIM; j++) {
    WITH1 = struda->atomlage[j];
    hh = WITH1->dist_entry;
    FORLIM1 = WITH1->koordinationszahl;
    for (i = 1; i <= FORLIM1; i++) {
      if (hh == NULL) {
	printf(" ERROR  not enough distances for atom ");
	TEMP.f = stdout;
	*TEMP.name = '\0';
	write_atom_name(&TEMP, WITH1->grundatom);
	putchar('\n');
      } else {
	if (!atome_miteinander_verbunden(WITH1->grundatom, hh->endatom))
	  erzeuge_bindung(WITH1->grundatom, hh->endatom);
	hh = hh->naechste_distanz;
      }
    }
  }
}


Static Void berechne_mitte(a, b, v)
kt_atom a, b;
double *v;
{
  long i;

  for (i = 0; i <= 2; i++)
    v[i] = (a.dessen_atomart->pkoord[i] + a.elementarzelle[i] +
	    b.dessen_atomart->pkoord[i] + b.elementarzelle[i]) / 2;
}


Static Void baue_o_atome_ein(strukturdaten, status)
kt_strukturdaten *strukturdaten;
boolean *status;
{
  long i, j, k, kk;
  kt_atom b1, b2, oa;
  vektor v;
  gittervektor g;
  kt_atomlage *neue_atomlage;
  boolean b, bb;
  long FORLIM;
  kt_atomlage *WITH1;
  long FORLIM1;

  k = 0;
  FORLIM = strukturdaten->anzatomlagen;
  for (i = 0; i < FORLIM; i++) {
    WITH1 = strukturdaten->atomlage[i];
    if (!strncmp(WITH1->element, "O ", sizeof(kt_element))) {
      kk = WITH1->aname[1] - '0';
      if (WITH1->aname[2] != ' ')
	kk = kk * 10 + WITH1->aname[2] - '0';
      if (kk > k)
	k = kk;
    }
  }
  *status = true;
  FORLIM = strukturdaten->anzatomlagen;
  for (i = 0; i < FORLIM; i++) {
    b1 = strukturdaten->atomlage[i]->grundatom;
    if (!atom_hat_gleiches_element(b1, "O ")) {
      do {
	bb = true;
	FORLIM1 = anzahl_bindungen_des_atoms(b1);
	for (j = 1; j <= FORLIM1; j++) {
	  berechne_nte_bindung(&b1, j, &b2);
	  if (b2.dessen_atomart != NULL) {
	    if (!atom_hat_gleiches_element(b2, "O ")) {
	      bb = false;
	      berechne_mitte(b1, b2, v);
	      int_vektor(v, g);
	      frac_vektor(v, v);
	      k++;
	      erzeuge_atomlage("O ", k, v, 2L, &neue_atomlage);
	      lade_atomlage(&neue_atomlage, strukturdaten, &b);
	      loesche_bindung(b1, b2);
	      if (b) {
		verschiebe_atome_leicht(neue_atomlage);
		oa.dessen_atomart =
		  strukturdaten->atomlage[strukturdaten->anzatomlagen - 1]->
		  atom[0];
		memcpy(oa.elementarzelle, g, sizeof(gittervektor));
		erzeuge_bindung(b1, oa);
		if (!atome_miteinander_verbunden(b2, oa))
		  erzeuge_bindung(b2, oa);
	      }
	    }
	  }
	}
      } while (!bb);
    }
  }
  FORLIM = strukturdaten->anzatomlagen;
  for (i = 0; i < FORLIM; i++) {
    WITH1 = strukturdaten->atomlage[i];
    if (!strncmp(WITH1->element, "O ", sizeof(kt_element)) &&
	WITH1->koordinationszahl > 2) {
      printf(" ERROR  there are too many bonds for %.6s\n", WITH1->aname);
      *status = false;
    }
  }
}


Static Void write_bindungen(t, atom)
_TEXT *t;
kt_atom atom;
{
  long j, k;
  kt_atom bind[maxanzahlbindungen];
  boolean b;

  fprintf(t->f, "\n Bonds from atom ");
  write_atom_name(t, atom);
  putc('\n', t->f);
  for (k = 1; k <= maxanzahlbindungen; k++) {
    berechne_nte_bindung(&atom, k, &bind[k - 1]);
    if (bind[k - 1].dessen_atomart != NULL) {
      fprintf(t->f, " (%ld)   -", k);
      write_atom_name(t, bind[k - 1]);
      fprintf(t->f, "%10.4f", distanz_zwischen_atome(atom, bind[k - 1]));
      b = true;
      for (j = 1; j < k; j++) {
	if ((sym_aequival_distanzen(atom, bind[k - 1], atom, bind[j - 1]) |
	     sym_aequival_distanzen(atom, bind[k - 1], bind[j - 1], atom)) &&
	    b) {
	  b = false;
	  fprintf(t->f, "    symm. equival. to bond (%ld)", j);
	}
      }
      putc('\n', t->f);
    }
  }
}


Static Void write_bindungs_matrix(t, atomlage)
_TEXT *t;
kt_atomlage atomlage;
{
  long j, k;
  kt_atomart *WITH;

  fprintf(t->f, " Atom %.6s\n", atomlage.aname);
  for (j = 1; j <= atomlage.anzatome; j++) {
    WITH = atomlage.atom[j - 1];
    fprintf(t->f, "%4ld:", j);
    for (k = 0; k < maxanzahlbindungen; k++) {
      if (WITH->bindung[k].dessen_atomart != NULL)
	write_atom_name(t, WITH->bindung[k]);
    }
    putc('\n', t->f);
  }
  putc('\n', t->f);
}


Static Void verknuepfe_t_atome_direkt()
{
  long i, di, j;
  double d, d2;
  kt_atom b1, b2;
  long FORLIM;
  kt_atomlage *WITH;
  long FORLIM1;

  FORLIM = strukturdaten.anzatomlagen;
  for (i = 0; i < FORLIM; i++) {
    WITH = strukturdaten.atomlage[i];
    if (!strncmp(WITH->element, "O ", sizeof(kt_element))) {
      while (WITH->koordinationszahl > 2) {
	printf(
	  " WARNING  for atom position %.6s the coordination number %ld is greater then 2\n",
	  WITH->aname, WITH->koordinationszahl);
	d = distanz_zwischen_atome(WITH->grundatom, WITH->atom[0]->bindung[0]);
	di = 1;
	FORLIM1 = WITH->koordinationszahl;
	for (j = 2; j <= FORLIM1; j++) {
	  d2 = distanz_zwischen_atome(WITH->grundatom,
				      WITH->atom[0]->bindung[j - 1]);
	  if (d2 > d) {
	    d = d2;
	    di = j;
	  }
	}
	loesche_bindung(WITH->grundatom, WITH->atom[0]->bindung[di - 1]);
      }
    }
  }
  for (i = strukturdaten.anzatomlagen - 1; i >= 0; i--) {
    WITH = strukturdaten.atomlage[i];
    if (!strncmp(WITH->element, "O ", sizeof(kt_element))) {
      if (anzahl_bindungen_des_atoms(WITH->grundatom) > 0) {
	berechne_nte_bindung(&WITH->grundatom, 1L, &b1);
	if (anzahl_bindungen_des_atoms(WITH->grundatom) == 2) {
	  berechne_nte_bindung(&WITH->grundatom, 2L, &b2);
	  loesche_bindung(WITH->grundatom, b2);
	  if (atome_miteinander_verbunden(WITH->grundatom, b1))
	    loesche_bindung(WITH->grundatom, b1);
	  erzeuge_bindung(b1, b2);
	}
	if (atome_miteinander_verbunden(WITH->grundatom, b1))
	  loesche_bindung(WITH->grundatom, b1);
	delete_atomlage(strukturdaten.atomlage[i], &strukturdaten);
      }
    }
  }
}


Static Void berechne_koord_seq(atoco, anz, koord_seq)
kt_atom atoco;
long anz;
long *koord_seq;
{
  long i, j, k, l, n;
  kt_atom sphae[2000];
  long z[12];
  long FORLIM, FORLIM1, FORLIM2;

  z[0] = 0;
  z[1] = 1;
  z[2] = koordinationszahl_des_atoms(atoco) + 1;
  sphae[0] = atoco;
  FORLIM = koordinationszahl_des_atoms(atoco);
  for (i = 1; i <= FORLIM; i++)
    berechne_nte_bindung(&atoco, i, &sphae[i]);
  n = z[2] + 1;
  for (i = 3; i <= anz; i++) {
    FORLIM1 = z[i - 1];
    for (j = z[i - 2]; j < FORLIM1; j++) {
      FORLIM2 = koordinationszahl_des_atoms(sphae[j]);
      for (k = 1; k <= FORLIM2; k++) {
	berechne_nte_bindung(&sphae[j], k, &sphae[n - 1]);
	l = z[i - 3];
	do {
	  l++;
	} while (!gleiche_atome(sphae[l - 1], sphae[n - 1]));
	if (l == n)
	  n++;
      }
    }
    z[i] = n - 1;
  }
  for (i = 1; i <= 10; i++)
    koord_seq[i - 1] = z[i + 1] - z[i];
}


Static Void write_koord_seq(f, z)
_TEXT *f;
long *z;
{
  long i;

  for (i = 0; i <= 9; i++)
    fprintf(f->f, "%4ld", z[i]);
  putchar('\n');
}


Static Void bestimme_topologie()
{
  long i, j, k;
  tt_koord_sequenz z[maxanzahlatomlagen];
  tt_koord_sequenz ze[20];
  long anzze;
  Char zecod[6], zecodneu[6];
  Char zcod[maxanzahlatomlagen][10][6];
  boolean bz[100], bze[100];
  long nzcod;
  boolean b;
  long FORLIM;
  kt_atomlage *WITH;
  _TEXT TEMP;

  verknuepfe_t_atome_direkt();
  printf("\n Coordination sequences:\n");
  FORLIM = strukturdaten.anzatomlagen;
  for (i = 0; i < FORLIM; i++) {
    WITH = strukturdaten.atomlage[i];
    if (WITH->koordinationszahl > 0) {
      berechne_koord_seq(WITH->grundatom, 11L, z[i]);
      printf(" %.6s", WITH->aname);
      TEMP.f = stdout;
      *TEMP.name = '\0';
      write_koord_seq(&TEMP, z[i]);
    }
  }
  printf(" Possible frameworks: \n");
  FORLIM = strukturdaten.anzatomlagen;
  for (i = 0; i < FORLIM; i++) {
    WITH = strukturdaten.atomlage[i];
    if (WITH->koordinationszahl > 0) {
      for (j = 0; j <= 9; j++) {
	for (k = 0; k <= 5; k++)
	  zcod[i][j][k] = ' ';
      }
    }
  }
  nzcod = 0;
  if (*coseq.name != '\0') {
    if (coseq.f != NULL)
      coseq.f = freopen(coseq.name, "r", coseq.f);
    else
      coseq.f = fopen(coseq.name, "r");
  } else
    rewind(coseq.f);
  if (coseq.f == NULL)
    _EscIO(FileNotFound);
  RESETBUF(coseq.f, Char);
  for (i = 0; i <= 5; i++) {
    zecodneu[i] = getc(coseq.f);
    if (zecodneu[i] == '\n')
      zecodneu[i] = ' ';
  }
  while (!BUFEOF(coseq.f)) {
    anzze = 0;
    do {
      memcpy(zecod, zecodneu, 6L);
      anzze++;
      for (i = 0; i <= 9; i++)
	fscanf(coseq.f, "%ld", &ze[anzze - 1][i]);
      fscanf(coseq.f, "%*[^\n]");
      getc(coseq.f);
      if (!BUFEOF(coseq.f)) {
	for (i = 0; i <= 5; i++) {
	  zecodneu[i] = getc(coseq.f);
	  if (zecodneu[i] == '\n')
	    zecodneu[i] = ' ';
	}
      } else
	memcpy(zecodneu, "******", 6L);
    } while (!strncmp(zecod, zecodneu, 6));
    FORLIM = strukturdaten.anzatomlagen;
    for (i = 0; i < FORLIM; i++)
      bz[i] = false;
    for (j = 0; j < anzze; j++)
      bze[j] = false;
    FORLIM = strukturdaten.anzatomlagen;
    for (i = 0; i < FORLIM; i++) {
      WITH = strukturdaten.atomlage[i];
      if (WITH->koordinationszahl > 0) {
	for (j = 0; j < anzze; j++) {
	  b = true;
	  for (k = 0; k <= 9; k++) {
	    if (ze[j][k] != z[i][k])
	      b = false;
	  }
	  if (b) {
	    bz[i] = true;
	    bze[j] = true;
	  }
	}
      }
    }
    b = true;
    for (j = 0; j < anzze; j++) {
      if (!bze[j])
	b = false;
    }
    if (!b)
      continue;
    nzcod++;
    FORLIM = strukturdaten.anzatomlagen;
    for (i = 0; i < FORLIM; i++) {
      WITH = strukturdaten.atomlage[i];
      if (WITH->koordinationszahl > 0) {
	if (bz[i])
	  memcpy(zcod[i][nzcod - 1], zecod, 6L);
      }
    }
  }
  FORLIM = strukturdaten.anzatomlagen;
  for (i = 0; i < FORLIM; i++) {
    WITH = strukturdaten.atomlage[i];
    if (WITH->koordinationszahl > 0) {
      printf(" %.6s", WITH->aname);
      for (j = 0; j < nzcod; j++)
	printf("%.6s", zcod[i][j]);
      putchar('\n');
    }
  }
}


typedef struct wegtyp {
  kt_atom atom[20];
  long anzatome;
} wegtyp;


/* Local variables for berechne_loop_konfig: */
struct LOC_berechne_loop_konfig {
  long anzloesungen;
  wegtyp loesung[50];
} ;

Local Void suche_weg(ort, ziel, schritte, weg, LINK)
kt_atom ort, ziel;
long schritte;
wegtyp weg;
struct LOC_berechne_loop_konfig *LINK;
{
  long i, j;
  kt_atom neuer_ort;
  boolean b;
  long FORLIM;

  if (schritte > 0) {
    FORLIM = anzahl_bindungen_des_atoms(ort);
    for (i = 1; i <= FORLIM; i++) {
      berechne_nte_bindung(&ort, i, &neuer_ort);
      weg.atom[schritte - 1] = neuer_ort;
      b = true;
      for (j = weg.anzatome - 1; j >= schritte; j--) {
	if (gleiche_atome(weg.atom[j], neuer_ort))
	  b = false;
      }
      if (b)
	suche_weg(neuer_ort, ziel, schritte - 1, weg, LINK);
    }
    return;
  }
  if (gleiche_atome(ort, ziel)) {
    LINK->anzloesungen++;
    LINK->loesung[LINK->anzloesungen - 1] = weg;
  }
}


Static Void berechne_loop_konfig(zentralatom)
kt_atom zentralatom;
{
  struct LOC_berechne_loop_konfig V;
  long i, j, k, l;
  wegtyp weg;
  kt_atom nachbaratom[maxanzahlbindungen];
  boolean b;
  long FORLIM;
  _TEXT TEMP;
  long FORLIM2;
  wegtyp *WITH;
  long FORLIM3;

  b = true;
  FORLIM = anzahl_bindungen_des_atoms(zentralatom);
  for (i = 0; i < FORLIM; i++) {
    berechne_nte_bindung(&zentralatom, i + 1, &nachbaratom[i]);
    if (gleiche_atome(zentralatom, nachbaratom[i])) {
      printf(" ERROR  bond ");
      TEMP.f = stdout;
      *TEMP.name = '\0';
      write_atom_name(&TEMP, zentralatom);
      printf("  -- ");
      TEMP.f = stdout;
      *TEMP.name = '\0';
      write_atom_name(&TEMP, nachbaratom[i]);
      printf(" not allowed\n");
      b = false;
    }
  }
  if (!b)
    return;
  FORLIM = anzahl_bindungen_des_atoms(zentralatom);
  for (i = 1; i <= FORLIM; i++) {
    for (j = 0; j <= i - 2; j++) {
      weg.anzatome = 1;
      do {
	weg.anzatome++;
	V.anzloesungen = 0;
	weg.atom[weg.anzatome - 1] = zentralatom;
	weg.atom[weg.anzatome - 2] = nachbaratom[i - 1];
	suche_weg(nachbaratom[i - 1], nachbaratom[j], weg.anzatome - 2, weg,
		  &V);
      } while (V.anzloesungen <= 0);
      printf("\n Angle ");
      TEMP.f = stdout;
      *TEMP.name = '\0';
      write_atomart_name(&TEMP, *nachbaratom[i - 1].dessen_atomart);
      printf("  -- ");
      TEMP.f = stdout;
      *TEMP.name = '\0';
      write_atomart_name(&TEMP, *zentralatom.dessen_atomart);
      printf("  -- ");
      TEMP.f = stdout;
      *TEMP.name = '\0';
      write_atomart_name(&TEMP, *nachbaratom[j].dessen_atomart);
      if (V.anzloesungen == 1)
	printf("  has 1 loop  with %ld atoms\n", V.loesung[0].anzatome);
      else
	printf("  has %ld loops with %ld atoms\n",
	       V.anzloesungen, V.loesung[0].anzatome);
      FORLIM2 = V.anzloesungen;
      for (k = 0; k < FORLIM2; k++) {
	WITH = &V.loesung[k];
	printf("  -");
	TEMP.f = stdout;
	*TEMP.name = '\0';
	write_atom_name(&TEMP, WITH->atom[WITH->anzatome - 1]);
	putchar(' ');
	FORLIM3 = WITH->anzatome;
	for (l = 1; l < FORLIM3; l++) {
	  if ((l & 3) == 0 && l > 1)
	    printf("   ");
	  TEMP.f = stdout;
	  *TEMP.name = '\0';
	  write_atom_name(&TEMP, WITH->atom[l - 1]);
	  putchar(' ');
	  if ((l & 3) == 3)
	    putchar('\n');
	}
	if ((WITH->anzatome & 3) != 0)
	  putchar('\n');
      }
    }
  }
}


Static Void loesche_alle_bindungen(struda)
kt_strukturdaten *struda;
{
  long i;
  kt_atom a;
  long FORLIM;
  kt_atomlage *WITH1;

  FORLIM = struda->anzatomlagen;
  for (i = 0; i < FORLIM; i++) {
    WITH1 = struda->atomlage[i];
    while (anzahl_bindungen_des_atoms(WITH1->grundatom) > 0) {
      berechne_nte_bindung(&WITH1->grundatom, 1L, &a);
      loesche_bindung(WITH1->grundatom, a);
    }
  }
}


typedef Char nametyp[6];

typedef struct _REC_symeq {
  kt_atom atona;
  nametyp erstername, zweitername;
} _REC_symeq;

typedef struct dls_symeq_karten {
  _REC_symeq symeq[maxanzahlsymeqkarten][20];
  long anzsymeq[maxanzahlsymeqkarten];
} dls_symeq_karten;

typedef struct _REC_tetcon {
  nametyp n[9];
  kt_atom a[9];
} _REC_tetcon;

typedef struct dls_tetcon_karten {
  _REC_tetcon tetcon[maxanzahlatomlagen];
  long anztetcon;
} dls_tetcon_karten;

typedef struct _REC_distcon {
  kt_atom atom1, atom2;
  nametyp atomname1, atomname2;
} _REC_distcon;

typedef struct dls_distan_karten {
  _REC_distcon distcon[maxindepdistanzen];
  long anzdistcon;
} dls_distan_karten;

typedef struct _REC_anglcon {
  kt_atom atom1, atom2, atomm;
  nametyp atomname1, atomnamem, atomname2;
} _REC_anglcon;

typedef struct dls_angle_karten {
  _REC_anglcon anglcon[maxindepwinkel];
  long anzanglcon;
} dls_angle_karten;

typedef struct _REC_dt_geg_distanz {
  kt_element el1, el2;
  long kz1, kz2;
  double distanz, dvar;
} _REC_dt_geg_distanz;

typedef _REC_dt_geg_distanz dt_geg_distanz[maxanzahlgegdistanzen];

typedef struct _REC_dt_geg_winkel {
  kt_element el1, el2, el3;
  long kz1, kz2, kz3;
  double winkel, wvar;
} _REC_dt_geg_winkel;

typedef _REC_dt_geg_winkel dt_geg_winkel[maxanzahlgegwinkel];


typedef struct _REC_bondis_type {
  kt_element e1, e2, e3;
  kt_atom a1, a2, a3;
} _REC_bondis_type;

typedef _REC_bondis_type bondis_type[20];


/* Local variables for write_input_file: */
struct LOC_write_input_file {
  kt_strukturdaten strukturdaten;
} ;

Local Void read_bindungs_groessen(f, dist, wink, LINK)
_TEXT *f;
_REC_dt_geg_distanz *dist;
_REC_dt_geg_winkel *wink;
struct LOC_write_input_file *LINK;
{
  long i, anz_d, anz_w;
  kt_element e1, e2, e3;
  long k1, k2, k3;
  _REC_dt_geg_distanz *WITH;
  _REC_dt_geg_winkel *WITH1;

  anz_d = 1;
  anz_w = 1;
  if (*f->name != '\0') {
    if (f->f != NULL)
      f->f = freopen(f->name, "r", f->f);
    else
      f->f = fopen(f->name, "r");
  } else
    rewind(f->f);
  if (f->f == NULL)
    _EscIO(FileNotFound);
  RESETBUF(f->f, Char);
  while (!BUFEOF(f->f)) {
    e3[0] = ' ';
    e1[1] = ' ';
    e2[1] = ' ';
    e3[1] = ' ';
    while (P_peek(f->f) == ']' || P_peek(f->f) == '[' || P_peek(f->f) == ' ')
      getc(f->f);
    *e1 = getc(f->f);
    if (e1[0] == '\n')
      e1[0] = ' ';
    if (P_peek(f->f) == ' ' || isalpha(P_peek(f->f))) {
      e1[1] = getc(f->f);
      if (e1[1] == '\n')
	e1[1] = ' ';
    }
    while (P_peek(f->f) == ']' || P_peek(f->f) == '[' || P_peek(f->f) == ' ')
      getc(f->f);
    fscanf(f->f, "%ld", &k1);
    while (P_peek(f->f) == ']' || P_peek(f->f) == '[' || P_peek(f->f) == ' ')
      getc(f->f);
    *e2 = getc(f->f);
    if (e2[0] == '\n')
      e2[0] = ' ';
    if (P_peek(f->f) == ' ' || isalpha(P_peek(f->f))) {
      e2[1] = getc(f->f);
      if (e2[1] == '\n')
	e2[1] = ' ';
    }
    while (P_peek(f->f) == ']' || P_peek(f->f) == '[' || P_peek(f->f) == ' ')
      getc(f->f);
    fscanf(f->f, "%ld", &k2);
    while (P_peek(f->f) == ']' || P_peek(f->f) == '[' || P_peek(f->f) == ' ')
      getc(f->f);
    if (P_peek(f->f) == ' ' || isalpha(P_peek(f->f))) {
      *e3 = getc(f->f);
      if (e3[0] == '\n')
	e3[0] = ' ';
      if (P_peek(f->f) == ' ' || isalpha(P_peek(f->f))) {
	e3[1] = getc(f->f);
	if (e3[1] == '\n')
	  e3[1] = ' ';
      }
      while (P_peek(f->f) == ']' || P_peek(f->f) == '[' || P_peek(f->f) == ' ')
	getc(f->f);
      fscanf(f->f, "%ld", &k3);
      while (P_peek(f->f) == ']' || P_peek(f->f) == '[' || P_peek(f->f) == ' ')
	getc(f->f);
    }
    if (e3[0] == ' ') {
      WITH = &dist[anz_d - 1];
      memcpy(WITH->el1, e1, sizeof(kt_element));
      WITH->kz1 = k1;
      memcpy(WITH->el2, e2, sizeof(kt_element));
      WITH->kz2 = k2;
      fscanf(f->f, "%lg%lg%*[^\n]", &WITH->distanz, &WITH->dvar);
      getc(f->f);
      anz_d++;
      continue;
    }
    WITH1 = &wink[anz_w - 1];
    memcpy(WITH1->el1, e1, sizeof(kt_element));
    WITH1->kz1 = k1;
    memcpy(WITH1->el2, e2, sizeof(kt_element));
    WITH1->kz2 = k2;
    memcpy(WITH1->el3, e3, sizeof(kt_element));
    WITH1->kz3 = k3;
    fscanf(f->f, "%lg%lg%*[^\n]", &WITH1->winkel, &WITH1->wvar);
    getc(f->f);
    anz_w++;
  }
  for (i = anz_d - 1; i < maxanzahlgegdistanzen; i++) {
    WITH = &dist[i];
    memcpy(WITH->el1, "  ", sizeof(kt_element));
    memcpy(WITH->el2, "  ", sizeof(kt_element));
    WITH->kz1 = 0;
    WITH->kz2 = 0;
    WITH->distanz = 0.0;
    WITH->dvar = 0.0;
  }
  for (i = anz_w - 1; i < maxanzahlgegwinkel; i++) {
    WITH1 = &wink[i];
    memcpy(WITH1->el1, "  ", sizeof(kt_element));
    memcpy(WITH1->el2, "  ", sizeof(kt_element));
    memcpy(WITH1->el3, "  ", sizeof(kt_element));
    WITH1->kz1 = 0;
    WITH1->kz2 = 0;
    WITH1->kz3 = 0;
    WITH1->winkel = 0.0;
    WITH1->wvar = 0.0;
  }
}

Local Void write_dls_atom_karten(f, s, LINK)
_TEXT *f;
kt_strukturdaten s;
struct LOC_write_input_file *LINK;
{
  long i;
  vektor v;
  kt_atomlage *WITH;

  for (i = 0; i < s.anzatomlagen; i++) {
    WITH = s.atomlage[i];
    berechne_atom_koordinaten(WITH->grundatom, v);
    fprintf(f->f, "ATOM   %.6s%8.5f%8.5f%8.5f  %.2s   ",
	    WITH->aname, v[0], v[1], v[2], WITH->element);
    write_atom_trilinpol(f, WITH->grundatom);
    putc('\n', f->f);
  }
}

Local Void write_loadat_atom_karten(f, s, LINK)
_TEXT *f;
kt_strukturdaten s;
struct LOC_write_input_file *LINK;
{
  long i;
  kt_atomlage *WITH;

  for (i = 0; i < s.anzatomlagen; i++) {
    WITH = s.atomlage[i];
    fprintf(f->f, "ATOM   %.6s%8.5f%8.5f%8.5f\n",
	    WITH->aname, WITH->atomlagekoord[0], WITH->atomlagekoord[1],
	    WITH->atomlagekoord[2]);
  }
}

Local Void write_dls_symeq_karten(f, dls_symeq, LINK)
_TEXT *f;
dls_symeq_karten dls_symeq;
struct LOC_write_input_file *LINK;
{
  long i, j, anzsymop;
  st_symop_array symop;
  long FORLIM, FORLIM1;
  _REC_symeq *WITH;

  FORLIM = LINK->strukturdaten.anzatomlagen;
  for (i = 0; i < FORLIM; i++) {
    FORLIM1 = dls_symeq.anzsymeq[i];
    for (j = 0; j < FORLIM1; j++) {
      WITH = &dls_symeq.symeq[i][j];
      fprintf(f->f, "SYMEQ  %.6s %.6s ", WITH->erstername, WITH->zweitername);
      welchsymop(WITH->atona.dessen_atomart->deren_atomlage->grundatom,
		 WITH->atona, rginfo, &anzsymop, symop);
      write_symmetrieoperator(f, symop[0]);
      putc('\n', f->f);
    }
  }
}

Local Void write_dls_tetcon_karten(f, dls_tetcon, LINK)
_TEXT *f;
dls_tetcon_karten dls_tetcon;
struct LOC_write_input_file *LINK;
{
  long i, j, FORLIM;
  _REC_tetcon *WITH;

  FORLIM = dls_tetcon.anztetcon;
  for (i = 0; i < FORLIM; i++) {
    WITH = &dls_tetcon.tetcon[i];
    fprintf(f->f, "TETCON       ");
    for (j = 0; j <= 8; j++)
      fprintf(f->f, "%.6s ", WITH->n[j]);
    putc('\n', f->f);
  }
}

Local Void bestimme_geg_distanzen(atom1, atom2, geg_dist_, d, v, b, LINK)
kt_atom atom1, atom2;
_REC_dt_geg_distanz *geg_dist_;
double *d, *v;
boolean *b;
struct LOC_write_input_file *LINK;
{
  dt_geg_distanz geg_dist;
  long l, kz_a1, kz_a2;
  kt_element el_a1, el_a2;
  _REC_dt_geg_distanz *WITH;

  memcpy(geg_dist, geg_dist_, sizeof(dt_geg_distanz));
  *b = false;
  *d = 0.0;
  *v = 0.0;
  memcpy(el_a1, atom1.dessen_atomart->deren_atomlage->element,
	 sizeof(kt_element));
  memcpy(el_a2, atom2.dessen_atomart->deren_atomlage->element,
	 sizeof(kt_element));
  kz_a1 = koordinationszahl_des_atoms(atom1);
  kz_a2 = koordinationszahl_des_atoms(atom2);
  l = 0;
  do {
    l++;
    WITH = &geg_dist[l - 1];
    if ((!strncmp(el_a1, WITH->el1, sizeof(kt_element)) &&
	 kz_a1 == WITH->kz1 &&
	 !strncmp(el_a2, WITH->el2, sizeof(kt_element)) &&
	 kz_a2 == WITH->kz2) ||
	(!strncmp(el_a1, WITH->el2, sizeof(kt_element)) &&
	 kz_a1 == WITH->kz2 &&
	 !strncmp(el_a2, WITH->el1, sizeof(kt_element)) &&
	 kz_a2 == WITH->kz1)) {
      *d = WITH->distanz;
      *v = WITH->dvar;
      *b = true;
    }
  } while (strncmp(geg_dist[l - 1].el1, "  ", sizeof(kt_element)) && !*b);
  if (!*b)
    printf(" WARNING  no information for distance %.2s[%ld] - %.2s[%ld]\n",
	   el_a1, kz_a1, el_a2, kz_a2);
}

Local Void bestimme_geg_winkel(atom1, atom2, atom3, geg_wink_, w, wv, b, LINK)
kt_atom atom1, atom2, atom3;
_REC_dt_geg_winkel *geg_wink_;
double *w, *wv;
boolean *b;
struct LOC_write_input_file *LINK;
{
  dt_geg_winkel geg_wink;
  long l, kz_a1, kz_a2, kz_a3;
  kt_element el_a1, el_a2, el_a3;
  _REC_dt_geg_winkel *WITH;

  memcpy(geg_wink, geg_wink_, sizeof(dt_geg_winkel));
  *w = 0.0;
  *wv = 0.0;
  *b = false;
  memcpy(el_a1, atom1.dessen_atomart->deren_atomlage->element,
	 sizeof(kt_element));
  memcpy(el_a2, atom2.dessen_atomart->deren_atomlage->element,
	 sizeof(kt_element));
  memcpy(el_a3, atom3.dessen_atomart->deren_atomlage->element,
	 sizeof(kt_element));
  kz_a1 = koordinationszahl_des_atoms(atom1);
  kz_a2 = koordinationszahl_des_atoms(atom2);
  kz_a3 = koordinationszahl_des_atoms(atom3);
  l = 0;
  do {
    l++;
    WITH = &geg_wink[l - 1];
    if ((!strncmp(el_a1, WITH->el1, sizeof(kt_element)) &&
	 kz_a1 == WITH->kz1 &&
	 !strncmp(el_a2, WITH->el2, sizeof(kt_element)) &&
	 kz_a2 == WITH->kz2 &&
	 !strncmp(el_a3, WITH->el3, sizeof(kt_element)) &&
	 kz_a3 == WITH->kz3) ||
	(!strncmp(el_a1, WITH->el3, sizeof(kt_element)) &&
	 kz_a1 == WITH->kz3 &&
	 !strncmp(el_a2, WITH->el2, sizeof(kt_element)) &&
	 kz_a2 == WITH->kz2 &&
	 !strncmp(el_a3, WITH->el1, sizeof(kt_element)) &&
	 kz_a3 == WITH->kz1)) {
      *w = WITH->winkel;
      *wv = WITH->wvar;
      *b = true;
    }
  } while (strncmp(geg_wink[l - 1].el1, "  ", sizeof(kt_element)) && !*b);
  if (!*b)
    printf(" WARNING  no information for angle %.2s[%ld] - %.2s[%ld] - %.2s[%ld]\n",
	   el_a1, kz_a1, el_a2, kz_a2, el_a3, kz_a3);
}

Local Void write_dls_distan_karten(f, geg_dist, dls_distan, LINK)
_TEXT *f;
_REC_dt_geg_distanz *geg_dist;
dls_distan_karten dls_distan;
struct LOC_write_input_file *LINK;
{
  long i;
  boolean b;
  double distanz, dvar;
  long FORLIM;
  _REC_distcon *WITH;

  FORLIM = dls_distan.anzdistcon;
  for (i = 0; i < FORLIM; i++) {
    WITH = &dls_distan.distcon[i];
    bestimme_geg_distanzen(WITH->atom1, WITH->atom2, geg_dist, &distanz,
			   &dvar, &b, LINK);
    fprintf(f->f, "DISTAN %.6s %.6s ", WITH->atomname1, WITH->atomname2);
    if (b)
      fprintf(f->f, "%10.6f%9.4f\n", distanz, 1 / dvar / 50);
    else
      fprintf(f->f, "%10.6f%9.4f\n", 1.0, 0.0);
  }
}

Local Void write_loadat_distan_karten(f, geg_dist, dls_distan, LINK)
_TEXT *f;
_REC_dt_geg_distanz *geg_dist;
dls_distan_karten dls_distan;
struct LOC_write_input_file *LINK;
{
  long i;
  boolean b;
  double distanz, dvar;
  long FORLIM;
  _REC_distcon *WITH;

  FORLIM = dls_distan.anzdistcon;
  for (i = 0; i < FORLIM; i++) {
    WITH = &dls_distan.distcon[i];
    bestimme_geg_distanzen(WITH->atom1, WITH->atom2, geg_dist, &distanz,
			   &dvar, &b, LINK);
    fprintf(f->f, "DISTAN %.6s %.6s ", WITH->atomname1, WITH->atomname2);
    if (b)
      fprintf(f->f, "%10.6f%9.4f\n", distanz, dvar);
    else
      fprintf(f->f, "%10.6f%9.4f\n", 1.0, 100.0);
  }
}

Local Void berechne_t_t_distanzen(d1, d2, w, d1v, d2v, wv, d, dv, LINK)
double d1, d2, w, d1v, d2v, wv, *d, *dv;
struct LOC_write_input_file *LINK;
{
  w /= 57.29577951;
  wv /= 57.29577951;
  *d = sqrt(d1 * d1 + d2 * d2 - 2 * d1 * d2 * cos(w));
  *dv = (d1 * d1v + d2 * d2v - (d1 * d2v + d2 * d1v) * cos(w) +
	 d1 * d2 * sin(w) * wv) / *d;
}

Local Void write_dls_angle_karten(f, geg_wink, geg_dist, dls_angle, LINK)
_TEXT *f;
_REC_dt_geg_winkel *geg_wink;
_REC_dt_geg_distanz *geg_dist;
dls_angle_karten dls_angle;
struct LOC_write_input_file *LINK;
{
  boolean b, bb;
  double d1, d2, d1v, d2v, w, wv, distanz, dvar;
  long i, FORLIM;
  _REC_anglcon *WITH;

  FORLIM = dls_angle.anzanglcon;
  for (i = 0; i < FORLIM; i++) {
    WITH = &dls_angle.anglcon[i];
    bestimme_geg_distanzen(WITH->atom1, WITH->atomm, geg_dist, &d1, &d1v, &bb,
			   LINK);
    bestimme_geg_distanzen(WITH->atom2, WITH->atomm, geg_dist, &d2, &d2v, &b,
			   LINK);
    if (!b)
      bb = false;
    bestimme_geg_winkel(WITH->atom1, WITH->atomm, WITH->atom2, geg_wink, &w,
			&wv, &b, LINK);
    if (!b)
      bb = false;
    if (bb) {
      berechne_t_t_distanzen(d1, d2, w, d1v, d2v, wv, &distanz, &dvar, LINK);
      fprintf(f->f, "DISTAN %.6s %.6s %10.6f%9.4f\n",
	      WITH->atomname1, WITH->atomname2, distanz, 1 / dvar / 50);
    } else
      fprintf(f->f, "DISTAN %.6s %.6s %10.6f%9.4f\n",
	      WITH->atomname1, WITH->atomname2, 1.0, 0.0);
  }
}

Local Void write_loadat_angle_karten(f, geg_wink, dls_angle, LINK)
_TEXT *f;
_REC_dt_geg_winkel *geg_wink;
dls_angle_karten dls_angle;
struct LOC_write_input_file *LINK;
{
  boolean b;
  double winkel, wvar;
  long i, FORLIM;
  _REC_anglcon *WITH;

  FORLIM = dls_angle.anzanglcon;
  for (i = 0; i < FORLIM; i++) {
    WITH = &dls_angle.anglcon[i];
    bestimme_geg_winkel(WITH->atom1, WITH->atomm, WITH->atom2, geg_wink,
			&winkel, &wvar, &b, LINK);
    fprintf(f->f, "ANGLE  %.6s %.6s %.6s ",
	    WITH->atomname1, WITH->atomnamem, WITH->atomname2);
    if (b)
      fprintf(f->f, "%10.6f%9.4f\n", winkel, wvar);
    else
      fprintf(f->f, "%10.6f%9.4f\n", 90.0, 0.0);
  }
}

Local Void write_dls_bondis_karten(t, dls_tetcon, geg_dist, geg_wink, LINK)
_TEXT *t;
dls_tetcon_karten dls_tetcon;
_REC_dt_geg_distanz *geg_dist;
_REC_dt_geg_winkel *geg_wink;
struct LOC_write_input_file *LINK;
{
  bondis_type bondis;
  long i, j, l, nn, kzz;
  double a, b, c, d, e, f, g, d1, d2, d1v, d2v, w1, w2, w1v, w2v, d11, d12,
	 d11v, d12v;
  boolean d1b, d2b, w1b, w2b, bb;
  long FORLIM;
  _REC_tetcon *WITH;
  _REC_bondis_type *WITH1;

  a = 1.0000;
  b = -0.0004;
  c = 0.0000;
  d = 145.00;
  e = 1.0000;
  f = 1.0000;
  g = 1.0000;
  nn = 0;
  FORLIM = dls_tetcon.anztetcon;
  for (i = 0; i < FORLIM; i++) {
    WITH = &dls_tetcon.tetcon[i];
    for (j = 1; j <= 4; j++) {
      WITH1 = &bondis[nn];
      WITH1->a1 = WITH->a[0];
      memcpy(WITH1->e1, WITH1->a1.dessen_atomart->deren_atomlage->element,
	     sizeof(kt_element));
      WITH1->a2 = WITH->a[j];
      memcpy(WITH1->e2, WITH1->a2.dessen_atomart->deren_atomlage->element,
	     sizeof(kt_element));
      WITH1->a3 = WITH->a[j + 4];
      memcpy(WITH1->e3, WITH1->a3.dessen_atomart->deren_atomlage->element,
	     sizeof(kt_element));
      bb = true;
      for (l = 0; l < nn; l++) {
	if (!strncmp(WITH1->e1, bondis[l].e1, sizeof(kt_element)) &&
	    !strncmp(WITH1->e2, bondis[l].e2, sizeof(kt_element)) &&
	    !strncmp(WITH1->e3, bondis[l].e3, sizeof(kt_element)))
	  bb = false;
      }
      if (bb)
	nn++;
    }
  }
  for (i = 0; i < nn; i++) {
    WITH1 = &bondis[i];
    kzz = WITH1->a3.dessen_atomart->deren_atomlage->koordinationszahl;
    WITH1->a3.dessen_atomart->deren_atomlage->koordinationszahl = 4;
    bestimme_geg_distanzen(WITH1->a1, WITH1->a2, geg_dist, &d1, &d1v, &d1b,
			   LINK);
    bestimme_geg_distanzen(WITH1->a3, WITH1->a2, geg_dist, &d2, &d2v, &d2b,
			   LINK);
    bestimme_geg_winkel(WITH1->a1, WITH1->a2, WITH1->a3, geg_wink, &w1, &w1v,
			&w1b, LINK);
    bestimme_geg_winkel(WITH1->a2, WITH1->a1, WITH1->a2, geg_wink, &w2, &w2v,
			&w2b, LINK);
    WITH1->a3.dessen_atomart->deren_atomlage->koordinationszahl = kzz;
    if (d1b) {
      a = d1;
      e = 1 / d1v;
    }
    if (w1b)
      d = w1;
    if (d1b && w2b) {
      berechne_t_t_distanzen(d1, d1, w2, d1v, d1v, w2v, &d11, &d11v, LINK);
      f = 1 / d11v;
    }
    if (d1b && d2b && w1b) {
      berechne_t_t_distanzen(d1, d2, w1, d1v, d2v, w1v, &d12, &d12v, LINK);
      g = 1 / d12v;
    }
    fprintf(t->f,
      "BONDIS %.2s  %.2s  %.2s   %10.5f%10.5f%10.5f%10.5f%5.1f%5.1f%5.1f\n",
      WITH1->e1, WITH1->e2, WITH1->e3, a, b, c, d, e / 50, f / 50, g / 50);
  }
}

Local Void korrigiere_koordinationszahlen(b, LINK)
boolean *b;
struct LOC_write_input_file *LINK;
{
  long i, kzn;
  Char c;
  long FORLIM;
  kt_atomlage *WITH;

  *b = true;
  FORLIM = LINK->strukturdaten.anzatomlagen;
  for (i = 0; i < FORLIM; i++) {
    WITH = LINK->strukturdaten.atomlage[i];
    kzn = anzahl_bindungen_des_atoms(WITH->grundatom);
    if (kzn != WITH->koordinationszahl) {
      if (kzn == 1)
	printf(" ERROR  for atom %.6s there is 1 bond \n", WITH->aname);
      else
	printf(" ERROR  for atom %.6s there are %ld bonds \n",
	       WITH->aname, kzn);
      printf("        but the given coordination number is %ld\n",
	     WITH->koordinationszahl);
      *b = false;
    }
  }
  if (*b)
    return;
  printf(" change coordination numbers to number of bonds? (y/n):");
  if (P_eoln(stdin))
    return;
  scanf("%c%*[^\n]", &c);
  getchar();
  if (c == '\n')
    c = ' ';
  if (c != 'j' && c != 'J' && c != 'y' && c != 'Y')
    return;
  *b = true;
  FORLIM = LINK->strukturdaten.anzatomlagen;
  for (i = 0; i < FORLIM; i++) {
    WITH = LINK->strukturdaten.atomlage[i];
    kzn = anzahl_bindungen_des_atoms(WITH->grundatom);
    if (kzn != WITH->koordinationszahl) {
      printf(" ... coordination number for atom position %.6s", WITH->aname);
      printf(" changed to %ld\n", kzn);
      WITH->koordinationszahl = kzn;
    }
  }
}

Local Void berechne_dls_input(dls_tetcon, dls_distan, dls_angle, dls_symeq, b,
			      LINK)
dls_tetcon_karten *dls_tetcon;
dls_distan_karten *dls_distan;
dls_angle_karten *dls_angle;
dls_symeq_karten *dls_symeq;
boolean *b;
struct LOC_write_input_file *LINK;
{
  long i, j, k, l, anzsymop;
  st_symop_array symeq0;
  kt_atom a1, a2, am, atom0;
  _REC_distcon *WITH1;
  long FORLIM, FORLIM1;
  kt_atomart *WITH2;
  _REC_tetcon *WITH3;
  long FORLIM2, FORLIM3;
  _REC_anglcon *WITH4;

  korrigiere_koordinationszahlen(b, LINK);
  if (!*b) {
    printf(" ... generation of input file interrupted\n");
    return;
  }
  for (i = 0; i < maxanzahlsymeqkarten; i++)
    dls_symeq->anzsymeq[i] = 0;
  for (i = 0; i <= 3; i++) {
    for (j = 0; j <= 8; j++)
      memcpy(dls_tetcon->tetcon[i].n[j], "      ", sizeof(nametyp));
  }
  dls_tetcon->anztetcon = 0;
  FORLIM = LINK->strukturdaten.anzatomlagen;
  for (i = 0; i < FORLIM; i++) {
    if (LINK->strukturdaten.atomlage[i]->koordinationszahl == 4) {
      *b = true;
      WITH2 = LINK->strukturdaten.atomlage[i]->atom[0];
      for (j = 0; j <= 3; j++) {
	if (koordinationszahl_des_atoms(WITH2->bindung[j]) != 2)
	  *b = false;
      }
      if (*b) {
	dls_tetcon->anztetcon++;
	WITH3 = &dls_tetcon->tetcon[dls_tetcon->anztetcon - 1];
	WITH3->a[0] = LINK->strukturdaten.atomlage[i]->grundatom;
	for (j = 1; j <= 4; j++) {
	  berechne_nte_bindung(WITH3->a, j, &WITH3->a[j]);
	  berechne_nte_bindung(&WITH3->a[j], 1L, &WITH3->a[j + 4]);
	  if (gleiche_atome(WITH3->a[j + 4], WITH3->a[0]))
	    berechne_nte_bindung(&WITH3->a[j], 2L, &WITH3->a[j + 4]);
	}
      }
    }
  }
  dls_distan->anzdistcon = 0;
  FORLIM = LINK->strukturdaten.anzatomlagen;
  for (i = 0; i < FORLIM; i++) {
    FORLIM1 = LINK->strukturdaten.atomlage[i]->koordinationszahl;
    for (j = 1; j <= FORLIM1; j++) {
      a1 = LINK->strukturdaten.atomlage[i]->grundatom;
      berechne_nte_bindung(&a1, j, &a2);
      *b = true;
      FORLIM2 = dls_distan->anzdistcon;
      for (k = 0; k < FORLIM2; k++) {
	WITH1 = &dls_distan->distcon[k];
	if (sym_aequival_distanzen(a1, a2, WITH1->atom1, WITH1->atom2) |
	    sym_aequival_distanzen(a1, a2, WITH1->atom2, WITH1->atom1))
	  *b = false;
      }
      if (*b) {
	if (dls_distan->anzdistcon < maxindepdistanzen) {
	  dls_distan->anzdistcon++;
	  dls_distan->distcon[dls_distan->anzdistcon - 1].atom1 = a1;
	  dls_distan->distcon[dls_distan->anzdistcon - 1].atom2 = a2;
	} else {
	  printf(" ERROR  too many symmetrically independent distances\n");
	  printf(
	    "        The program is compiled for a maximum of %ld independent distances.\n",
	    (long)maxindepdistanzen);
	  printf(
	    "        Increase constant \"maxindepdistanzen\" and compile again.\n");
	}
      }
    }
  }
  dls_angle->anzanglcon = 0;
  FORLIM = LINK->strukturdaten.anzatomlagen;
  for (i = 0; i < FORLIM; i++) {
    FORLIM1 = LINK->strukturdaten.atomlage[i]->koordinationszahl;
    for (j = 1; j <= FORLIM1; j++) {
      for (l = 1; l < j; l++) {
	am = LINK->strukturdaten.atomlage[i]->grundatom;
	berechne_nte_bindung(&am, j, &a1);
	berechne_nte_bindung(&am, l, &a2);
	*b = true;
	FORLIM3 = dls_angle->anzanglcon;
	for (k = 0; k < FORLIM3; k++) {
	  WITH4 = &dls_angle->anglcon[k];
	  if (sym_aequival_winkel(a1, am, a2, WITH4->atom1, WITH4->atomm,
				  WITH4->atom2) | sym_aequival_winkel(a1, am,
		a2, WITH4->atom2, WITH4->atomm, WITH4->atom1))
	    *b = false;
	}
	if (*b) {
	  if (dls_angle->anzanglcon < maxindepwinkel) {
	    dls_angle->anzanglcon++;
	    dls_angle->anglcon[dls_angle->anzanglcon - 1].atom1 = a1;
	    dls_angle->anglcon[dls_angle->anzanglcon - 1].atom2 = a2;
	    dls_angle->anglcon[dls_angle->anzanglcon - 1].atomm = am;
	  } else {
	    printf(" ERROR  too many symmetrically independent angles\n");
	    printf(
	      "        The program is compiled for a maximum of %ld independent angles.\n",
	      (long)maxindepwinkel);
	    printf(
	      "        Increase constant \"maxindepwinkel\" and compile again.\n");
	  }
	}
      }
    }
  }

  FORLIM = dls_angle->anzanglcon;
  /* bringe atom1 in Lage x,y,z */
  for (i = 0; i < FORLIM; i++) {
    WITH4 = &dls_angle->anglcon[i];
    atom0 = WITH4->atom1.dessen_atomart->deren_atomlage->grundatom;
    welchsymop(WITH4->atom1, atom0, rginfo, &anzsymop, symeq0);
    alg_symmetrieoperation(symeq0[0], WITH4->atom1, &WITH4->atom1);
    alg_symmetrieoperation(symeq0[0], WITH4->atomm, &WITH4->atomm);
    alg_symmetrieoperation(symeq0[0], WITH4->atom2, &WITH4->atom2);
  }
  *b = true;
}

Local Void erzeuge_atomname(atoco, neuername, dls_symeq, LINK)
kt_atom atoco;
Char *neuername;
dls_symeq_karten *dls_symeq;
struct LOC_write_input_file *LINK;
{
  long i;
  boolean b;
  kt_atom gruato;
  kt_atomlage *WITH1;
  long FORLIM;
  _REC_symeq *WITH2;

  memcpy(neuername, atoco.dessen_atomart->deren_atomlage->aname,
	 sizeof(kt_atomlagename));
  memset(neuername + sizeof(kt_atomlagename), ' ',
	 sizeof(nametyp) - sizeof(kt_atomlagename));
  gruato = atoco.dessen_atomart->deren_atomlage->grundatom;
  if (gleiche_atome(atoco, gruato))
    return;
  WITH1 = atoco.dessen_atomart->deren_atomlage;
  b = true;
  FORLIM = dls_symeq->anzsymeq[WITH1->atomlagenr - 1];
  for (i = 0; i < FORLIM; i++) {
    if (gleiche_atome(atoco, dls_symeq->symeq[WITH1->atomlagenr - 1][i].atona)) {
      b = false;
      memcpy(neuername,
	     dls_symeq->symeq[WITH1->atomlagenr - 1][i].zweitername,
	     sizeof(nametyp));
    }
  }
  if (!b)
    return;
  WITH2 = &dls_symeq->symeq[WITH1->atomlagenr - 1]
    [dls_symeq->anzsymeq[WITH1->atomlagenr - 1]];
  WITH2->atona = atoco;
  memcpy(WITH2->erstername, neuername, sizeof(nametyp));
  i = 1;
  while (WITH2->erstername[i - 1] != ' ')
    i++;
  dls_symeq->anzsymeq[WITH1->atomlagenr - 1]++;
  neuername[i - 1] = '*';
  neuername[i] = (Char)(dls_symeq->anzsymeq[WITH1->atomlagenr - 1] + 'A' - 1);
  memcpy(WITH2->zweitername, neuername, sizeof(nametyp));
}

Local Void loesche_redundante_karten(dls_tetcon, dls_distan, dls_angle, LINK)
dls_tetcon_karten *dls_tetcon;
dls_distan_karten *dls_distan;
dls_angle_karten *dls_angle;
struct LOC_write_input_file *LINK;
{
  long i, j, k, l, m;
  boolean b;
  long FORLIM, FORLIM1;
  _REC_distcon *WITH2;
  _REC_tetcon *WITH3;
  _REC_anglcon *WITH4;

  k = 0;
  FORLIM = dls_distan->anzdistcon;
  for (i = 0; i < FORLIM; i++) {
    b = true;
    FORLIM1 = dls_tetcon->anztetcon;
    for (j = 0; j < FORLIM1; j++) {
      WITH2 = &dls_distan->distcon[i];
      WITH3 = &dls_tetcon->tetcon[j];
      for (l = 1; l <= 4; l++) {
	if (sym_aequival_distanzen(WITH2->atom1, WITH2->atom2, WITH3->a[0],
	      WITH3->a[l]) | sym_aequival_distanzen(WITH2->atom1,
	      WITH2->atom2, WITH3->a[l], WITH3->a[0]))
	  b = false;
      }
    }
    if (b) {
      k++;
      dls_distan->distcon[k - 1] = dls_distan->distcon[i];
    }
  }
  dls_distan->anzdistcon = k;
  k = 0;
  FORLIM = dls_angle->anzanglcon;
  for (i = 0; i < FORLIM; i++) {
    b = true;
    FORLIM1 = dls_tetcon->anztetcon;
    for (j = 0; j < FORLIM1; j++) {
      WITH4 = &dls_angle->anglcon[i];
      WITH3 = &dls_tetcon->tetcon[j];
      for (l = 1; l <= 4; l++) {
	if (sym_aequival_winkel(WITH4->atom1, WITH4->atomm, WITH4->atom2,
	      WITH3->a[0], WITH3->a[l],
	      WITH3->a[l + 4]) | sym_aequival_winkel(WITH4->atom1,
	      WITH4->atomm, WITH4->atom2, WITH3->a[l + 4], WITH3->a[l],
	      WITH3->a[0]))
	  b = false;
	for (m = 1; m < l; m++) {
	  if (sym_aequival_winkel(WITH4->atom1, WITH4->atomm, WITH4->atom2,
		WITH3->a[l], WITH3->a[0],
		WITH3->a[m]) | sym_aequival_winkel(WITH4->atom1, WITH4->atomm,
		WITH4->atom2, WITH3->a[m], WITH3->a[0], WITH3->a[l]))
	    b = false;
	}
      }
    }
    if (b) {
      k++;
      dls_angle->anglcon[k - 1] = dls_angle->anglcon[i];
    }
  }
  dls_angle->anzanglcon = k;
}


Static Void write_input_file(f, num, strukturdaten_, distdat)
_TEXT *f;
long num;
kt_strukturdaten strukturdaten_;
_TEXT *distdat;
{
  struct LOC_write_input_file V;
  dls_tetcon_karten dls_tetcon;
  dls_distan_karten dls_distan;
  dls_angle_karten dls_angle;
  dls_symeq_karten dls_symeq;
  dt_geg_distanz geg_dist;
  dt_geg_winkel geg_wink;
  long i, j;
  boolean b;
  vektor v;
  long FORLIM;
  _REC_tetcon *WITH;
  _REC_distcon *WITH1;
  _REC_anglcon *WITH2;

  V.strukturdaten = strukturdaten_;
  read_bindungs_groessen(distdat, geg_dist, geg_wink, &V);
  berechne_dls_input(&dls_tetcon, &dls_distan, &dls_angle, &dls_symeq, &b, &V);
  if (b) {
    if (num == 1) {
      loesche_redundante_karten(&dls_tetcon, &dls_distan, &dls_angle, &V);
      FORLIM = dls_tetcon.anztetcon;
      for (i = 0; i < FORLIM; i++) {
	WITH = &dls_tetcon.tetcon[i];
	for (j = 0; j <= 8; j++)
	  erzeuge_atomname(WITH->a[j], WITH->n[j], &dls_symeq, &V);
      }
    }
    FORLIM = dls_distan.anzdistcon;
    for (i = 0; i < FORLIM; i++) {
      WITH1 = &dls_distan.distcon[i];
      erzeuge_atomname(WITH1->atom1, WITH1->atomname1, &dls_symeq, &V);
      erzeuge_atomname(WITH1->atom2, WITH1->atomname2, &dls_symeq, &V);
    }
    if (num == 1 || num == 2) {
      FORLIM = dls_angle.anzanglcon;
      for (i = 0; i < FORLIM; i++) {
	WITH2 = &dls_angle.anglcon[i];
	erzeuge_atomname(WITH2->atom1, WITH2->atomname1, &dls_symeq, &V);
	erzeuge_atomname(WITH2->atom2, WITH2->atomname2, &dls_symeq, &V);
      }
    }
    if (num == 3) {
      FORLIM = dls_angle.anzanglcon;
      for (i = 0; i < FORLIM; i++) {
	WITH2 = &dls_angle.anglcon[i];
	erzeuge_atomname(WITH2->atom1, WITH2->atomname1, &dls_symeq, &V);
	erzeuge_atomname(WITH2->atomm, WITH2->atomnamem, &dls_symeq, &V);
	erzeuge_atomname(WITH2->atom2, WITH2->atomname2, &dls_symeq, &V);
      }
    }
    if (num == 1 || num == 2) {
      fprintf(f->f, "TITLE %.80s\n", V.strukturdaten.name);
      if (num == 1)
	fprintf(f->f, "DLS-76   1    10   2 0         2                1\n");
      if (num == 2)
	fprintf(f->f, "DLS-76   1    10     0         2                1\n");
      if (!strncmp(V.strukturdaten.kristallsystem, "TRIG",
		   sizeof(st_kristallsystem)))
	memcpy(V.strukturdaten.kristallsystem, "HEX ",
	       sizeof(st_kristallsystem));
      fprintf(f->f,
	      "CELL   %.4s         %10.4f%10.4f%10.4f%10.4f%10.4f%10.4f\n",
	      V.strukturdaten.kristallsystem,
	      V.strukturdaten.zellpar.laenge[0],
	      V.strukturdaten.zellpar.laenge[1],
	      V.strukturdaten.zellpar.laenge[2],
	      V.strukturdaten.zellpar.winkel[0],
	      V.strukturdaten.zellpar.winkel[1],
	      V.strukturdaten.zellpar.winkel[2]);
      write_dls_atom_karten(f, V.strukturdaten, &V);
      write_dls_symeq_karten(f, dls_symeq, &V);
      if (num == 1)
	write_dls_bondis_karten(f, dls_tetcon, geg_dist, geg_wink, &V);
      write_dls_distan_karten(f, geg_dist, dls_distan, &V);
      write_dls_angle_karten(f, geg_wink, geg_dist, dls_angle, &V);
    }
    if (num == 1)
      write_dls_tetcon_karten(f, dls_tetcon, &V);
    if (num == 3) {
      if (*f->name != '\0') {
	if (f->f != NULL)
	  f->f = freopen(f->name, "w", f->f);
	else
	  f->f = fopen(f->name, "w");
      } else {
	if (f->f != NULL)
	  rewind(f->f);
	else
	  f->f = tmpfile();
      }
      if (f->f == NULL)
	_EscIO(FileNotFound);
      SETUPBUF(f->f, Char);
      write_loadat_atom_karten(f, V.strukturdaten, &V);
      write_dls_symeq_karten(f, dls_symeq, &V);
      write_loadat_distan_karten(f, geg_dist, dls_distan, &V);
      write_loadat_angle_karten(f, geg_wink, dls_angle, &V);
    }
    fprintf(f->f, "END\n");
    fprintf(f->f, "FINISH\n");
  }
  b = false;
  v[0] = 1.0;
  v[1] = 0.0;
  v[2] = 0.0;
  if (polare_richtung(v, rginfo)) {
    printf(
      " WARNING  in this space group, the origin is not defined in the x direction \n");
    printf(
      "          The x coordinate of one atom must be fixed with a NOREF card.\n");
  }
  v[0] = 0.0;
  v[1] = 1.0;
  v[2] = 0.0;
  if (polare_richtung(v, rginfo)) {
    printf(
      " WARNING  in this space group, the origin is not defined in the y direction \n");
    printf(
      "          The y coordinate of one atom must be fixed with a NOREF card.\n");
  }
  v[0] = 0.0;
  v[1] = 0.0;
  v[2] = 1.0;
  if (polare_richtung(v, rginfo)) {
    printf(
      " WARNING  in this space group, the origin is not defined in the z direction \n");
    printf(
      "          The z coordinate of one atom must be fixed with a NOREF card.\n");
  }
}


Static Void m_initialisiere(strukturdaten)
kt_strukturdaten *strukturdaten;
{
  memcpy(strukturdaten->raumgruppe, "                              ",
	 sizeof(st_raumgruppename));
  berechnet &= ~(1 << 0);
  berechnet &= ~(1 << ((long)distanzen - (long)struktur));
  berechnet &= ~(1 << ((long)bindungen - (long)struktur));
  strudatnew_offen = false;
  if (*list.name != '\0') {
    if (list.f != NULL)
      list.f = freopen(list.name, "w", list.f);
    else
      list.f = fopen(list.name, "w");
  } else {
    if (list.f != NULL)
      rewind(list.f);
    else
      list.f = tmpfile();
  }
  if (list.f == NULL)
    _EscIO(FileNotFound);
  SETUPBUF(list.f, Char);
  printf(
    " ******************************************************************************\n");
  printf(
    " *                                                                            *\n");
  printf(
    " *   Program  KRIBER        Version 1.0    (January 1991)                     *\n");
  printf(
    " *   ===============                                                          *\n");
  printf(
    " *                                                                            *\n");
  printf(
    " *   an interactive PASCAL program to                                         *\n");
  printf(
    " *    - calculate distances and angles                                        *\n");
  printf(
    " *    - generate input files for the programs DLS-76 and LOADAT (XRS-82)      *\n");
  printf(
    " *    - calculate coordination squences and loop configurations               *\n");
  printf(
    " *                                                                            *\n");
  printf(
    " *   Author:   Roland Bialek, Institut fuer Kristallografie und Petrografie   *\n");
  printf(
    " *             ETH-Z, CH-8092 Zuerich, Switzerland                            *\n");
  printf(
    " *                                                                            *\n");
  printf(
    " *    ===> type help to get a summary of the commands                         *\n");
  printf(
    " *              quit or exit to leave the program                             *\n");
  printf(
    " *                                                                            *\n");
  printf(
    " ******************************************************************************\n");
}


Static Void m_write_hilfe()
{
  printf("\n *******************************************************************\n");
  printf(" *                                                                 *\n");
  printf(" *          S U M M A R Y    O F   A L L   C O M M A N D S         *\n");
  printf(" *                                                                 *\n");
  printf(" *******************************************************************\n\n");
  printf(" read      crystal structure data                        ===> reacs \n");
  printf(" read      data from first entry in file STRUDAT         ===> reafcs\n");
  printf(" read      data from next entry in file STRUDAT          ===> reancs\n");
  printf(" write     structure data on file STRUDATNEW             ===> wrie  \n");
  printf(" display   all entries in file STRUDAT                   ===> dise  \n");
  printf("                                                                    \n");
  printf(" display   crystal structure                             ===> discs \n");
  printf(" write     crystal structure on file LIST                ===> wrics \n");
  printf(" add       atom position                                 ===> adda  \n");
  printf(" delete    atom position                                 ===> dela  \n");
  printf(" display   space group                                   ===> diss  \n");
  printf(" write     space group on file LIST                      ===> wris  \n");
  printf(" display   coordinates                                   ===> disc  \n");
  printf(" write     coordinates on file LIST                      ===> wric  \n");
  printf("                                                                    \n");
  printf("        Type <ret> to continue                                      \n");
  scanf("%*[^\n]");
  getchar();
  printf("                                                                    \n");
  printf(" calculate distances                                     ===> cald  \n");
  printf(" display   all distances                                 ===> disad \n");
  printf(" display   distances from one atom                       ===> disd  \n");
  printf(" display   distances and angles                          ===> disda \n");
  printf(" display   distances and interatomic vectors             ===> disdv \n");
  printf(" write     all distances on file LIST                    ===> wriad \n");
  printf(" write     distances from one atom on file LIST          ===> wrid  \n");
  printf(" write     distances and angles on file LIST             ===> wrida \n");
  printf(" write     distances and interatomic vectors on file LIST===> wridv \n");
  printf("                                                                    \n");
  printf(" create    all bonds using shortest distances            ===> creab \n");
  printf(" create    a bond                                        ===> creb  \n");
  printf(" delete    a bond                                        ===> delb  \n");
  printf(" delete    all bonds                                     ===> delab \n");
  printf(" display   bonds from one atom                           ===> disb  \n");
  printf(" display   all bonds                                     ===> disab \n");
  printf(" write     bonds from one atom on file LIST              ===> wrib  \n");
  printf(" write     all bonds on file LIST                        ===> wriab \n");
  printf("                                                                    \n");
  printf("                                                                    \n");
  printf("        Type <ret> to continue                                      \n");
  scanf("%*[^\n]");
  getchar();
  printf("                                                                    \n");
  printf(" add       O atoms (oxygen bridges)                      ===> addo  \n");
  printf(" delete    oxygen bridges, connect non O-atoms directly  ===> delo  \n");
  printf("                                                                    \n");
  printf(" calculate coordination sequences                        ===> calcs \n");
  printf(" calculate loop configuration                            ===> callc \n");
  printf(" determine topology of the framework                     ===> dett  \n");
  printf("                                                                    \n");
  printf(" write     input file for DLS-76 on file DLSINPUT        ===> wriid \n");
  printf(" write     input file for LOADAT on file LOADATINPUT     ===> wriil \n");
  printf("                                                                    \n");
  printf("                                                                    \n");
  printf(" display   a summary of the commands                     ===> help  \n");
  printf(" quit or exit the program                                ===> quit  \n");
  printf("                                                      or ===> exit  \n");
  printf("                                                                    \n");
  printf("                                                                    \n");
  printf(
    " *******************************************************************\n\n\n\n\n");
  /* Fuer Kontrollzwecke  cons : kontrolliere Raumgruppe                      */
  /* Fuer Kontrollzwecke  conb : schreibe alle Bindungen                      */
  /* Fuer Kontrollzwecke  calda: berechne Distanzen algebraisch               */
}


Static Void m_read_strukturdaten()
{
  long i;
  boolean eintrag_gefunden;
  _TEXT TEMP;

  if (*strudat.name != '\0') {
    if (strudat.f != NULL)
      strudat.f = freopen(strudat.name, "r", strudat.f);
    else
      strudat.f = fopen(strudat.name, "r");
  } else
    rewind(strudat.f);
  if (strudat.f == NULL)
    _EscIO(FileNotFound);
  RESETBUF(strudat.f, Char);
  printf(" Enter compound identification code to read data from the file\n");
  printf(" STRUDAT or <ret> to enter the data interactively:");
  if (P_eof(stdin)) {
    rewind(stdin);
    TEMP.f = stdin;
    *TEMP.name = '\0';
    read_strukturdaten(&TEMP, &strukturdaten, true);
    memcpy(strukturdaten.abk, "???       ", sizeof(abkbezeichnung));
    berechnet |= 1 << 0;
    berechnet &= ~(1 << ((long)distanzen - (long)struktur));
    berechnet &= ~(1 << ((long)bindungen - (long)struktur));
    return;
  }
  for (i = 0; i <= 9; i++)
    strukturdaten.abk[i] = ' ';
  i = 0;
  while (!P_eoln(stdin) && i < 10) {
    i++;
    strukturdaten.abk[i - 1] = getchar();
    if (strukturdaten.abk[i - 1] == '\n')
      strukturdaten.abk[i - 1] = ' ';
  }
  if (strukturdaten.abk[9] != ' ')
    printf(" ERROR  compound identification code too long\n");
  scanf("%*[^\n]");
  getchar();
  if (*strudat.name != '\0') {
    if (strudat.f != NULL)
      strudat.f = freopen(strudat.name, "r", strudat.f);
    else
      strudat.f = fopen(strudat.name, "r");
  } else
    rewind(strudat.f);
  if (strudat.f == NULL)
    _EscIO(FileNotFound);
  RESETBUF(strudat.f, Char);
  suche_strukturdaten_eintrag(&strudat, strukturdaten.abk, &eintrag_gefunden);
  if (!eintrag_gefunden)
    return;
  read_strukturdaten(&strudat, &strukturdaten, false);
  berechnet |= 1 << 0;
  berechnet &= ~(1 << ((long)distanzen - (long)struktur));
  berechnet &= ~(1 << ((long)bindungen - (long)struktur));
}


Static Void m_read_erste_strukturdaten()
{
  boolean b;

  if (*strudat.name != '\0') {
    if (strudat.f != NULL)
      strudat.f = freopen(strudat.name, "r", strudat.f);
    else
      strudat.f = fopen(strudat.name, "r");
  } else
    rewind(strudat.f);
  if (strudat.f == NULL)
    _EscIO(FileNotFound);
  RESETBUF(strudat.f, Char);
  suche_naechsten_eintrag(&strudat, strukturdaten.abk, &b);
  if (!b) {
    printf(" ERROR  no entry found on file STRUDAT\n");
    return;
  }
  printf(" ... entry %.10s found\n", strukturdaten.abk);
  read_strukturdaten(&strudat, &strukturdaten, false);
  berechnet |= 1 << 0;
  berechnet &= ~(1 << ((long)distanzen - (long)struktur));
  berechnet &= ~(1 << ((long)bindungen - (long)struktur));
}


Static Void m_read_naechste_strukturdaten()
{
  boolean b;

  suche_naechsten_eintrag(&strudat, strukturdaten.abk, &b);
  if (!b) {
    printf(" ERROR  no entry found on file STRUDAT\n");
    return;
  }
  printf(" ... entry %.10s found\n", strukturdaten.abk);
  read_strukturdaten(&strudat, &strukturdaten, false);
  berechnet |= 1 << 0;
  berechnet &= ~(1 << ((long)distanzen - (long)struktur));
  berechnet &= ~(1 << ((long)bindungen - (long)struktur));
}


Static Void m_write_strukturdaten(f, strukturdaten)
_TEXT *f;
kt_strukturdaten strukturdaten;
{
  if ((berechnet & (1 << 0)) == 0)
    m_read_strukturdaten();
  write_strukturdaten(f, strukturdaten);
}


Static Void m_contr_st_raumgruppe()
{
  vektor v;
  _TEXT TEMP;

  TEMP.f = stdin;
  *TEMP.name = '\0';
  readln_raumgruppename(&TEMP, strukturdaten.raumgruppe);
  read_raumgruppe(&symdat, strukturdaten.raumgruppe, &rginfo);
  memcpy(strukturdaten.kristallsystem, rginfo.kristallsystem,
	 sizeof(st_kristallsystem));
  TEMP.f = stdout;
  *TEMP.name = '\0';
  write_st_raumgruppe(&TEMP, rginfo);
  v[0] = 1.0;
  v[1] = 0.0;
  v[2] = 0.0;
  if (polare_richtung(v, rginfo)) {
    printf(" possible shift in ");
    TEMP.f = stdout;
    *TEMP.name = '\0';
    write_vektor(&TEMP, v);
    putchar('\n');
  }
  v[0] = 0.0;
  v[1] = 1.0;
  v[2] = 0.0;
  if (polare_richtung(v, rginfo)) {
    printf(" possible shift in ");
    TEMP.f = stdout;
    *TEMP.name = '\0';
    write_vektor(&TEMP, v);
    putchar('\n');
  }
  v[0] = 0.0;
  v[1] = 0.0;
  v[2] = 1.0;
  if (!polare_richtung(v, rginfo))
    return;
  printf(" possible shift in ");
  TEMP.f = stdout;
  *TEMP.name = '\0';
  write_vektor(&TEMP, v);
  putchar('\n');
}


Static Void m_write_koordinaten(f)
_TEXT *f;
{
  long i, FORLIM;

  if ((berechnet & (1 << 0)) == 0)
    m_read_strukturdaten();
  FORLIM = strukturdaten.anzatomlagen;
  for (i = 0; i < FORLIM; i++)
    write_atomlage_koordinaten(f, *strukturdaten.atomlage[i]);
}


Static Void m_berechne_distanzen(anzdistanzen)
long anzdistanzen;
{
  long i, FORLIM;

  if ((berechnet & (1 << 0)) == 0)
    m_read_strukturdaten();
  FORLIM = strukturdaten.anzatomlagen;
  for (i = 0; i < FORLIM; i++)
    berechne_distanzen(strukturdaten.atomlage[i]->grundatom, strukturdaten,
		       anzdistanzen, 8.0);
  berechnet |= 1 << ((long)distanzen - (long)struktur);
}


Static Void m_write_distanzen(f)
_TEXT *f;
{
  kt_atom a;
  _TEXT TEMP;

  if ((berechnet & (1 << ((long)distanzen - (long)struktur))) == 0)
    m_berechne_distanzen((long)maxanzahldistanzen);
  printf(" display distances from atom:");
  TEMP.f = stdin;
  *TEMP.name = '\0';
  readln_atom(&TEMP, &a);
  if (a.dessen_atomart != NULL)
    write_distanzen(f, a);
  else
    printf(" ERROR  cannot display distances\n");
}


Static Void m_write_alle_distanzen(f)
_TEXT *f;
{
  long i, FORLIM;

  if ((berechnet & (1 << ((long)distanzen - (long)struktur))) == 0)
    m_berechne_distanzen((long)maxanzahldistanzen);
  FORLIM = strukturdaten.anzatomlagen;
  for (i = 0; i < FORLIM; i++)
    write_distanzen(f, strukturdaten.atomlage[i]->grundatom);
}


Static Void m_write_distanzen_und_winkel(f)
_TEXT *f;
{
  long i, FORLIM;

  if ((berechnet & (1 << ((long)distanzen - (long)struktur))) == 0)
    m_berechne_distanzen((long)maxanzahldistanzen);
  FORLIM = strukturdaten.anzatomlagen;
  for (i = 0; i < FORLIM; i++)
    write_distanzen_und_winkel(f, strukturdaten.atomlage[i]->grundatom);
}


Static Void m_write_distanzen_und_vektoren(f)
_TEXT *f;
{
  long i, FORLIM;

  if ((berechnet & (1 << ((long)distanzen - (long)struktur))) == 0)
    m_berechne_distanzen((long)maxanzahldistanzen);
  FORLIM = strukturdaten.anzatomlagen;
  for (i = 0; i < FORLIM; i++)
    write_distanzen_und_vektoren(f, strukturdaten.atomlage[i]->grundatom);
}


Static Void m_berechne_bindungen()
{
  if ((berechnet & (1 << ((long)distanzen - (long)struktur))) == 0)
    m_berechne_distanzen((long)maxanzahldistanzen);
  berechne_alle_bindungen(&strukturdaten);
  berechnet |= 1 << ((long)bindungen - (long)struktur);
}


Static Void m_write_bindungen(f)
_TEXT *f;
{
  kt_atom a;
  _TEXT TEMP;

  if ((berechnet & (1 << ((long)bindungen - (long)struktur))) == 0)
    m_berechne_bindungen();
  printf(" display bonds from atom:");
  TEMP.f = stdin;
  *TEMP.name = '\0';
  readln_atom(&TEMP, &a);
  if (a.dessen_atomart != NULL)
    write_bindungen(f, a);
  else
    printf(" ERROR  cannot display bonds\n");
}


Static Void m_write_alle_bindungen(f)
_TEXT *f;
{
  long i, FORLIM;

  if ((berechnet & (1 << ((long)bindungen - (long)struktur))) == 0)
    m_berechne_bindungen();
  FORLIM = strukturdaten.anzatomlagen;
  for (i = 0; i < FORLIM; i++)
    write_bindungen(f, strukturdaten.atomlage[i]->grundatom);
}


Static Void m_write_bindungs_matrix(f)
_TEXT *f;
{
  long i, FORLIM;

  FORLIM = strukturdaten.anzatomlagen;
  for (i = 0; i < FORLIM; i++)
    write_bindungs_matrix(f, *strukturdaten.atomlage[i]);
}


Static Void m_DLSinput()
{
  Char c;
  long n;

  if ((berechnet & (1 << ((long)bindungen - (long)struktur))) == 0)
	/*#### BK ut */
	  m_berechne_bindungen();
  if (*dlsinp.name != '\0') {
    if (dlsinp.f != NULL)
      dlsinp.f = freopen(dlsinp.name, "w", dlsinp.f);
    else
      dlsinp.f = fopen(dlsinp.name, "w");
  } else {
    if (dlsinp.f != NULL)
      rewind(dlsinp.f);
    else
      dlsinp.f = tmpfile();
  }
  if (dlsinp.f == NULL)
    _EscIO(FileNotFound);
  SETUPBUF(dlsinp.f, Char);
  printf(" DLSinput with TETCON cards? (y/n):");
  if (!P_eoln(stdin)) {   /*#### BK ut */
    scanf("%c%*[^\n]", &c);
    getchar();
    if (c == '\n')
      c = ' ';
    if (c == 'j' || c == 'J' || c == 'y' || c == 'Y')
      n = 1;
    if (c == 'n' || c == 'N')
      n = 2;
  }
  write_input_file(&dlsinp, n, strukturdaten, &distdat);
}


Static Void m_LOADATinput()
{   /*#### BK loadatinput*/
  if (*loadainp.name != '\0') {
    if (loadainp.f != NULL)
      loadainp.f = freopen(loadainp.name, "w", loadainp.f);
    else
      loadainp.f = fopen(loadainp.name, "w");
  } else {
    if (loadainp.f != NULL)
      rewind(loadainp.f);
    else
      loadainp.f = tmpfile();
  }
  if (loadainp.f == NULL)
    _EscIO(FileNotFound);
  SETUPBUF(loadainp.f, Char);
  if ((berechnet & (1 << ((long)bindungen - (long)struktur))) == 0)
	/*#### BK loadatinput */
	  m_berechne_bindungen();
  write_input_file(&loadainp, 3L, strukturdaten, &distdat);
}


Static Void m_berechne_o_positionen()
{
  boolean status;

  if ((berechnet & (1 << ((long)bindungen - (long)struktur))) == 0)
    m_berechne_bindungen();
  baue_o_atome_ein(&strukturdaten, &status);
  berechnet &= ~(1 << ((long)distanzen - (long)struktur));
}


Static Void m_write_strukturdaten_karte()
{
  if (!strudatnew_offen) {   /*#### BK ew */
    if (*strudatn.name != '\0') {
      if (strudatn.f != NULL)
	strudatn.f = freopen(strudatn.name, "w", strudatn.f);
      else
	strudatn.f = fopen(strudatn.name, "w");
    } else {
      if (strudatn.f != NULL)
	rewind(strudatn.f);
      else
	strudatn.f = tmpfile();
    }
    if (strudatn.f == NULL)
      _EscIO(FileNotFound);
    SETUPBUF(strudatn.f, Char);
    strudatnew_offen = true;
  }
  /*#### BK ew */
  write_strukturdaten_karte(&strudatn, strukturdaten);
}


Static Void m_loesche_atomlage()
{
  kt_atomlage *pala;
  _TEXT TEMP;
  kt_atomart *WITH;

  printf(" Atom position to be deleted:");
  if (P_eof(stdin))
    return;
  TEMP.f = stdin;
  *TEMP.name = '\0';
  read_atomlage_bez(&TEMP, &pala);
  scanf("%*[^\n]");
  getchar();
  if (pala == NULL) {
    printf(" ERROR  cannot delete this atom\n");
    return;
  }
  delete_atomlage(pala, &strukturdaten);
  berechnet &= ~(1 << ((long)distanzen - (long)struktur));
  WITH = pala->atom[0];
  while (WITH->bindung[0].dessen_atomart != NULL)
    loesche_bindung(pala->grundatom, WITH->bindung[0]);
}


Static Void m_erzeuge_atomlage()
{
  kt_atomlage *neue_atomlage;
  boolean b;
  _TEXT TEMP;

  printf(" Add atom position no. %ld\n", strukturdaten.anzatomlagen + 1);
  printf(" label (element symbol + number)   coordinates   coordination number\n");
  neue_atomlage = (kt_atomlage *)Malloc(sizeof(kt_atomlage));
  printf(" (%2ld)  :", strukturdaten.anzatomlagen + 1);
  TEMP.f = stdin;
  *TEMP.name = '\0';
  read_atomlage(&TEMP, &neue_atomlage);
  frac_vektor(neue_atomlage->atomlagekoord, neue_atomlage->atomlagekoord);
  lade_atomlage(&neue_atomlage, &strukturdaten, &b);
  berechnet &= ~(1 << ((long)distanzen - (long)struktur));
}


Static Void m_erzeuge_bindung()
{
  kt_atom a1, a2;
  double d;
  long n;
  _TEXT TEMP;

  printf(" Create new bond:             first atom:");
  TEMP.f = stdin;
  *TEMP.name = '\0';
  readln_atom(&TEMP, &a1);
  if (a1.dessen_atomart != NULL) {
    a2.dessen_atomart = NULL;
    printf(" second atom or distance no. (from disd):");
    while ((!P_eoln(stdin)) & (P_peek(stdin) == ' '))
      getc(stdin);
    if (!P_eoln(stdin)) {
      if (P_peek(stdin) == '(')
	getc(stdin);
      if (P_peek(stdin) >= '1' && P_peek(stdin) <= '9' || P_peek(stdin) == '0') {
	scanf("%ld%*[^\n]", &n);
	getchar();
	berechne_nte_distanz(a1, n, &d, &a2);
      } else {
	TEMP.f = stdin;
	*TEMP.name = '\0';
	readln_atom(&TEMP, &a2);
      }
    }
    if (a2.dessen_atomart != NULL)
      erzeuge_bindung(a1, a2);
    else
      printf(" ERROR  cannot create this bond\n");
  } else
    printf(" ERROR  cannot create this bond\n");
  berechnet |= 1 << ((long)bindungen - (long)struktur);
}


Static Void m_loesche_bindung()
{
  kt_atom a1, a2;
  long n;
  _TEXT TEMP;

  printf(" Delete bond:             first atom:");
  TEMP.f = stdin;
  *TEMP.name = '\0';
  readln_atom(&TEMP, &a1);
  if (a1.dessen_atomart == NULL) {
    printf(" ERROR  cannot delete this bond\n");
    return;
  }
  a2.dessen_atomart = NULL;
  printf(" second atom or bond no. (from disb):");
  while ((!P_eoln(stdin)) & (P_peek(stdin) == ' '))
    getc(stdin);
  if (!P_eoln(stdin)) {
    if (P_peek(stdin) == '(')
      getc(stdin);
    if (P_peek(stdin) >= '1' && P_peek(stdin) <= '9' || P_peek(stdin) == '0') {
      scanf("%ld%*[^\n]", &n);
      getchar();
      if (n > 0 && n <= maxanzahlbindungen)
	berechne_nte_bindung(&a1, n, &a2);
      else
	a2.dessen_atomart = NULL;
      if (a2.dessen_atomart == NULL) {
	printf(" ERROR  there is no bond no. %ld for atom \n", n);
	TEMP.f = stdout;
	*TEMP.name = '\0';
	write_atom_name(&TEMP, a1);
      }
    } else {
      TEMP.f = stdin;
      *TEMP.name = '\0';
      readln_atom(&TEMP, &a2);
    }
  }
  if (a2.dessen_atomart != NULL)
    loesche_bindung(a1, a2);
  else
    printf(" ERROR  cannot delete this bond\n");
}


Static Void m_loesche_alle_bindung()
{
  loesche_alle_bindungen(&strukturdaten);
}


Static Void m_verknuepfe_t_atome_direkt()
{
  if ((berechnet & (1 << ((long)bindungen - (long)struktur))) == 0)
    m_berechne_bindungen();
  verknuepfe_t_atome_direkt();
}


Static Void m_berechne_koord_seq()
{
  long i;
  tt_koord_sequenz z;
  long FORLIM;
  _TEXT TEMP;

  if ((berechnet & (1 << ((long)bindungen - (long)struktur))) == 0)
    m_berechne_bindungen();
  printf("\n Coordination sequences of the T-atoms:\n");
  FORLIM = strukturdaten.anzatomlagen;
  for (i = 0; i < FORLIM; i++) {
    berechne_koord_seq(strukturdaten.atomlage[i]->grundatom, 11L, z);
    printf(" %.6s", strukturdaten.atomlage[i]->aname);
    TEMP.f = stdout;
    *TEMP.name = '\0';
    write_koord_seq(&TEMP, z);
  }
}


Static Void m_bestimme_topologie()
{
  if ((berechnet & (1 << ((long)bindungen - (long)struktur))) == 0)
    m_berechne_bindungen();
  bestimme_topologie();
}


Static Void m_berechne_loop_konfig()
{
  long i, FORLIM;
  _TEXT TEMP;

  if ((berechnet & (1 << ((long)bindungen - (long)struktur))) == 0)
    m_berechne_bindungen();
  FORLIM = strukturdaten.anzatomlagen;
  for (i = 0; i < FORLIM; i++) {
    printf("\n Loop configuration for ");
    TEMP.f = stdout;
    *TEMP.name = '\0';
    write_atom_name(&TEMP, strukturdaten.atomlage[i]->grundatom);
    putchar('\n');
    berechne_loop_konfig(strukturdaten.atomlage[i]->grundatom);
  }
}


Static Void m_write_entry_liste()
{
  long i;

  if (*strudat.name != '\0') {
    if (strudat.f != NULL)
      strudat.f = freopen(strudat.name, "r", strudat.f);
    else
      strudat.f = fopen(strudat.name, "r");
  } else
    rewind(strudat.f);
  if (strudat.f == NULL)
    _EscIO(FileNotFound);
  RESETBUF(strudat.f, Char);
  printf(" Entries on file STRUDAT\n");
  while (!BUFEOF(strudat.f)) {
    if (P_peek(strudat.f) == '*') {
      i = 0;
      putchar(' ');
      while (!P_eoln(strudat.f)) {
	putchar(P_peek(strudat.f));
	getc(strudat.f);
	i++;
      }
      putchar('\n');
      if (i > 9)
	printf(" ERROR  compound identification code too long\n");
    }
    fscanf(strudat.f, "%*[^\n]");
    getc(strudat.f);
  }
}


Static Void m_read_befehl(befehl)
Char *befehl;
{
  long i;

  for (i = 0; i <= 7; i++)
    befehl[i] = ' ';
  printf(" ===>");
  i = 0;
  if (P_eof(stdin)) {
    rewind(stdin);
    return;
  }
  while ((!P_eoln(stdin)) & (P_peek(stdin) == ' '))
    getc(stdin);
  while (!P_eoln(stdin) && i < 8) {
    i++;
    befehl[i - 1] = getchar();
    if (befehl[i - 1] == '\n')
      befehl[i - 1] = ' ';
  }
  scanf("%*[^\n]");
  getchar();
  if (befehl[7] != ' ')
    printf(" ERROR  command too long\n");
  for (i = 0; i <= 7; i++) {
    if (isupper(befehl[i]))
      befehl[i] = _tolower(befehl[i]);
  }
}


main(argc, argv)
int argc;
Char *argv[];
{  /* main */
  _TEXT TEMP;

  PASCAL_MAIN(argc, argv);
  coseq.f = NULL;
  strcpy(coseq.name, "coseq");
  list.f = NULL;
  strcpy(list.name, "list");
  loadainp.f = NULL;
  strcpy(loadainp.name, "loadainp");
  dlsinp.f = NULL;
  strcpy(dlsinp.name, "dlsinp");
  distdat.f = NULL;
  strcpy(distdat.name, "distdat");
  strudatn.f = NULL;
  strcpy(strudatn.name, "strudatn");
  symdat.f = NULL;
  strcpy(symdat.name, "symdat");
  strudat.f = NULL;
  strcpy(strudat.name, "strudat");
  m_initialisiere(&strukturdaten);
  do {
    m_read_befehl(befehl);
    if (strncmp(befehl, "        ", sizeof(mt_befehl))) {
      if (!strncmp(befehl, "calcs   ", sizeof(mt_befehl)))
	m_berechne_koord_seq();
      else if (!strncmp(befehl, "cald    ", sizeof(mt_befehl)))
	m_berechne_distanzen((long)maxanzahldistanzen);
      else if (!strncmp(befehl, "calda   ", sizeof(mt_befehl)))
	berechne_distanzen_algebraisch(rginfo);
      else if (!strncmp(befehl, "creab   ", sizeof(mt_befehl)))
	m_berechne_bindungen();
      else if (!strncmp(befehl, "callc   ", sizeof(mt_befehl)))
	m_berechne_loop_konfig();
      else if (!strncmp(befehl, "addo    ", sizeof(mt_befehl)))
	m_berechne_o_positionen();
      else if (!strncmp(befehl, "adda    ", sizeof(mt_befehl)))
	m_erzeuge_atomlage();
      else if (!strncmp(befehl, "creb    ", sizeof(mt_befehl)))
	m_erzeuge_bindung();
      else if (!strncmp(befehl, "conb    ", sizeof(mt_befehl))) {
	TEMP.f = stdout;
	*TEMP.name = '\0';
	m_write_bindungs_matrix(&TEMP);
      } else if (!strncmp(befehl, "cons    ", sizeof(mt_befehl)))
	m_contr_st_raumgruppe();
      else if (!strncmp(befehl, "dela    ", sizeof(mt_befehl)))
	m_loesche_atomlage();
      else if (!strncmp(befehl, "delab   ", sizeof(mt_befehl)))
	m_loesche_alle_bindung();
      else if (!strncmp(befehl, "delb    ", sizeof(mt_befehl)))
	m_loesche_bindung();
      else if (!strncmp(befehl, "delo    ", sizeof(mt_befehl)))
	m_verknuepfe_t_atome_direkt();
      else if (!strncmp(befehl, "dett    ", sizeof(mt_befehl)))
	m_bestimme_topologie();
      else if (!strncmp(befehl, "disab   ", sizeof(mt_befehl))) {
	TEMP.f = stdout;
	*TEMP.name = '\0';
	m_write_alle_bindungen(&TEMP);
      } else if (!strncmp(befehl, "disb    ", sizeof(mt_befehl))) {
	TEMP.f = stdout;
	*TEMP.name = '\0';
	m_write_bindungen(&TEMP);
      } else if (!strncmp(befehl, "discs   ", sizeof(mt_befehl))) {
	TEMP.f = stdout;
	*TEMP.name = '\0';
	m_write_strukturdaten(&TEMP, strukturdaten);
      } else if (!strncmp(befehl, "disad   ", sizeof(mt_befehl))) {
	TEMP.f = stdout;
	*TEMP.name = '\0';
	m_write_alle_distanzen(&TEMP);
      } else if (!strncmp(befehl, "disd    ", sizeof(mt_befehl))) {
	TEMP.f = stdout;
	*TEMP.name = '\0';
	m_write_distanzen(&TEMP);
      } else if (!strncmp(befehl, "disda   ", sizeof(mt_befehl))) {
	TEMP.f = stdout;
	*TEMP.name = '\0';
	m_write_distanzen_und_winkel(&TEMP);
      } else if (!strncmp(befehl, "disdv   ", sizeof(mt_befehl))) {
	TEMP.f = stdout;
	*TEMP.name = '\0';
	m_write_distanzen_und_vektoren(&TEMP);
      } else if (!strncmp(befehl, "disc    ", sizeof(mt_befehl))) {
	TEMP.f = stdout;
	*TEMP.name = '\0';
	m_write_koordinaten(&TEMP);
      } else if (!strncmp(befehl, "dise    ", sizeof(mt_befehl)))
	m_write_entry_liste();
      else if (!strncmp(befehl, "diss    ", sizeof(mt_befehl))) {
	TEMP.f = stdout;
	*TEMP.name = '\0';
	write_st_raumgruppe(&TEMP, rginfo);
      } else if (!strncmp(befehl, "help    ", sizeof(mt_befehl)))
	m_write_hilfe();
      else if (!strncmp(befehl, "quit    ", sizeof(mt_befehl)))
	putchar('\n');
      else if (!strncmp(befehl, "exit    ", sizeof(mt_befehl)))
	putchar('\n');
      else if (!strncmp(befehl, "reacs   ", sizeof(mt_befehl)))
	m_read_strukturdaten();
      else if (!strncmp(befehl, "reafcs  ", sizeof(mt_befehl)))
	m_read_erste_strukturdaten();
      else if (!strncmp(befehl, "reancs  ", sizeof(mt_befehl)))
	m_read_naechste_strukturdaten();
      else if (!strncmp(befehl, "wrie    ", sizeof(mt_befehl)))
	m_write_strukturdaten_karte();
      else if (!strncmp(befehl, "wrics   ", sizeof(mt_befehl)))
	m_write_strukturdaten(&list, strukturdaten);
      else if (!strncmp(befehl, "wrida   ", sizeof(mt_befehl)))
	m_write_distanzen_und_winkel(&list);
      else if (!strncmp(befehl, "wridv   ", sizeof(mt_befehl)))
	m_write_distanzen_und_vektoren(&list);
      else if (!strncmp(befehl, "wris    ", sizeof(mt_befehl)))
	write_st_raumgruppe(&list, rginfo);
      else if (!strncmp(befehl, "wric    ", sizeof(mt_befehl)))
	m_write_koordinaten(&list);
      else if (!strncmp(befehl, "wriad   ", sizeof(mt_befehl)))
	m_write_alle_distanzen(&list);
      else if (!strncmp(befehl, "wrid    ", sizeof(mt_befehl)))
	m_write_distanzen(&list);
      else if (!strncmp(befehl, "wriab   ", sizeof(mt_befehl)))
	m_write_alle_bindungen(&list);
      else if (!strncmp(befehl, "wrib    ", sizeof(mt_befehl)))
	m_write_bindungen(&list);
      else if (!strncmp(befehl, "wriid   ", sizeof(mt_befehl)))
	m_DLSinput();
      else if (!strncmp(befehl, "wriil   ", sizeof(mt_befehl)))
	m_LOADATinput();
      else
	printf(" ERROR  unknown command %.8s\n", befehl);
    }
    putchar('\n');
  } while (strncmp(befehl, "quit    ", sizeof(mt_befehl)) &&
	   strncmp(befehl, "exit    ", sizeof(mt_befehl)));
  printf(
    " ******************************************************************************\n");
  printf(
    " *   end of program  KRIBER                                                   *\n");
  printf(
    " ******************************************************************************\n");
  if (strudat.f != NULL)
    fclose(strudat.f);
  if (symdat.f != NULL)
    fclose(symdat.f);
  if (strudatn.f != NULL)
    fclose(strudatn.f);
  if (distdat.f != NULL)
    fclose(distdat.f);
  if (dlsinp.f != NULL)
    fclose(dlsinp.f);
  if (loadainp.f != NULL)
    fclose(loadainp.f);
  if (list.f != NULL)
    fclose(list.f);
  if (coseq.f != NULL)
    fclose(coseq.f);
  exit(EXIT_SUCCESS);
}



/* End. */
