program KRIB(input,output,symdat,strudat,dlsinp(* ### BK ut *),loadainp
             (* ### BK loadatinput *),list,distdat,
             strudatn(* ### BK ew *),coseq);
(******************************************************************************)
(*                                                                            *)
(*   Program  KRIBER              Version 1.0       (January 1991)            *)
(*   ===============                                                          *)
(*                                                                            *)
(*   Author:   Roland Bialek, Institut fuer Kristallografie und Petrografie   *)
(*             ETH-Zentrum, CH-8092 Zuerich, Switzerland                      *)
(*                                                                            *)
(*   an interactive PASCAL program to                                         *)
(*    - calculate distances and angles                                        *)
(*    - generate input files for the programs DLS-76 and LOADAT (XRS-82)      *)
(*    - calculate coordination sequences and loop configurations              *)
(*                                                                            *)
(*   Program Installation and Modification                                    *)
(*   -------------------------------------                                    *)
(*  The program is written in standard PASCAL and has been implemented and    *)
(*  fully tested on a CYBER 855 running under NOS/VE 1.5.2. It has also been  *)
(*  used successfully on a MicroVax II machine under VMS 5.2                  *)
(*                                                                            *)
(*  There is only one extension to the standard PASCAL used: The identifiers  *)
(*  contain the underscore (_) and the dollar sign ($). If your PASCAL        *)
(*  does not have this extension, you can remove these two signs with your    *)
(*  editor.                                                                   *)
(*                                                                            *)
(*  Input:  The program runs interactively.                                   *)
(*                                                                            *)
(*  Output: The output is written to the terminal screen (longest line is     *)
(*          80 columns wide).                                                 *)
(*                                                                            *)
(*  Files:  SYMDAT, STRUDAT, DISTDAT and COSEQ are input files, DLSINPUT,     *)
(*          LOADATINPUT, LIST and STRUDATNEW are output files.                *)
(*                                                                            *)
(*  Arrays: Following constants can be changed in the constant declaration    *)
(*          part:                                                             *)
(*          maxanzahlatomlagen    :   maximum number of atom positions        *)
(*          maxanzahlbindungen    :   maximum number of bonds per atom        *)
(*          maxanzahlsymeqkarten  :   maximum number of symeq cards           *)
(*          maxindepwinkel        :   maximum number of independent angles    *)
(*          maxindepdistanzen     :   maximum number of independent distances *)
(*          maxanzahlgegwinkel    :   maximum number of prescribed angles     *)
(*          maxanzahlgegdistanzen :   maximum number of prescribed distances  *)
(*          maxanzahldistanzen    :   maximum number of calculated distances  *)
(*                                                                            *)
(*  Modified by Bernt Karasch, modifications marked with '### BK'             *)
(*                                                                            *)
(******************************************************************************)
const maxanzahlatomlagen = 100;
      maxanzahlbindungen = 8;
      maxanzahlsymeqkarten = 100;
      maxindepwinkel = 100;
      maxindepdistanzen = 100;
      maxanzahlgegwinkel = 50;
      maxanzahlgegdistanzen = 50;
      maxanzahldistanzen = 9;

type vektor = array[1..3] of real;
     gittervektor = packed array[1..3] of integer;
     tensor = array[1..3,1..3] of real;
     rotationsmatrix = array[1..3] of array[1..3] of integer;
     symmetrieoperator = record
         r: rotationsmatrix;
         t: vektor;
         end;
     psymmetrieoperator = ^symmetrieoperator;
     st$symop_array = array[1..48] of symmetrieoperator;
     st$kristallsystem = packed array[1..4] of char;
     st$raumgruppename =  packed array[1..30] of char;
     st$raumgruppe = record
         raumgruppenr: integer;
         raumgruppename: st$raumgruppename;
         kristallsystem: st$kristallsystem;
         zaehligkeit: integer;
         symop: array[1..192] of psymmetrieoperator;
         anz_rotmat,anztranslationen: integer;
         translation: array[1..4] of vektor;
         zentrosymm: boolean;
         multtab: array[1..48,1..48] of integer;
         end;
     abkbezeichnung = packed array[1..10] of char;
     string80 = packed array[1..80] of char;
     zellparameter = record
         laenge, winkel: array[1..3] of real;
         end;
     kt$atomlagename = packed array[1..6] of char;
     pkt$atomlage = ^kt$atomlage;
     pkt$atomart = ^kt$atomart;
     kt$atom = record
         dessen_atomart: pkt$atomart;
         elementarzelle: gittervektor;
         end;
     kt$atomart = record
         deren_atomlage: pkt$atomlage;
         atomartnr: integer;
         pkoord: vektor;
         trilinpol: symmetrieoperator;
         bindung: array[1..maxanzahlbindungen] of kt$atom;
         end;
     kt$element = packed array[1..2] of char;
     p_distanzen_tabellen_eintrag = ^distanzen_tabellen_eintrag;
     kt$atomlage = record
         atomlagenr: integer;
         atomlagekoord: vektor;
         symmetrielage: psymmetrieoperator;
         aname: kt$atomlagename;
         element: kt$element;
         koordinationszahl: integer;
         anzatome: integer;
         atom: array[1..192] of pkt$atomart;
         intifa: array[1..192] of integer;
         dist_entry: p_distanzen_tabellen_eintrag;
         grundatom: kt$atom;
         end;
     kt$strukturdaten = record
         name,lit: string80;
         abk: abkbezeichnung;
         zellpar: zellparameter;
         raumgruppe: st$raumgruppename;
         kristallsystem: st$kristallsystem;
         anzatomlagen: integer;
         atomlage: array[1..maxanzahlatomlagen] of pkt$atomlage;
         end;
     distanzen_tabellen_eintrag = record
         laenge: real;
         endatom: kt$atom;
         naechste_distanz: p_distanzen_tabellen_eintrag;
         end;
     tt$koord_sequenz = array[1..10] of integer;
     berechenbare_dinge = (struktur, distanzen, bindungen);
     mt$befehl = packed array[1..8] of char;

var strudat: text;
    symdat: text;
    strudatn(* ### BK ew *): text;
    distdat: text;
    dlsinp(* ### BK ut *): text;
    loadainp(* ### BK loadatinput *): text;
    list: text;
    coseq: text;
    befehl: mt$befehl;
    berechnet: packed array[struktur..bindungen] of boolean;
    strudatnew_offen: boolean;
    st$metrik_tensor: tensor;
    strukturdaten: kt$strukturdaten;
    rginfo: st$raumgruppe;

function skalar_produkt(a,b: vektor): real;

var x: real;
    i,j: integer;

begin
x:= 0;
for i:= 1 to 3 do for j:= 1 to 3 do x:= x + a[i]*st$metrik_tensor[i,j]*b[j];
skalar_produkt:=x;
end;

procedure load_zellparameter(r1,r2,r3,w1,w2,w3: real);

begin
st$metrik_tensor[1,1]:= sqr(r1);
st$metrik_tensor[2,2]:= sqr(r2);
st$metrik_tensor[3,3]:= sqr(r3);
st$metrik_tensor[1,2]:= r1*r2*cos(w3*0.0174533);
st$metrik_tensor[2,3]:= r2*r3*cos(w1*0.0174533);
st$metrik_tensor[3,1]:= r3*r1*cos(w2*0.0174533);
st$metrik_tensor[2,1]:= r1*r2*cos(w3*0.0174533);
st$metrik_tensor[3,2]:= r2*r3*cos(w1*0.0174533);
st$metrik_tensor[1,3]:= r3*r1*cos(w2*0.0174533);
end;

function gleiche_vektoren(v1,v2: vektor): boolean;

begin
gleiche_vektoren:= (abs(v1[1] - v2[1]) < 0.001)
               and (abs(v1[2] - v2[2]) < 0.001)
               and (abs(v1[3] - v2[3]) < 0.001);
end;

function gleiche_rotationsmatrizen(x,y: rotationsmatrix): boolean;

begin
gleiche_rotationsmatrizen:=
  (x[1,1]=y[1,1]) and (x[1,2]=y[1,2]) and (x[1,3]=y[1,3]) and
  (x[2,1]=y[2,1]) and (x[2,2]=y[2,2]) and (x[2,3]=y[2,3]) and
  (x[3,1]=y[3,1]) and (x[3,2]=y[3,2]) and (x[3,3]=y[3,3]);
end;

function gleiche_symmetrieoperatoren(x,y: symmetrieoperator): boolean;

begin
gleiche_symmetrieoperatoren:=
gleiche_rotationsmatrizen(x.r,y.r) and gleiche_vektoren(x.t,y.t);
end;

procedure frac_vektor(va: vektor; var vb: vektor);

var i: integer;

begin
for i:= 1 to 3 do begin
  if va[i]+0.0000001 < 0 then vb[i]:= va[i]-trunc(va[i]+0.0000001)+1
                        else vb[i]:= va[i]-trunc(va[i]+0.0000001) ;
  end;
end;

procedure int_vektor(v: vektor; var  g: gittervektor);

var i: integer;

begin
for i:= 1 to 3 do begin
  if v[i]+0.0000001 < 0 then g[i]:= trunc(v[i]+0.0000001)-1
                        else g[i]:= trunc(v[i]+0.0000001) ;
  end;
end;

procedure write_vektor(var f: text; v: vektor);

begin
write(f,' (',v[1]:7:4,v[2]:8:4,v[3]:8:4,')');
end;

procedure set_einheits_symop(var s: symmetrieoperator);

var i,j: integer;

begin
for i:= 1 to 3 do s.t[i]:= 0;
for i:= 1 to 3 do for j:= 1 to 3 do s.r[i,j]:= 0;
for i:= 1 to 3 do s.r[i,i]:= 1;
end;

procedure mult_rotmat_mit_vektor(r: rotationsmatrix; v: vektor;
                                 var vn: vektor);
begin
vn[1]:= r[1,1]*v[1] + r[1,2]*v[2] + r[1,3]*v[3];
vn[2]:= r[2,1]*v[1] + r[2,2]*v[2] + r[2,3]*v[3];
vn[3]:= r[3,1]*v[1] + r[3,2]*v[2] + r[3,3]*v[3];
end;

procedure mult_symop_mit_vektor(s: symmetrieoperator; v: vektor;
                                var vn: vektor);
begin
vn[1]:= s.t[1] + s.r[1,1]*v[1] + s.r[1,2]*v[2] + s.r[1,3]*v[3];
vn[2]:= s.t[2] + s.r[2,1]*v[1] + s.r[2,2]*v[2] + s.r[2,3]*v[3];
vn[3]:= s.t[3] + s.r[3,1]*v[1] + s.r[3,2]*v[2] + s.r[3,3]*v[3];
end;

procedure mult_rotationsmatrizen(a,b: rotationsmatrix;
                                 var c: rotationsmatrix);

var i,j: integer;

begin
for i:= 1 to 3 do for j:= 1 to 3 do
  c[i,j]:= a[i,1]*b[1,j] + a[i,2]*b[2,j] + a[i,3]*b[3,j];
end;

procedure mult_symmetrieoperatoren(a,b: symmetrieoperator;
                                   var c: symmetrieoperator);

begin
mult_rotationsmatrizen(a.r,b.r,c.r);
mult_symop_mit_vektor(a,b.t,c.t);
end;

procedure subtrahiere_symmetrieoperatoren(x,y: symmetrieoperator;
                                          var z:symmetrieoperator);

var i,j: integer;

begin
for i:= 1 to 3 do for j:= 1 to 3 do z.r[i,j]:= x.r[i,j] - y.r[i,j];
for i:= 1 to 3 do z.t[i]:= x.t[i] - y.t[i];
end;

procedure write_symmetrieoperator(var t: text; x: symmetrieoperator);

var i,j,nn: integer;
    tt: text;
    ersterterm: boolean;

begin
rewrite(tt);
for i:= 1 to 3 do begin
  ersterterm:= true;
  for j:= 1 to 3 do if x.r[i,j] <> 0 then begin
    if x.r[i,j] < 0 then write(tt,'-')
      else if not ersterterm then write(tt,'+');
    if abs(x.r[i,j]) <> 1 then write(tt,abs(x.r[i,j]):1);
    if j=1 then write(tt,'X');
    if j=2 then write(tt,'Y');
    if j=3 then write(tt,'Z');
    ersterterm:= false;
    end;
  if abs(x.t[i]) > 0.001 then begin
    if x.t[i] < 0 then write(tt,'-') else write(tt,'+');
    nn:= 0;
    repeat nn:= nn+1 until abs(x.t[i]*nn - round(x.t[i]*nn)) < 0.001;
    write(tt,round(abs(x.t[i]*nn)):1);
    if nn > 1 then write(tt,'/',nn:1);
    ersterterm:= false;
    end
  else if ersterterm then write(tt,'0');
  if I < 3 then write(tt,',');
  end;
for i:= 1 to 36 do write(tt,' ');
reset(tt);
for i:= 1 to 22 do begin write(t,tt^); get(tt); end;
end;

procedure write_st$raumgruppe(var t : text; rginfo: st$raumgruppe);

var i,j: integer;

begin
with rginfo do begin
  writeln(t,' Symmetry information from file SYMDAT');
  writeln(t,' space group ',raumgruppename);
  writeln(t);
  if anztranslationen > 0 then begin
    write(t,'        ');
    for i:= 1 to anztranslationen do begin
      write(t,'(');
      for j:= 1 to 3 do begin
        if abs(translation[i,j])<0.0001 then write(t,' 0');
        if abs(translation[i,j]-0.5)<0.0001 then write(t,' 1/2');
        if abs(translation[i,j]-(1/3))<0.0001 then write(t,' 1/3');
        if abs(translation[i,j]-(2/3))<0.0001 then write(t,' 2/3');
        if j<>3 then write(t,',');
        end;
      write(t,')+   ');
      end;
    writeln(t);  writeln(t);
    end;
  for i:= 1 to anz_rotmat do begin
    write(t,' (',i:3,') '); write_symmetrieoperator(t,symop[i]^); writeln(t);
    end;
  writeln(t);
  end;
end;

procedure read_symmetrieoperator(var f: text; var symop: symmetrieoperator);

var i1,i2: real;
    vorz: integer;
    i,j,k: integer;
    rr: real;

begin
while f^=' ' do get(f);
with symop do begin
  for i:= 1 to  3 do for j:= 1 to 3 do r[i,j]:=0;
  for i:= 1 to  3 do t[i]:= 0;
  k:= 1;
  repeat
  vorz:= 1;
  if f^=',' then begin
    k:= k+1;
    get(f);
    end;
  if f^='-' then begin
    vorz:= -1;
    get(f);
    end;
  if f^='+' then  get(f);
  if f^ in ['0'..'9'] then begin
    read(f,i1);
    if not eoln(f) then
      if f^='/' then begin
        get(f);
        read(f,i2);
        rr:=i1/i2;
        end
      else rr:= i1
    else rr:= i1;
    end
  else if f^ in ['X','Y','Z'] then rr:= 1;
  rr:= rr*vorz;
  if f^='X' then r[k,1]:= round(rr)
  else if f^='Y' then r[k,2]:= round(rr)
  else if f^='Z' then r[k,3]:= round(rr)
  else t[k]:= rr;
  if f^ in ['X','Y','Z'] then get(f);
  if f^ = ')' then get(f);
  until eoln(f);
  end;
end;

procedure readln_raumgruppename(var t: text; var rn: st$raumgruppename);

var i: integer;

begin
for i:=1 to 30 do rn[i]:= ' ';
while t^=' ' do get(t);
i:= 0;
while not eoln(t) do begin
  i:=i+1;
  read(t,rn[i]);
  if rn[i] in ['a'..'z'] then rn[i]:= chr(ord(rn[i])+ord('A')-ord('a'));
  end;
readln(t);
end;

procedure suche_raumgruppe_entry(var t: text; rn: st$raumgruppename;
                                 var tra: symmetrieoperator);

var i,j: integer;
    rnd: st$raumgruppename;
    tran: symmetrieoperator;
    tt: text;
    b: boolean;

begin
j:= 0;
for i:=1 to 30 do if (rn[i]='(') then j:= i;
if j>0 then begin
  rn[j]:=' ';
  rewrite(tt);
  i:= j;
  while rn[i]<>')' do begin i:= i+1; write(tt,rn[i]) end;
  writeln(tt);
  reset(tt);
  read_symmetrieoperator(tt,tra); readln(tt);
  for i:= j to 30 do rn[i]:=' ';
  end
else set_einheits_symop(tra);
reset(t);
j:= 0;
for i:= 1 to 30 do if rn[i]<>' ' then begin j:= j+1; rn[j]:= rn[i]; end;
for i:= j+1 to 30 do rn[i]:=' ';
for i:= 1 to 30 do rnd[i]:=' ';
repeat
  b:= false;
  repeat
    if eof(t) then b:= true else if t^ = '*' then b:= true else readln(t);
    until b;
  if eof(t) then b:= false else begin
    get(t);  for j:= 1 to 16 do read(t,rnd[j]);
    if rnd = 'END                           ' then b:= false;
    end;
  until (rnd = rn) or (not b);
if not b then begin
  writeln(' ERROR  space group ',rn,' not found => P1');
  rn:='P1                            ';
  suche_raumgruppe_entry(t,rn,tra);
  end;
if t^='=' then begin
  get(t); get(t); get(t);
  readln_raumgruppename(t,rn);
  suche_raumgruppe_entry(t,rn,tran);
  mult_symmetrieoperatoren(tra,tran,tra);
  end
end;

procedure erstelle_multtab(var rginfo: st$raumgruppe);

var i,j: integer;
    r: rotationsmatrix;
    b: boolean;

begin
with rginfo do for i:= 1 to anz_rotmat do for j:= 1 to anz_rotmat do begin
  mult_rotationsmatrizen(symop[i]^.r,symop[j]^.r,r);
  b:= false;
  multtab[i,j]:=anz_rotmat+1;
  repeat
    multtab[i,j]:=multtab[i,j]-1;
    if multtab[i,j]=0 then begin
      writeln(' ERROR  space group is wrong (not complete?)');
      b:= true;
      end
    else if gleiche_rotationsmatrizen(r,symop[multtab[i,j]]^.r) then b:=true;
    until b
  end;
end;

procedure inverser_symmetrieoperator(sa: symmetrieoperator;
                                     var sb: symmetrieoperator);

type augm_matrix = array[1..4,1..4] of real;

var s,ss: augm_matrix;
    i,j,n: integer;
    z,k: array[1..4] of integer;
    b: boolean;

procedure austauschschritt(var s: augm_matrix; x,y: integer);

var i,j: integer;

begin
for i:= 1 to 4 do for j:= 1 to 4 do
  if (i<>x) and (j<>y) then s[i,j]:= s[i,j] - s[x,j]*s[i,y]/s[x,y];
for i:= 1 to 4 do if i<>x then s[i,y]:= s[i,y]/s[x,y];
for j:= 1 to 4 do if j<>y then s[x,j]:= -s[x,j]/s[x,y];
s[x,y]:=1/s[x,y];
end;

begin
for i:= 1 to 3 do for j:= 1 to 3 do s[i,j]:= sa.r[i,j];
for i:= 1 to 3 do s[i,4]:= sa.t[i];
for j:= 1 to 3 do s[4,j]:= 0;
s[4,4]:= 1;
for j:= 1 to 4 do z[j]:=j;
for i:= 1 to 4 do k[i]:=-i;
b:= false;
repeat
  i:= 0; repeat i:= i+1 until k[i]<0;
  j:= 0; repeat j:= j+1 until (z[j]>0) and (s[i,j]<>0);
  austauschschritt(s,i,j);
  n:= k[i]; k[i]:= z[j]; z[j]:= n;
  if (z[1]<0) and (z[2]<0) and (z[3]<0) and (z[4]<0) then b:= true;
  until b;
for i:= 1 to 4 do for j:= 1 to 4 do ss[k[i],-z[j]]:= s[i,j];
for i:= 1 to 3 do for j:= 1 to 3 do sb.r[i,j]:=round(ss[i,j]);
for i:= 1 to 3 do sb.t[i]:= ss[i,4];
end;

procedure read_raumgruppe_entry(var symdat: text; rn: st$raumgruppename;
                                transf: symmetrieoperator;
                                var rginfo: st$raumgruppe);

var i,j,k: integer;
    inv_transf: symmetrieoperator;

procedure berechne_bravaistyp(var rginfo: st$raumgruppe);

begin
with rginfo do begin
 anztranslationen:= 0;
 if raumgruppename[1]='P' then anztranslationen:= 0
 else if raumgruppename[1]='A' then begin
   anztranslationen:= 1;
   translation[1,1]:= 0;   translation[1,2]:= 1/2; translation[1,3]:= 1/2;
   end
 else if raumgruppename[1]='B' then begin
   anztranslationen:= 1;
   translation[1,1]:= 1/2; translation[1,2]:= 0;   translation[1,3]:= 1/2;
   end
 else if raumgruppename[1]='C' then begin
   anztranslationen:= 1;
   translation[1,1]:= 1/2; translation[1,2]:= 1/2; translation[1,3]:= 0;
   end
 else if raumgruppename[1]='F' then begin
   anztranslationen:= 3;
   translation[1,1]:= 0;   translation[1,2]:= 1/2; translation[1,3]:= 1/2;
   translation[2,1]:= 1/2; translation[2,2]:= 0;   translation[2,3]:= 1/2;
   translation[3,1]:= 1/2; translation[3,2]:= 1/2; translation[3,3]:= 0;
   end
 else if raumgruppename[1]='I' then begin
   anztranslationen:= 1;
   translation[1,1]:= 1/2; translation[1,2]:= 1/2; translation[1,3]:= 1/2;
   end
 else if raumgruppename[1]='R' then begin
   if kristallsystem <>'RHO ' then begin
     anztranslationen:= 2;
     translation[1,1]:= 1/3; translation[1,2]:= 2/3; translation[1,3]:= 2/3;
     translation[2,1]:= 2/3; translation[2,2]:= 1/3; translation[2,3]:= 1/3;
     end;
   end
 else writeln(' ERROR  space group ',raumgruppename[1],
              ' has unknown centering');
 end;
end;

procedure spieglesymopansymzent(var rginfo: st$raumgruppe);

var i: integer;
    inv: symmetrieoperator;

begin
set_einheits_symop(inv);
for i:= 1 to 3 do inv.r[i,i]:= -1;
with rginfo do begin
  for i:= 1 to zaehligkeit do begin
    new(symop[zaehligkeit+i]);
    mult_symmetrieoperatoren(inv,symop[i]^,symop[zaehligkeit+i]^);
    end;
  zaehligkeit:= zaehligkeit*2;
  end;
end;

procedure bestimme_setting(rginfo: st$raumgruppe;
                           var kristallsystem: st$kristallsystem);

var n: array[1..3] of boolean;
    rr: array[1..3] of integer;
    i,j: integer;

begin
for i:=1 to 3 do n[i]:= false;
with rginfo do for i:= 1 to zaehligkeit do with symop[i]^ do begin
  for j:= 1 to 3 do rr[j]:=r[1,j]+r[2,j]+r[3,j];
  if (rr[1]<0) and (rr[2]<0) and (rr[3]>0) then n[3]:= true;
  if (rr[1]>0) and (rr[2]>0) and (rr[3]<0) then n[3]:= true;
  if (rr[1]<0) and (rr[2]>0) and (rr[3]<0) then n[2]:= true;
  if (rr[1]>0) and (rr[2]<0) and (rr[3]>0) then n[2]:= true;
  if (rr[1]>0) and (rr[2]<0) and (rr[3]<0) then n[1]:= true;
  if (rr[1]<0) and (rr[2]>0) and (rr[3]>0) then n[1]:= true;
  end;
if (n[1]=false) and (n[2]=false) and (n[3]=true ) then kristallsystem:='MON1';
if (n[1]=false) and (n[2]=true ) and (n[3]=false) then kristallsystem:='MON2';
if (n[1]=true ) and (n[2]=false) and (n[3]=false) then kristallsystem:='MON3';
end;

begin
with rginfo do begin
  raumgruppename:= rn;
  read(symdat,raumgruppenr);
  while symdat^=' ' do get(symdat);
  for i:= 1 to  4 do read(symdat,kristallsystem[i]);
  readln(symdat,i);
  if i=-1 then zentrosymm:= true else zentrosymm:= false;
  anz_rotmat:= 0;
  while symdat^='*' do readln(symdat);
  while symdat^<>'=' do begin
    anz_rotmat:= anz_rotmat + 1;
    new(symop[anz_rotmat]);
    read_symmetrieoperator(symdat,symop[anz_rotmat]^); readln(symdat);
    end;
  zaehligkeit:= anz_rotmat;
  if zentrosymm then spieglesymopansymzent(rginfo);
  inverser_symmetrieoperator(transf,inv_transf);
  for i:= 1 to zaehligkeit do begin
    mult_symmetrieoperatoren(transf,symop[i]^,symop[i]^);
    mult_symmetrieoperatoren(symop[i]^,inv_transf,symop[i]^);
    frac_vektor(symop[i]^.t,symop[i]^.t);
    end;
  berechne_bravaistyp(rginfo);
  anz_rotmat:= zaehligkeit;
  for i:= 1 to anztranslationen do for j:= 1 to zaehligkeit do begin
    new(symop[i*zaehligkeit + j]);
    symop[i*zaehligkeit + j]^:=symop[j]^;
    with symop[i*zaehligkeit + j]^ do
      for k:= 1 to 3 do t[k]:= t[k] + translation[i,k];
    with symop[i*zaehligkeit+j]^ do frac_vektor(t,t);
    end;
  zaehligkeit:= (anztranslationen+1)*zaehligkeit;
  erstelle_multtab(rginfo);
  if kristallsystem='MON ' then bestimme_setting(rginfo,kristallsystem);
  end;
end;

procedure read_raumgruppe(var t: text; rn: st$raumgruppename;
                          var rg: st$raumgruppe);

var tra: symmetrieoperator;

begin
suche_raumgruppe_entry(t,rn,tra);
read_raumgruppe_entry(t,rn,tra,rg);
end;

function polare_richtung(v: vektor; rginfo: st$raumgruppe): boolean;

var b: boolean;
    i: integer;
    v2: vektor;

begin
b:= true;
with rginfo do for i:= 1 to zaehligkeit do begin
  mult_rotmat_mit_vektor(symop[i]^.r,v,v2);
  if not gleiche_vektoren(v2,v) then b:= false;
  end;
polare_richtung:= b;
end;

function gleiche_atome(x,y: kt$atom): boolean;

begin
gleiche_atome:= (x.dessen_atomart = y.dessen_atomart)
                and (x.elementarzelle[1] = y.elementarzelle[1])
                and (x.elementarzelle[2] = y.elementarzelle[2])
                and (x.elementarzelle[3] = y.elementarzelle[3]);
end;

procedure berechne_atom_trilinpol(a: kt$atom; var ssy: symmetrieoperator);

var i: integer;

begin
ssy:= a.dessen_atomart^.trilinpol;
for i:=1 to 3 do ssy.t[i]:= ssy.t[i] + a.elementarzelle[i];
end;

procedure berechne_symmetrielage(rginfo: st$raumgruppe; v: vektor;
                                     var s : symmetrieoperator);

type sympar = array[0..192] of array [1..4] of real;

var i,j,k,n: integer;
    w: vektor;
    g: gittervektor;
    a: sympar;
    ss: symmetrieoperator;

procedure simplex(sy: sympar; var y: symmetrieoperator);

type reihe = array[1..3] of integer;
     matrix = array[1..3,1..4] of real;

var yy: matrix;
    n : reihe;
    ssy: sympar;
    i,j: integer;
    loesung_io: boolean;

procedure austauschschritt(var s:sympar; x,y: integer);

var i,j: integer;

begin
for i:= 1 to 192 do for j:= 1 to 4 do
  if (i<>x) and (j<>y) then s[i,j]:= s[i,j] - s[x,j]*s[i,y]/s[x,y];
for j:= 1 to 4 do if j<>y then s[x,j]:= -s[x,j]/s[x,y];
for i:= 1 to 192 do s[i,y]:= 0;
end;

procedure sym(var sy: sympar; n: reihe; var yy: matrix);

var i,j,m: integer;
    pa: array[1..3] of integer;

begin
for i:= 1 to 4 do sy[0,i]:= 0;
for i:= 1 to 3 do pa[i]:= 0;
for i:= 1 to 3 do begin
  m:= 192;
  repeat m:= m-1
  until ((abs(sy[m,n[i]])>0.00001)
    and (m<>pa[1]) and (m<>pa[2]) and (m<>pa[3])) or (m=0);
  if m <> 0 then begin
    austauschschritt(sy,m,n[i]);
    pa[n[i]]:=m;
    end;
  end;
for i:= 1 to 3 do
  if pa[i] <> 0 then for j:= 1 to 4 do yy[i,j]:= sy[pa[i],j]
  else begin
    for j:= 1 to 4 do yy[i,j]:= 0;
    yy[i,i]:= 1;
    end;
end;

begin
ssy:= sy;
n[1]:=1; n[2]:=2; n[3]:=3;
sym(ssy,n,yy);
loesung_io:=true;
for i:= 1 to 3 do for j:= 1 to 3 do
  if abs(round(yy[i,j])-yy[i,j]) > 0.00001 then loesung_io:=false;
for i:= 1 to 3 do
  if (abs(yy[i,i]) > 0.00001) and (abs(yy[i,4]) > 0.00001) then
    loesung_io:=false;
if not loesung_io then begin
  ssy:= sy;
  n[1]:=2; n[2]:=1; n[3]:=3;
  sym(ssy,n,yy);
  end;
for i:= 1 to 3 do for j:= 1 to 3 do y.r[i,j]:= round(yy[i,j]);
for i:= 1 to 3 do y.t[i]:= yy[i,4];
end;

begin
n:= 0;
for i:= 1 to 192 do for j:= 1 to 4 do a[i,j]:= 0;
for i:= 2 to rginfo.zaehligkeit do begin
  mult_symop_mit_vektor(rginfo.symop[i]^,v,w);
  int_vektor(w,g);
  frac_vektor(w,w);
  if gleiche_vektoren(v,w) then begin
    ss:= rginfo.symop[i]^;
    for j:= 1 to 3 do ss.t[j]:= ss.t[j] - g[j];
    subtrahiere_symmetrieoperatoren(rginfo.symop[1]^,ss,ss);
    n:= n+1;
    for j:= 1 to 3 do begin
      for k:= 1 to 3 do a[n*3-3+j,k]:= ss.r[j,k];
      a[n*3-3+j,4]:= ss.t[j];
      end;
    end;
  end;
if n > 0 then simplex(a,s) else s:= rginfo.symop[1]^;
end;

procedure berechne_atomlagekoordinaten(rginfo: st$raumgruppe;
                                       atomlage: pkt$atomlage);

var i,j: integer;
    g: gittervektor;
    s: symmetrieoperator;

begin
with atomlage^ do begin
  frac_vektor(atomlagekoord,atomlagekoord);
  new(symmetrielage);
  berechne_symmetrielage(rginfo,atomlagekoord,symmetrielage^);
  int_vektor(symmetrielage^.t,g);
  frac_vektor(symmetrielage^.t,symmetrielage^.t);
  for i:= 1 to 3 do atomlagekoord[i]:= atomlagekoord[i] - g[i];
  anzatome:= 0;
  for i:= 1 to rginfo.zaehligkeit do begin
    mult_symmetrieoperatoren(rginfo.symop[i]^,symmetrielage^,s);
    frac_vektor(s.t,s.t);
    intifa[i]:=anzatome+1;
    for j:= 1 to anzatome do
      if gleiche_symmetrieoperatoren(s,atom[j]^.trilinpol) then intifa[i]:=j;
    if intifa[i]>anzatome then begin
      anzatome:= anzatome + 1;
      new(atom[anzatome]);
      with atom[anzatome]^ do begin
        trilinpol:= s;
        atomartnr:= anzatome;
        deren_atomlage:= atomlage;
        for j:= 1 to maxanzahlbindungen do bindung[j].dessen_atomart:= nil;
        end;
      end;
    end;
  for i:= 1 to anzatome do with atom[i]^ do begin
    mult_symop_mit_vektor(trilinpol,atomlagekoord,pkoord);
    int_vektor(pkoord,g);
    frac_vektor(pkoord,pkoord);
    for j:= 1 to 3 do trilinpol.t[j]:= trilinpol.t[j] - g[j];
    end;
  atomlagekoord:= atom[1]^.pkoord;
  end;
end;

procedure write_atom_trilinpol(var t: text; a: kt$atom);

var ssy: symmetrieoperator;

begin
berechne_atom_trilinpol(a,ssy);
write_symmetrieoperator(t,ssy);
end;

procedure alg_symmetrieoperation(s: symmetrieoperator; a: kt$atom;
                                 var b: kt$atom);

var i: integer;
    p: pkt$atomlage;
    sa,sr: symmetrieoperator;
    ga,gr: gittervektor;

begin
p:= a.dessen_atomart^.deren_atomlage;
berechne_atom_trilinpol(a,sa);
mult_symmetrieoperatoren(s,sa,sa);
int_vektor(sa.t,ga);
frac_vektor(sa.t,sa.t);
i:= 0;
repeat
  i:= i+1;
  sr:= p^.atom[i]^.trilinpol;
  int_vektor(sr.t,gr);
  frac_vektor(sr.t,sr.t);
  until gleiche_symmetrieoperatoren(sr,sa);
b.dessen_atomart:= p^.atom[i];
for i:= 1 to 3 do b.elementarzelle[i]:= ga[i] - gr[i];
end;

procedure berechne_atom_koordinaten(a: kt$atom; var v: vektor);

var i: integer;

begin
for i:=1 to 3 do v[i]:= a.elementarzelle[i] + a.dessen_atomart^.pkoord[i];
end;

procedure write_atomart_name(var f: text; a: kt$atomart);

var i: integer;

begin
write(f,' ');
for i:= 1 to 4 do write(f,a.deren_atomlage^.aname[i]);
write(f,'(',a.atomartnr:3,')');
end;

procedure write_atom_name(var f: text; a: kt$atom);

begin
write_atomart_name(f,a.dessen_atomart^);
with a do
  write(f,'(',elementarzelle[1]:2,elementarzelle[2]:2,elementarzelle[3]:2,')');
end;

procedure write_atom_koordinaten(var t: text; a: kt$atom);

var v: vektor;

begin
berechne_atom_koordinaten(a,v);
write_vektor(t,v);
end;

procedure write_strukturdaten(var t : text; strukturdaten: kt$strukturdaten);

var i: integer;

begin
writeln(t);
with strukturdaten do begin
  writeln(t,' Name: ',name);
  writeln(t,' Lit: ',lit);
  writeln(t);
  with zellpar do begin
    writeln(t,' Cell parameters:  a = ',laenge[1]:6:3,
              '  b = ',laenge[2]:6:3,'  c = ',laenge[3]:6:3);
    writeln(t,'                   alpha = ',winkel[1]:6:3,
              '  beta = ',winkel[2]:6:3,'  gamma = ',winkel[3]:6:3);
    end;
  writeln(t);
  writeln(t,' Space group: ',raumgruppe);
  writeln(t);
  writeln(t,' Atoms with coordination number and atoms per unit cell  ');
  for i:= 1 to anzatomlagen do with atomlage[i]^ do begin
    write(t,' (',i:2,')   ',aname:6);
    write_atom_koordinaten(t,grundatom);
    write(t,koordinationszahl:4,anzatome:4,'   ');
    write_atom_trilinpol(t,grundatom); writeln(t);
    end;
  end;
end;

procedure write_strukturdaten_karte(var t : text; struda: kt$strukturdaten);

var i: integer;

begin
with struda do begin
  writeln(t,'*XXX');
  writeln(t,name);
  writeln(t,lit);
  writeln(t,raumgruppe);
  with zellpar do writeln(t,laenge[1]:8:3,laenge[2]:8:3,laenge[3]:8:3
                           ,winkel[1]:8:3,winkel[2]:8:3,winkel[3]:8:3);
  for i:= 1 to anzatomlagen do with atomlage[i]^ do begin
    write(t,aname:6,atomlagekoord[1]:9:5,atomlagekoord[2]:9:5,
                      atomlagekoord[3]:9:5);
    if koordinationszahl > 0 then write(t,koordinationszahl:8); writeln(t);
    end;
  writeln(t,'------------------------------');
  end;
end;

procedure write_atomlage_koordinaten(var f: text; atomlage: kt$atomlage);

var i: integer;

begin
writeln(f);
writeln(f,' Coordinates for atom ',atomlage.aname);
for i:= 1 to atomlage.anzatome do with atomlage.atom[i]^ do begin
  write_atomart_name(f,atomlage.atom[i]^);
  write(f,'  '); write_vektor(f,pkoord);
  write(f,'    '); write_symmetrieoperator(f,trilinpol); writeln(f);
  end;
end;

procedure read_string80(var t:text; var s: string80);

var i: integer;

begin
for i:= 1 to 80 do s[i]:=' ';
i:=0; while not eoln(t) do begin i:=i+1; read(t,s[i]) end; readln(t);
end;

procedure set_zellparameter(r1,r2,r3,r4,r5,r6: real; var z: zellparameter);

begin
with z do begin
  laenge[1]:=r1;  laenge[2]:=r2;  laenge[3]:=r3;
  winkel[1]:=r4;  winkel[2]:=r5;  winkel[3]:=r6;
  end;
load_zellparameter(r1,r2,r3,r4,r5,r6);
end;

procedure read_zellparameter(var t: text; var zellpar: zellparameter;
                             var kristallsystem: st$kristallsystem);

var i,ii: integer;
    r: array[1..6] of real;

begin
if not (t^ in ['0'..'9','-','+',' ']) then
  for i:= 1 to 4 do read(t,kristallsystem[i]);
ii:= 0;
while not eoln(t) do begin
  ii:=ii+1; read(t,r[ii]);
  while (not eoln(t)) and (t^ = ' ') do get(t);
  end;
readln(t);
if ii=6 then
  set_zellparameter(r[1],r[2],r[3],r[4],r[5],r[6],zellpar)
else if (kristallsystem = 'CUB ') and (ii= 1 ) then
  set_zellparameter(r[1],r[1],r[1], 90 , 90 , 90 ,zellpar)
else if (kristallsystem = 'TET ') and (ii= 2 ) then
  set_zellparameter(r[1],r[1],r[2], 90 , 90 , 90 ,zellpar)
else if (kristallsystem = 'ORT ') and (ii= 3 ) then
  set_zellparameter(r[1],r[2],r[3], 90 , 90 , 90 ,zellpar)
else if (kristallsystem = 'TRIG') and (ii= 2 ) then
  set_zellparameter(r[1],r[1],r[2], 90 , 90 , 120,zellpar)
else if (kristallsystem = 'HEX ') and (ii= 2 ) then
  set_zellparameter(r[1],r[1],r[2], 90 , 90 , 120,zellpar)
else if (kristallsystem = 'MON3') and (ii= 4 ) then
  set_zellparameter(r[1],r[2],r[3],r[4], 90 , 90 ,zellpar)
else if (kristallsystem = 'MON1') and (ii= 4 ) then
  set_zellparameter(r[1],r[2],r[3], 90 , 90 ,r[4],zellpar)
else if (kristallsystem = 'MON2') and (ii= 4 ) then
  set_zellparameter(r[1],r[2],r[3], 90 ,r[2], 90 ,zellpar)
else if (kristallsystem = 'RHO ') and (ii= 2 ) then
  set_zellparameter(r[1],r[1],r[1],r[2],r[2],r[2],zellpar)
else writeln(' ERROR  cannot read the cell parameters');
end;

procedure suche_naechsten_eintrag(var t: text; var abk: abkbezeichnung;
                                  var b: boolean);

var i: integer;

begin
b:= false;
for i:= 1 to 10 do abk[i]:= ' ';
repeat if eof(t) then b:= true else if t^ = '*' then b:= true else readln(t);
  until b;
if eof(t) then b:= false else begin
  get(t);
  i:= 0; while (not eoln(t)) and (i<10) do begin i:= i+1; read(t,abk[i]) end;
  readln(t);
  if abk[10]<>' ' then writeln(' WARNING  entry name on file STRUDAT too long');
  if (abk = 'END       ') or (abk = 'end       ') then b:= false;
  end;
end;

procedure suche_strukturdaten_eintrag(var t: text; gegabk: abkbezeichnung;
                                      var b: boolean);

var abk: abkbezeichnung;

begin
repeat suche_naechsten_eintrag(t,abk,b) until (abk = gegabk) or (not b);
if b then writeln(output,' ... entry ',abk,' found')
     else writeln(output,' ERROR  entry ',gegabk,' not found on file STRUDAT');
end;

procedure erzeuge_atomlage(e: kt$element; n: integer; koord: vektor;
                           kz: integer; var a: pkt$atomlage);

begin
new(a);
with a^ do begin
  atomlagekoord:= koord;
  element:=e;
  aname:= '      '; aname[1]:=e[1]; aname[2]:=e[2];
  koordinationszahl:=kz;
  dist_entry:= nil;
  if n>0 then if n<10 then aname[3]:= chr(n+ord('1')-1)
                      else begin
                           aname[3]:= chr((n div 10)+ord('1')-1);
                           aname[4]:= chr((n mod 10)+ord('1')-1);
                        end;
  if (aname[2]=' ') and (aname[3]<>' ') then begin
    aname[2]:=aname[3]; aname[3]:=aname[4]; aname[4]:=' ';
    end;
  end;
berechne_atomlagekoordinaten(rginfo,a);
a^.grundatom.elementarzelle[1]:=0;
a^.grundatom.elementarzelle[2]:=0;
a^.grundatom.elementarzelle[3]:=0;
a^.grundatom.dessen_atomart:= a^.atom[1];
end;

procedure lade_atomlage(var neue_atomlage: pkt$atomlage;
                        var struda: kt$strukturdaten; var b: boolean);

var i,j: integer;

begin
with struda do if anzatomlagen < maxanzahlatomlagen then begin
  b:= true;
  with neue_atomlage^ do for i:= 1 to anzatomlagen do begin
    for j:= 1 to atomlage[i]^.anzatome do
      if gleiche_vektoren(atomlagekoord,atomlage[i]^.atom[j]^.pkoord) then begin
        write(' ERROR  atom position ',aname,
              ' has the same coordinates as atom');
        write_atomart_name(output,atomlage[i]^.atom[j]^); writeln;
        writeln('        and therefore has not been loaded');
        b:= false;
        end;
    if aname=atomlage[i]^.aname then
      writeln(' WARNING  atom position ',aname,
              ' has same name as another atom position');
    end;
  if b then begin
    anzatomlagen:= anzatomlagen +1;
    atomlage[anzatomlagen]:= neue_atomlage;
    atomlage[anzatomlagen]^.atomlagenr:=anzatomlagen;
    end;
  end
else begin
  b:= false;
  writeln(' ERROR  too many atom positions.',
          ' New atom position cannot be loaded.');
  writeln('        The program is compiled for a maximum of ',
          maxanzahlatomlagen:1,' atom positions.');
  writeln('        Increase constant "maxanzahlatomlagen" and compile again.');
  end;
end;

procedure read_atomlage(var t: text; var atomlage: pkt$atomlage);

var element: kt$element;
    atom_label, koordinationszahl: integer;
    koord: vektor;

begin
if t^ in ['A'..'Z','a'..'z'] then read(t,element[1]) else element[1]:=' ';
if t^ in ['A'..'Z','a'..'z'] then read(t,element[2]) else element[2]:=' ';
if t^ in ['1'..'9','0'] then read(t,atom_label) else atom_label:=0;
read(t,koord[1],koord[2],koord[3]);
while (not eoln(t)) and (t^ = ' ') do get(t);
if not eoln(t) then read(t,koordinationszahl) else begin
  koordinationszahl:=0;
  if ((element='SI') or (element='Si')) then koordinationszahl:=4;
  if ((element='AL') or (element='Al')) then koordinationszahl:=4;
  if (element='P ') then koordinationszahl:=4;
  if (element='T ') then koordinationszahl:=4;
  if (element='O ') then koordinationszahl:=2;
  end;
readln(t);
erzeuge_atomlage(element,atom_label,koord,koordinationszahl,atomlage);
end;

procedure read_strukturdaten(var t: text; var strukturdaten: kt$strukturdaten;
                             frage: boolean);

var neue_atomlage: pkt$atomlage;
    b: boolean;

begin
with strukturdaten do begin
  if frage then write(output,' Name: '); read_string80(t,name);
  if frage then write(output,' Lit:  '); read_string80(t,lit);
  if frage then write(' Space group: '); readln_raumgruppename(t, raumgruppe);
  read_raumgruppe(symdat,raumgruppe,rginfo);
  kristallsystem:= rginfo.kristallsystem;
  if frage then write(' Cell parameters: ');
     read_zellparameter(t,zellpar, kristallsystem);
  if frage then begin
    writeln(' Add atom position,  end with ''*'' ');
    writeln(' label (element symbol + number)   coordinates   coordination ',
            'number');
    end;
  anzatomlagen:= 0;
  if frage then write(' Atom',(anzatomlagen+1):3,': ');
  while t^ in ['A'..'Z','a'..'z'] do begin
    read_atomlage(t,neue_atomlage);
    with neue_atomlage^ do frac_vektor(atomlagekoord,atomlagekoord);
    lade_atomlage(neue_atomlage,strukturdaten,b);
    if frage then write(' Atom',(anzatomlagen+1):3,': ');
    end;
  end;
if frage then readln(t);
end;

function atom_hat_gleiches_element(a: kt$atom; e: kt$element): boolean;

begin
with a.dessen_atomart^.deren_atomlage^ do
  atom_hat_gleiches_element:= (element[1] = e[1]) and (element[2] = e[2]);
end;

function symmetrieaequivalente_atome(a,b: kt$atom): boolean;

begin
symmetrieaequivalente_atome:=
   (a.dessen_atomart^.deren_atomlage = b.dessen_atomart^.deren_atomlage);
end;

procedure welchsymop(a,b: kt$atom; rginfo: st$raumgruppe;
                     var anzsymop: integer; var sr: st$symop_array);

var i,j: integer;
    c: kt$atom;
    sa,sb,sc: symmetrieoperator;
    gb,gc: gittervektor;

begin
anzsymop:=0;
if symmetrieaequivalente_atome(a,b) then begin
  berechne_atom_trilinpol(a,sa);
  berechne_atom_trilinpol(b,sb);
  int_vektor(sb.t,gb);
  frac_vektor(sb.t,sb.t);
  with rginfo do for i:= 1 to zaehligkeit do begin
    mult_symmetrieoperatoren(symop[i]^,sa,sc);
    int_vektor(sc.t,gc);
    frac_vektor(sc.t,sc.t);
    if gleiche_symmetrieoperatoren(sc,sb) then begin
      anzsymop:= anzsymop + 1;
      sr[anzsymop]:=  rginfo.symop[i]^;
      for j:= 1 to 3 do sr[anzsymop].t[j]:= sr[anzsymop].t[j] - gc[j] + gb[j];
      end;
    end;
  end;
end;

function sym_aequival_distanzen(a1,a2,a3,a4: kt$atom): boolean;

var symop: st$symop_array;
    anzsymop, i: integer;
    a3n: kt$atom;
    b: boolean;

begin
b:= false;
welchsymop(a2,a4,rginfo,anzsymop, symop);
for i:= 1 to anzsymop do begin
  alg_symmetrieoperation(symop[i],a1,a3n);
  if gleiche_atome(a3n,a3) then b:= true;
  end;
sym_aequival_distanzen:= b;
end;

function sym_aequival_winkel(a1,a2,a3,a4,a5,a6: kt$atom): boolean;

var symop: st$symop_array;
    anzsymop, i: integer;
    a4n,a6n: kt$atom;
    b: boolean;

begin
b:= false;
welchsymop(a2,a5,rginfo,anzsymop, symop);
for i:= 1 to anzsymop do begin
  alg_symmetrieoperation(symop[i],a1,a4n);
  alg_symmetrieoperation(symop[i],a3,a6n);
  if gleiche_atome(a4n,a4) and gleiche_atome(a6n,a6) then b:= true;
  end;
sym_aequival_winkel:= b;
end;

procedure delete_atomlage(var atomlage: kt$atomlage;
                          var struda: kt$strukturdaten);

var j,n: integer;

begin
n:= atomlage.atomlagenr;
atomlage.atomlagenr:= 0;
with struda do begin
  anzatomlagen:= anzatomlagen-1;
  for j:= n to anzatomlagen do begin
    atomlage[j]:= atomlage[j+1];
    atomlage[j]^.atomlagenr:= j;
    end;
  end;
end;

procedure read_atomlage_bez(var f: text; var pala:pkt$atomlage);

var i: integer;
    nn: packed array[1..6] of char;

begin
pala:=nil;
while (not eoln(f)) and (f^=' ') do get(f);
if not eoln(f) then begin
  for i:=1 to 6 do nn[i]:=' ';
  i:= 0; while (f^<>' ') and (f^<>'(') and (not eoln(f)) do
    begin i:= i+1; read(nn[i]); end;
  with strukturdaten do for i:= 1 to anzatomlagen do begin
    if nn=atomlage[i]^.aname then pala:= atomlage[i];
    end;
  end;
if pala=nil then writeln(' ERROR  atom position ',nn,' not found')
end;

procedure readln_atom(var t: text; var a: kt$atom);

var ss,ssy: symmetrieoperator;
    i,j,n: integer;
    pala: pkt$atomlage;
    v: gittervektor;
    b: boolean;

begin
a.dessen_atomart:= nil;
read_atomlage_bez(t,pala);
if pala<>nil then with pala^ do begin
  a.dessen_atomart:= atom[1];
  a.elementarzelle[1]:=0;
  a.elementarzelle[2]:=0;
  a.elementarzelle[3]:=0;
  while (not eoln(t)) and (t^ = ' ') do get(t);
  if not eoln(t) then begin
    if t^='(' then begin
      while (not eoln(t)) and ((t^=' ') or (t^=')') or (t^='(')) do get(t);
      if not eoln(t) then read(n);
      if (n>0) and (n<=a.dessen_atomart^.deren_atomlage^.anzatome) then begin
        a.dessen_atomart:= atom[n];
        while (not eoln(t)) and ((t^=' ') or (t^=')') or (t^='(')) do get(t);
        if not eoln(t) then
          read(t,a.elementarzelle[1],a.elementarzelle[2],a.elementarzelle[3]);
        end
      else begin
        write(' ERROR  there is no atom ');
        writeln(a.dessen_atomart^.deren_atomlage^.aname,' (',n:1,')');
        a.dessen_atomart:= nil;
        end;
      end
    else begin
      read_symmetrieoperator(t,ss);
      int_vektor(ss.t,a.elementarzelle);
      frac_vektor(ss.t,ss.t);
      b:= true;
      for i:= 1 to anzatome do begin
        ssy:=atom[i]^.trilinpol;
        int_vektor(ssy.t,v);
        frac_vektor(ssy.t,ssy.t);
        if gleiche_symmetrieoperatoren(ss,ssy) and b then begin
          a.dessen_atomart:= atom[i];
          for j:= 1 to 3 do a.elementarzelle[j]:=a.elementarzelle[j]-v[j];
          b:= false;
          end;
        end;
      if b then begin
        write(' ERROR  atom ',a.dessen_atomart^.deren_atomlage^.aname,
              ' with symmetry operator ');
        write_symmetrieoperator(output,ss); writeln(' not found');
        a.dessen_atomart:= nil;
        end;
      end;
    end;
  end;
if a.dessen_atomart = nil then writeln(' ERROR  atom not found');
readln(t);
end;

function koordinationszahl_des_atoms(a: kt$atom): integer;

begin
koordinationszahl_des_atoms:=
  a.dessen_atomart^.deren_atomlage^.koordinationszahl;
end;

procedure verschiebe_atome_leicht(var atomlage: kt$atomlage);

var i: integer;
    ran: real;

begin
ran:=0.23456789;
with atomlage do begin
  ran:=abs(ran+atomlagekoord[1])-trunc(abs(ran+atomlagekoord[1]));
  atomlagekoord[1]:= atomlagekoord[1] + (2*ran-1)/1000;
  ran:=abs(ran+atomlagekoord[2])-trunc(abs(ran+atomlagekoord[2]));
  atomlagekoord[2]:= atomlagekoord[2] + (2*ran-1)/1000;
  ran:=abs(ran+atomlagekoord[3])-trunc(abs(ran+atomlagekoord[3]));
  atomlagekoord[3]:= atomlagekoord[3] + (2*ran-1)/1000;
  mult_symop_mit_vektor(symmetrielage^,atomlagekoord,atomlagekoord);
  for i:= 1 to anzatome do begin
    mult_symop_mit_vektor(atom[i]^.trilinpol,atomlagekoord,atom[i]^.pkoord);
    end;
  end;
end;

function volumen(a,b,c,aa,bb,cc:real):real;

var acos,bcos,ccos,v: real;

begin
acos:= cos(aa*0.0174533); if abs(acos)<0.00000001 then acos:=0;
bcos:= cos(bb*0.0174533); if abs(bcos)<0.00000001 then bcos:=0;
ccos:= cos(cc*0.0174533); if abs(ccos)<0.00000001 then ccos:=0;
ccos:= cos(cc*0.0174533); if abs(ccos)<0.00000001 then ccos:=0;
v:= 1-sqr(acos)-sqr(bcos)-sqr(ccos)+2*acos*bcos*ccos;
if abs(v)<0.00000001 then v:=0;
volumen:= a*b*c*sqrt(v);
end;

procedure berechne_distanzen_algebraisch(rginfo: st$raumgruppe);

var i: integer;
    s: symmetrieoperator;

begin
writeln('  Atom - Atom      Distance');
with rginfo do for i:= 2 to zaehligkeit do begin
  subtrahiere_symmetrieoperatoren(symop[1]^,symop[i]^,s);
  write('   1     ',i:2,'     ');
  write_symmetrieoperator(output,s); writeln;
  end
end;

procedure berechne_distanzen(zatom: kt$atom; strukturdaten: kt$strukturdaten;
                             max_anzahl: integer; max_laenge: real);

var ii,i,j,k,m,n,o,me,ne,oe: integer;
    vv: vektor;
    b: boolean;
    hh,neue_distanz,neue_dist_tab: p_distanzen_tabellen_eintrag;
    vol: real;

begin
with strukturdaten.zellpar do begin
  vol:= volumen(laenge[1],laenge[2],laenge[3],winkel[1],winkel[2],winkel[3]);
  me:= trunc(max_laenge*laenge[2]*laenge[3]
       *sqrt(1-sqr(cos(winkel[1]*0.0174533)))/vol) + 1;
  ne:= trunc(max_laenge*laenge[1]*laenge[3]
       *sqrt(1-sqr(cos(winkel[2]*0.0174533)))/vol) + 1;
  oe:= trunc(max_laenge*laenge[1]*laenge[2]
       *sqrt(1-sqr(cos(winkel[3]*0.0174533)))/vol) + 1;
  end;
neue_dist_tab:= nil;
new(neue_distanz);
max_laenge:= sqr(max_laenge);
ii:= 0;
for j:= 1 to strukturdaten.anzatomlagen do with strukturdaten.atomlage[j]^ do
  for k:= 1 to anzatome do
    for m:= -me to me do for n:= -ne to ne do for o:= -oe to oe do begin
      vv[1]:= atom[k]^.pkoord[1] + m - zatom.dessen_atomart^.pkoord[1];
      vv[2]:= atom[k]^.pkoord[2] + n - zatom.dessen_atomart^.pkoord[2];
      vv[3]:= atom[k]^.pkoord[3] + o - zatom.dessen_atomart^.pkoord[3];
      neue_distanz^.laenge:=skalar_produkt(vv,vv);
      with neue_distanz^.endatom do begin
        elementarzelle[1]:= m;
        elementarzelle[2]:= n;
        elementarzelle[3]:= o;
        dessen_atomart:=atom[k];
        end;
      if (not gleiche_atome(zatom,neue_distanz^.endatom))
         and (neue_distanz^.laenge <= max_laenge) then begin
        b:= false;
        if neue_dist_tab <> nil then
          if neue_dist_tab^.laenge < neue_distanz^.laenge then begin
            hh:=neue_dist_tab;
              repeat
              if hh^.naechste_distanz = nil then b:= true
              else if hh^.naechste_distanz^.laenge > neue_distanz^.laenge then
                                      b:= true
              else hh:= hh^.naechste_distanz;
              until b;
            neue_distanz^.naechste_distanz:= hh^.naechste_distanz;
            hh^.naechste_distanz:= neue_distanz;
            end;
        if b=false then begin
          neue_distanz^.naechste_distanz:=neue_dist_tab;
          neue_dist_tab:= neue_distanz;
          end;
        ii:= ii+1;
        if ii > max_anzahl then begin
          hh:= neue_dist_tab;
          for i:= 2 to max_anzahl do hh:= hh^.naechste_distanz;
          neue_distanz:= hh^.naechste_distanz;
          hh^.naechste_distanz:= nil;
          end
        else new(neue_distanz);
        end;
      end;
hh:= neue_dist_tab;
while hh <> nil do begin
  hh^.laenge:= sqrt(hh^.laenge);
  hh:= hh^.naechste_distanz;
  end;
zatom.dessen_atomart^.deren_atomlage^.dist_entry:=neue_dist_tab;
end;

procedure write_distanzen(var t : text; a: kt$atom);

var hh: p_distanzen_tabellen_eintrag;
    i,anzs: integer;
    symop: st$symop_array;
    aa: kt$atom;

begin
writeln(t);
write(t,' Distances from atom '); write_atom_name(t,a); writeln(t,'  to');
welchsymop(a.dessen_atomart^.deren_atomlage^.grundatom,a,rginfo,anzs, symop);
hh:= a.dessen_atomart^.deren_atomlage^.dist_entry; i:=0;
while hh <> nil do with hh^ do begin
  i:= i+1; write(t,' (',i:1,')   -');
  if anzs>0 then alg_symmetrieoperation(symop[1],endatom,aa) else aa:=endatom;
  write_atom_name(t,aa); writeln(t,laenge:11:4); hh:= naechste_distanz;
  end;
end;

procedure berechne_nte_distanz(a: kt$atom; n: integer;
                               var d: real; var b: kt$atom);

var hh: p_distanzen_tabellen_eintrag;
    i: integer;

begin
b.dessen_atomart:= nil; d:= 0;
hh:= a.dessen_atomart^.deren_atomlage^.dist_entry; i:=0;
while (hh <> nil) and (i<n) do begin
  i:= i+1;
  b:= hh^.endatom; d:= hh^.laenge; hh:= hh^.naechste_distanz;
  end;
if i<>n then b.dessen_atomart:= nil;
if b.dessen_atomart = nil then
  writeln(' ERROR  distance no. ',n:1,' has not been calculated');
end;

function distanz_zwischen_atome(a,b: kt$atom): real;

var v,va: vektor;
    i: integer;

begin
berechne_atom_koordinaten(a,va);
berechne_atom_koordinaten(b,v);
for i:= 1 to 3 do v[i]:= v[i] - va[i];
distanz_zwischen_atome:= sqrt(skalar_produkt(v,v));
end;

function winkel_zwischen_atome(a,b,c: kt$atom): real;

var va,vb,vc: vektor;
    i: integer;
    cn,cz,cosw,sin2w: real;

begin
berechne_atom_koordinaten(a,va);
berechne_atom_koordinaten(b,vb);
berechne_atom_koordinaten(c,vc);
for i:= 1 to 3 do va[i]:= va[i] - vb[i];
for i:= 1 to 3 do vc[i]:= vc[i] - vb[i];
cn:=skalar_produkt(va,vc);
cz:=sqrt(skalar_produkt(va,va)*skalar_produkt(vc,vc));
if cz=0 then cosw:=1 else cosw:=cn/cz;
if cosw > 1.0 then cosw:= 1.0;
if abs(cosw)< 0.0000000001 then winkel_zwischen_atome:= 90
  else begin
    sin2w:=1 - sqr(cosw);
    if sin2w<0 then sin2w:=0;
    if cosw > 0 then
      winkel_zwischen_atome:= arctan(abs(sqrt(sin2w)/cosw))*57.29577951
    else winkel_zwischen_atome:= 180 -
                                 arctan(abs(sqrt(sin2w)/cosw))*57.29577951;
    end;
end;

procedure write_distanzen_und_winkel(var t : text; a:kt$atom);

var hh1,hh2: p_distanzen_tabellen_eintrag;
    i: integer;
    w: real;

begin
write(t,' Distances and angles for atom '); write_atom_name(t,a);
writeln(t);
writeln(t,'                                       ',
          '(1)      (2)      (3)      (4)      (5)');
i:= 0;
hh1:= a.dessen_atomart^.deren_atomlage^.dist_entry;
while (hh1 <> nil) and (i < 6) do begin
  i:= i + 1;
  write(t,' (',i:1,')   -');
  write_atom_name(t,hh1^.endatom); write(t,hh1^.laenge:8:4);
  hh2:= a.dessen_atomart^.deren_atomlage^.dist_entry;
  while hh2 <> hh1 do begin
    w:= winkel_zwischen_atome(hh1^.endatom,a,hh2^.endatom);
    write(t,w:9:3);
    hh2:= hh2^.naechste_distanz;
    end;
  writeln(t);
  hh1:= hh1^.naechste_distanz;
  end;
writeln(t);
end;

procedure write_distanzen_und_vektoren(var t : text; a:kt$atom);

var hh: p_distanzen_tabellen_eintrag;
    i,j: integer;
    v,va: vektor;

begin
write(t,' Distances and interatomic vectors for atom '); write_atom_name(t,a);
writeln(t);
i:= 0;
hh:= a.dessen_atomart^.deren_atomlage^.dist_entry;
while hh <> nil do begin
  i:= i + 1;
  write(t,' (',i:1,')   -');
  write_atom_name(t,hh^.endatom); write(t,hh^.laenge:8:4);
  berechne_atom_koordinaten(a,va);
  berechne_atom_koordinaten(hh^.endatom,v);
  for j:= 1 to 3 do v[j]:= v[j] - va[j];
  write(t,'     '); write_vektor(t,v); writeln(t);
  hh:= hh^.naechste_distanz;
  end;
writeln(t);
end;

procedure berechne_nte_bindung(var x: kt$atom; n: integer; var y: kt$atom);

begin
y:= x.dessen_atomart^.bindung[n];
y.elementarzelle[1]:= y.elementarzelle[1] + x.elementarzelle[1];
y.elementarzelle[2]:= y.elementarzelle[2] + x.elementarzelle[2];
y.elementarzelle[3]:= y.elementarzelle[3] + x.elementarzelle[3];
end;

function anzahl_bindungen_des_atoms(a: kt$atom): integer;

var j: integer;

begin
with a.dessen_atomart^ do
  if bindung[maxanzahlbindungen].dessen_atomart<>nil then
         j:= maxanzahlbindungen
  else begin
    j:= 0; repeat j:= j+1 until bindung[j].dessen_atomart = nil;
    j:= j-1;
    end;
anzahl_bindungen_des_atoms:= j;
end;

function atome_miteinander_verbunden(a1,a2: kt$atom): boolean;

var i: integer;
    b: boolean;
    a3: kt$atom;

begin
b:= false;
for i:= 1 to anzahl_bindungen_des_atoms(a1) do begin
  berechne_nte_bindung(a1,i,a3);
  if gleiche_atome(a2,a3) then b:= true;
  end;
atome_miteinander_verbunden:= b;
end;

procedure erzeuge_bindung(a1,a2: kt$atom);

var i,j,j_a1,j_a2,k: integer;
    b1,b2: kt$atom;
    zuwenig_bindungen1,zuwenig_bindungen2: boolean;

begin
if atome_miteinander_verbunden(a1,a2) then begin
  write(output,' WARNING  there is already a bond ');
  write_atom_name(output,a1);
  write(output,'  -- '); write_atom_name(output,a2); writeln(output);
  end
else begin
  zuwenig_bindungen1:=false;
  zuwenig_bindungen2:=false;
  j_a1:= anzahl_bindungen_des_atoms(a1);
  j_a2:= anzahl_bindungen_des_atoms(a2);
  for i:=1 to rginfo.zaehligkeit do begin
    alg_symmetrieoperation(rginfo.symop[i]^,a1,b1);
    alg_symmetrieoperation(rginfo.symop[i]^,a2,b2);
    for k:= 1 to 3 do
      b2.elementarzelle[k]:=b2.elementarzelle[k] - b1.elementarzelle[k];
    j:= j_a1;
    with b1.dessen_atomart^ do begin
      repeat j:= j+1;
        if bindung[j].dessen_atomart = nil then bindung[j]:= b2;
        until gleiche_atome(bindung[j],b2) or (j = maxanzahlbindungen);
      if not gleiche_atome(bindung[j],b2) then zuwenig_bindungen1:=true;
      end;
    for k:= 1 to 3 do b1.elementarzelle[k]:= -b2.elementarzelle[k];
    j:= j_a2;
    with b2.dessen_atomart^ do begin
      repeat j:= j+1;
        if bindung[j].dessen_atomart = nil then bindung[j]:= b1;
        until gleiche_atome(bindung[j],b1) or (j = maxanzahlbindungen);
      if not gleiche_atome(bindung[j],b1) then zuwenig_bindungen2:=true;
      end;
    end;
  if zuwenig_bindungen1 then begin
    write(' ERROR  too many bonds for atom ');write_atom_name(output,a1);
    writeln;
    writeln('        The program is compiled for a maximum of ',
            maxanzahlbindungen:1,' bonds.');
    writeln('        Increase constant "maxanzahlbindungen"',
            ' and compile again.');
    end;
  if zuwenig_bindungen2 then begin
    write(' ERROR  too many bonds for ');write_atom_name(output,a2); writeln;
    writeln('        The program is compiled for a maximum of ',
            maxanzahlbindungen:1,' bonds.');
    writeln('        Increase constant "maxanzahlbindungen"',
            ' and compile again.');
    end;
  if zuwenig_bindungen1 or zuwenig_bindungen2 then begin
    write(' ERROR  bond '); write_atom_name(output,a1); write('  -- ');
    write_atom_name(output,a2); writeln(' not created ');
    with a1.dessen_atomart^.deren_atomlage^ do
      for j:= 1 to anzatome do for k:= j_a1+1 to maxanzahlbindungen do
        atom[j]^.bindung[k].dessen_atomart:= nil;
    with a2.dessen_atomart^.deren_atomlage^ do
      for j:= 1 to anzatome do for k:= j_a2+1 to maxanzahlbindungen do
        atom[j]^.bindung[k].dessen_atomart:= nil;
    end
  else begin
    write(' ... new bond '); write_atom_name(output,a1); write('  -- ');
    write_atom_name(output,a2); writeln(' created ');
    end;
  if anzahl_bindungen_des_atoms(a1) > koordinationszahl_des_atoms(a1) then
    writeln(' WARNING  too many bonds for ',
             a1.dessen_atomart^.deren_atomlage^.aname);
  if anzahl_bindungen_des_atoms(a2) > koordinationszahl_des_atoms(a2) then
    if a1.dessen_atomart^.deren_atomlage<>a2.dessen_atomart^.deren_atomlage then
      writeln(' WARNING  too many bonds for ',
               a2.dessen_atomart^.deren_atomlage^.aname);
  end;
end;

procedure loesche_bindung(a1,a2: kt$atom);

var aa: kt$atom;
    b1,b2: array[1..maxanzahlbindungen] of boolean;
    i,j,k: integer;

begin
if atome_miteinander_verbunden(a1,a2) then begin
  for i:= 1 to maxanzahlbindungen do begin
    berechne_nte_bindung(a1,i,aa);
    b1[i]:= false;
    if aa.dessen_atomart<>nil then
      if (not sym_aequival_distanzen(a1,a2,a1,aa))
      and (not sym_aequival_distanzen(a2,a1,a1,aa)) then b1[i]:= true;
    berechne_nte_bindung(a2,i,aa);
    b2[i]:= false;
    if aa.dessen_atomart<>nil then
      if (not sym_aequival_distanzen(a1,a2,a2,aa))
      and (not sym_aequival_distanzen(a2,a1,a2,aa)) then b2[i]:= true;
    end;
  k:= 0;
  with a1.dessen_atomart^.deren_atomlage^ do begin
    for i:= 1 to maxanzahlbindungen do if b1[i] then begin
      k:= k+1;
      for j:= 1 to anzatome do with atom[j]^ do bindung[k]:= bindung[i];
      end;
    for i:= (k+1) to maxanzahlbindungen do
      for j:= 1 to anzatome do with atom[j]^ do bindung[i].dessen_atomart:= nil;
    end;
  if a1.dessen_atomart^.deren_atomlage<>a2.dessen_atomart^.deren_atomlage then
    begin
    k:= 0;
    with a2.dessen_atomart^.deren_atomlage^ do begin
      for i:= 1 to maxanzahlbindungen do if b2[i] then begin
        k:= k+1;
        for j:= 1 to anzatome do with atom[j]^ do bindung[k]:= bindung[i];
        end;
      for i:= (k+1) to maxanzahlbindungen do
        for j:= 1 to anzatome do with atom[j]^ do
          bindung[i].dessen_atomart:= nil;
      end;
    end;
  write(' ... old bond '); write_atom_name(output,a1); write('  -- ');
  write_atom_name(output,a2); writeln(' broken ');
  end
else begin
  write(' WARNING  there is no bond '); write_atom_name(output,a1);
  write('  -- '); write_atom_name(output,a2); writeln;
  end;
end;

procedure berechne_alle_bindungen(var struda: kt$strukturdaten);

var i,j: integer;
    hh: p_distanzen_tabellen_eintrag;

begin
with struda do for j:= 1 to anzatomlagen do with atomlage[j]^ do begin
  hh:= dist_entry;
  for i:= 1 to koordinationszahl do begin
    if hh = nil then begin
      write(' ERROR  not enough distances for atom ');
      write_atom_name(output,grundatom); writeln(output);
      end
    else begin
      if not atome_miteinander_verbunden(grundatom,hh^.endatom) then
        erzeuge_bindung(grundatom,hh^.endatom);
      hh:= hh^.naechste_distanz;
      end;
    end;
  end;
end;

procedure berechne_mitte(a,b: kt$atom; var v: vektor);

var i: integer;

begin
for i:= 1 to 3 do
  v[i]:= (a.dessen_atomart^.pkoord[i] + a.elementarzelle[i] +
          b.dessen_atomart^.pkoord[i] + b.elementarzelle[i])/2;
end;

procedure baue_o_atome_ein(var strukturdaten: kt$strukturdaten;
                                  var status: boolean);

var i,j,k,kk: integer;
    b1,b2,oa: kt$atom;
    v: vektor;
    g: gittervektor;
    neue_atomlage: pkt$atomlage;
    b,bb: boolean;

begin
k:= 0;
with strukturdaten do for i:= 1 to anzatomlagen do with atomlage[i]^ do
  if element='O ' then begin
    kk:= ord(aname[2])-ord('0');
    if aname[3]<>' ' then kk:= kk*10+ord(aname[3])-ord('0');
    if kk>k then k:= kk;
    end;
status:= true;
with strukturdaten do for i:= 1 to anzatomlagen do begin
  b1:=atomlage[i]^.grundatom;
  if not atom_hat_gleiches_element(b1,'O ') then begin
    repeat
    bb:= true;
    for j:= 1 to anzahl_bindungen_des_atoms(b1) do begin
      berechne_nte_bindung(b1,j,b2);
      if not (b2.dessen_atomart=nil) then
      if not atom_hat_gleiches_element(b2,'O ') then begin
        bb:= false;
        berechne_mitte(b1,b2,v);
        int_vektor(v,g);
        frac_vektor(v,v);
        k:= k+1;
        erzeuge_atomlage('O ',k,v,2,neue_atomlage);
        lade_atomlage(neue_atomlage,strukturdaten,b);
        loesche_bindung(b1,b2);
        if b then begin
          verschiebe_atome_leicht(neue_atomlage^);
          oa.dessen_atomart:=atomlage[anzatomlagen]^.atom[1];
          oa.elementarzelle:= g;
          erzeuge_bindung(b1,oa);
          if not atome_miteinander_verbunden(b2,oa) then erzeuge_bindung(b2,oa);
          end;
        end;
      end
    until bb;
    end;
  end;
with strukturdaten do for i:= 1 to anzatomlagen do with atomlage[i]^ do
  if (element='O ') and (koordinationszahl>2) then begin
    writeln(' ERROR  there are too many bonds for ',aname);
    status:= false;
    end;
end;

procedure write_bindungen(var t : text; atom: kt$atom);

var j,k: integer;
    bind: array[1..maxanzahlbindungen] of kt$atom;
    b: boolean;

begin
writeln(t);
write(t,' Bonds from atom '); write_atom_name(t,atom); writeln(t);
for k:= 1 to maxanzahlbindungen do begin
  berechne_nte_bindung(atom,k,bind[k]);
  if bind[k].dessen_atomart <> nil then begin
    write(t,' (',k:1,')   -'); write_atom_name(t,bind[k]);
    write(t,distanz_zwischen_atome(atom,bind[k]):10:4);
    b:= true;
    for j:= 1 to k-1 do
      if (sym_aequival_distanzen(atom,bind[k],atom,bind[j]) or
          sym_aequival_distanzen(atom,bind[k],bind[j],atom)) and b then begin
        b:= false;
        write(t,'    symm. equival. to bond (',j:1,')');
        end;
    writeln(t);
    end;
  end;
end;

procedure write_bindungs_matrix(var t : text; atomlage: kt$atomlage);

var j,k: integer;

begin
with atomlage do begin
  writeln(t,' Atom ',aname);
  for j:= 1 to anzatome do with atom[j]^ do begin
    write(t,j:4,':');
    for k:= 1 to maxanzahlbindungen do
      if bindung[k].dessen_atomart <> nil then write_atom_name(t,bindung[k]);
    writeln(t);
    end;
  end;
writeln(t);
end;

procedure verknuepfe_t_atome_direkt;

var i,di,j: integer;
    d,d2: real;
    b1,b2: kt$atom;

begin
with strukturdaten do for i:= 1 to anzatomlagen do with atomlage[i]^ do
  if element='O ' then
  while koordinationszahl > 2 do begin
    writeln(' WARNING  for atom position ',aname,' the coordination number ',
            koordinationszahl:1,' is greater then 2');
    d:= distanz_zwischen_atome(grundatom,atom[1]^.bindung[1]);
    di:= 1;
    for j:= 2 to koordinationszahl do begin
      d2:= distanz_zwischen_atome(grundatom,atom[1]^.bindung[j]);
      if d2 > d then begin d:= d2; di:= j; end;
      end;
    loesche_bindung(grundatom,atom[1]^.bindung[di]);
    end;
with strukturdaten do for i:= anzatomlagen downto 1 do with atomlage[i]^ do
  if element='O ' then if anzahl_bindungen_des_atoms(grundatom) > 0 then begin
    berechne_nte_bindung(grundatom,1,b1);
    if anzahl_bindungen_des_atoms(grundatom) = 2 then begin
      berechne_nte_bindung(grundatom,2,b2);
      loesche_bindung(grundatom,b2);
      if atome_miteinander_verbunden(grundatom,b1) then
        loesche_bindung(grundatom,b1);
      erzeuge_bindung(b1,b2);
      end;
    if atome_miteinander_verbunden(grundatom,b1) then
      loesche_bindung(grundatom,b1);
    delete_atomlage(strukturdaten.atomlage[i]^,strukturdaten);
    end;
end;

procedure berechne_koord_seq(atoco: kt$atom; anz: integer;
                            var koord_seq: tt$koord_sequenz);

var i,j,k,l,n: integer;
    sphae: array[1..2000] of kt$atom;
    z: array[0..11] of integer;

begin
z[0]:= 0; z[1]:= 1;
z[2]:= koordinationszahl_des_atoms(atoco)+1;
sphae[1]:= atoco;
for i:= 1 to koordinationszahl_des_atoms(atoco) do
  berechne_nte_bindung(atoco,i,sphae[i+1]);
n:= z[2]+1;
for i:= 3 to anz do begin
  for j:= z[i-2]+1 to z[i-1] do
    for k:= 1 to koordinationszahl_des_atoms(sphae[j]) do begin
      berechne_nte_bindung(sphae[j],k,sphae[n]);
      l:=z[i-3];
      repeat l:= l+1 until gleiche_atome(sphae[l],sphae[n]);
      if l=n then n:= n+1;
      end;
  z[i]:= n-1;
  end;
for i:= 1 to 10 do koord_seq[i]:= z[i+1]-z[i];
end;

procedure write_koord_seq(var f: text; z: tt$koord_sequenz);

var i: integer;

begin
for i:= 1 to 10 do write(f,z[i]:4); writeln;
end;

procedure bestimme_topologie;

var i,j,k: integer;
    z: array[1..maxanzahlatomlagen] of tt$koord_sequenz;
    ze: array[1..20] of tt$koord_sequenz;
    anzze: integer;
    zecod,zecodneu: packed array[1..6] of char;
    zcod: array[1..maxanzahlatomlagen,1..10] of packed array[1..6] of char;
    bz,bze: array[1..100] of boolean;
    nzcod: integer;
    b: boolean;

begin
verknuepfe_t_atome_direkt;
writeln(output);
writeln(output,' Coordination sequences:');
with strukturdaten do begin
 for i:= 1 to anzatomlagen do with atomlage[i]^ do
  if koordinationszahl>0 then begin
    berechne_koord_seq(grundatom,11,z[i]);
    write(output,' ',aname); write_koord_seq(output,z[i]);
    end;
  writeln(' Possible frameworks: ');
  for i:= 1 to anzatomlagen do with atomlage[i]^ do if koordinationszahl>0 then
    for j:= 1 to 10 do for k:= 1 to 6 do zcod[i,j,k]:=' ';
  nzcod:=0;
  reset(coseq);
  for i:=1 to 6 do read(coseq,zecodneu[i]);
  while not eof(coseq) do begin
    anzze:= 0;
    repeat
      zecod:=zecodneu;
      anzze:= anzze+1;
      for i:= 1 to 10 do read(coseq,ze[anzze,i]); readln(coseq);
      if not eof(coseq) then begin
        for i:=1 to 6 do read(coseq,zecodneu[i]);
        end
      else zecodneu:='******';
      until zecod<>zecodneu;
    with strukturdaten do for i:= 1 to anzatomlagen do bz[i]:= false;
    for j:= 1 to anzze do bze[j]:= false;
    for i:= 1 to anzatomlagen do with atomlage[i]^ do
    if koordinationszahl>0 then for j:= 1 to anzze do begin
      b:= true;
      for k:= 1 to 10 do if ze[j,k]<>z[i,k] then b:= false;
      if b then begin
        bz[i]:= true; bze[j]:= true;
        end;
      end;
    b:= true;
    for j:= 1 to anzze do if not bze[j] then b:= false;
    if b then begin
      nzcod:= nzcod + 1;
      for i:= 1 to anzatomlagen do with atomlage[i]^ do
        if koordinationszahl>0 then if bz[i] then zcod[i,nzcod]:=zecod;
      end;
    end;
  for i:= 1 to anzatomlagen do with atomlage[i]^ do
    if koordinationszahl>0 then begin
      write(output,' ',aname);
      for j:= 1 to nzcod do write(zcod[i,j]); writeln;
      end;
  end;
end;

procedure berechne_loop_konfig(zentralatom: kt$atom);

type wegtyp = record
       atom: array[1..20] of kt$atom;
       anzatome: integer;
       end;

var i,j,k,l, anzloesungen: integer;
    weg: wegtyp;
    nachbaratom: array[1..maxanzahlbindungen] of kt$atom;
    loesung: array[1..50] of wegtyp;
    b: boolean;

procedure suche_weg(ort,ziel: kt$atom; schritte: integer; weg: wegtyp);

var i,j: integer;
    neuer_ort: kt$atom;
    b: boolean;

begin
if schritte > 0 then for i:= 1 to anzahl_bindungen_des_atoms(ort) do begin
  berechne_nte_bindung(ort,i,neuer_ort);
  weg.atom[schritte]:= neuer_ort;
  b:= true;
  for j:= weg.anzatome downto (schritte+1) do
    if gleiche_atome(weg.atom[j],neuer_ort) then b:= false;
  if b then suche_weg(neuer_ort,ziel,schritte-1,weg);
  end
else if gleiche_atome(ort,ziel) then begin
  anzloesungen:= anzloesungen+1;
  loesung[anzloesungen]:= weg;
  end;
end;

begin
b:= true;
for i:= 1 to anzahl_bindungen_des_atoms(zentralatom) do begin
  berechne_nte_bindung(zentralatom,i,nachbaratom[i]);
  if gleiche_atome(zentralatom,nachbaratom[i]) then begin
    write(' ERROR  bond '); write_atom_name(output,zentralatom); write('  -- ');
    write_atom_name(output,nachbaratom[i]); writeln(' not allowed');
    b:= false;
    end;
  end;
if b then for i:= 1 to anzahl_bindungen_des_atoms(zentralatom) do
  for j:= 1 to (i-1) do begin
    weg.anzatome:=1;
    repeat
      weg.anzatome:= weg.anzatome+1;
      anzloesungen:= 0;
      weg.atom[weg.anzatome]:= zentralatom;
      weg.atom[weg.anzatome-1]:= nachbaratom[i];
      suche_weg(nachbaratom[i],nachbaratom[j],weg.anzatome-2,weg);
      until anzloesungen > 0;
    writeln;
    write(' Angle ');
    write_atomart_name(output,nachbaratom[i].dessen_atomart^); write('  -- ');
    write_atomart_name(output,zentralatom.dessen_atomart^);    write('  -- ');
    write_atomart_name(output,nachbaratom[j].dessen_atomart^);
    if anzloesungen=1 then
      writeln('  has 1 loop  with ',loesung[1].anzatome:1,' atoms')
    else
      writeln('  has ',anzloesungen:1,' loops with ',
              loesung[1].anzatome:1,' atoms');
    for k:= 1 to anzloesungen do with loesung[k] do begin
      write(output,'  -');
      write_atom_name(output,atom[anzatome]); write(output,' ');
      for l:= 1 to (anzatome-1) do begin
        if ((l mod 4) = 0) and (l>1) then write(output,'   ');
        write_atom_name(output,atom[l]); write(output,' ');
        if (l mod 4) = 3 then writeln(output);
        end;
      if (anzatome mod 4) <> 0 then writeln(output);
      end;
    end;
end;

procedure loesche_alle_bindungen(var struda: kt$strukturdaten);

var i: integer;
    a: kt$atom;

begin
with struda do for i:= 1 to anzatomlagen do with atomlage[i]^ do
  while anzahl_bindungen_des_atoms(grundatom)>0 do begin
    berechne_nte_bindung(grundatom,1,a);
    loesche_bindung(grundatom,a);
    end;
end;

procedure write_input_file(var f: text; num: integer;
                           strukturdaten: kt$strukturdaten;
                           var distdat: text);

type nametyp = packed array[1..6] of char;
     dls_symeq_karten = record
       symeq: array[1..maxanzahlsymeqkarten,1..20] of record
                atona: kt$atom;
                erstername,zweitername: nametyp;
                end;
       anzsymeq: array[1..maxanzahlsymeqkarten] of integer;
       end;
     dls_tetcon_karten = record
        tetcon: array[1..maxanzahlatomlagen] of record
                  n: array[0..8] of nametyp;
                  a: array[0..8] of kt$atom;
                  end;
        anztetcon: integer;
        end;
     dls_distan_karten = record
       distcon: array[1..maxindepdistanzen] of record
                  atom1,atom2: kt$atom;
                  atomname1,atomname2: nametyp;
                  end;
       anzdistcon: integer;
       end;
     dls_angle_karten = record
       anglcon: array[1..maxindepwinkel] of record
                  atom1,atom2,atomm: kt$atom;
                  atomname1,atomnamem,atomname2: nametyp;
                  end;
       anzanglcon: integer;
       end;
    dt$geg_distanz = array[1..maxanzahlgegdistanzen] of record
      el1,el2: kt$element;
      kz1,kz2: integer;
      distanz,dvar: real;
      end;
    dt$geg_winkel = array[1..maxanzahlgegwinkel] of record
      el1,el2,el3: kt$element;
      kz1,kz2,kz3: integer;
      winkel,wvar: real;
      end;

var dls_tetcon: dls_tetcon_karten;
    dls_distan: dls_distan_karten;
    dls_angle: dls_angle_karten;
    dls_symeq: dls_symeq_karten;
    geg_dist: dt$geg_distanz;
    geg_wink: dt$geg_winkel;
    i,j: integer;
    b: boolean;
    v: vektor;

procedure read_bindungs_groessen(var f: text; var dist: dt$geg_distanz;
                                              var wink: dt$geg_winkel);

var i,anz_d,anz_w: integer;
    art,c: char;
    e1,e2,e3: kt$element;
    k1,k2,k3: integer;

begin
anz_d:=1; anz_w:=1;
reset(f);
while not eof(f) do begin
  e3[1]:=' '; e1[2]:=' '; e2[2]:=' '; e3[2]:=' ';
  while f^ in [' ','[',']'] do get(f);
  read(f,e1[1]); if f^ in ['A'..'Z','a'..'z',' '] then read(f,e1[2]);
  while f^ in [' ','[',']'] do get(f);
  read(f,k1);
  while f^ in [' ','[',']'] do get(f);
  read(f,e2[1]); if f^ in ['A'..'Z','a'..'z',' '] then read(f,e2[2]);
  while f^ in [' ','[',']'] do get(f);
  read(f,k2);
  while f^ in [' ','[',']'] do get(f);
  if f^ in ['A'..'Z','a'..'z',' '] then begin
    read(f,e3[1]); if f^ in ['A'..'Z','a'..'z',' '] then read(f,e3[2]);
    while f^ in [' ','[',']'] do get(f);
    read(f,k3);
    while f^ in [' ','[',']'] do get(f);
    end;
  if e3[1]=' ' then with dist[anz_d] do begin
    el1:=e1; kz1:= k1;
    el2:=e2; kz2:= k2;
    readln(f,distanz,dvar);
    anz_d:= anz_d+1;
    end
  else with wink[anz_w] do begin
    el1:=e1; kz1:= k1;
    el2:=e2; kz2:= k2;
    el3:=e3; kz3:= k3;
    readln(f,winkel,wvar);
    anz_w:= anz_w+1;
    end;
  end;
for i:= anz_d to maxanzahlgegdistanzen do with dist[i] do begin
  el1:='  ';  el2:='  ';
  kz1:=0;     kz2:=0;
  distanz:= 0.0; dvar:= 0.0;
  end;
for i:= anz_w to maxanzahlgegwinkel do with wink[i] do begin
  el1:='  ';  el2:='  ';  el3:='  ';
  kz1:=0;     kz2:=0;     kz3:=0;
  winkel:= 0.0; wvar:= 0.0;
  end;
end;

procedure write_dls_atom_karten(var f: text; s: kt$strukturdaten);

var i: integer;
    v: vektor;

begin
with s do for i:= 1 to anzatomlagen do with atomlage[i]^ do begin
  berechne_atom_koordinaten(grundatom,v);
  write(f,'ATOM   ',aname,v[1]:8:5,v[2]:8:5,v[3]:8:5,'  ',element,'   ');
  write_atom_trilinpol(f,grundatom); writeln(f);
  end;
end;

procedure write_loadat_atom_karten(var f: text; s: kt$strukturdaten);

var i: integer;

begin
with s do for i:= 1 to anzatomlagen do with atomlage[i]^ do
  writeln(f,'ATOM   ',aname,atomlagekoord[1]:8:5,atomlagekoord[2]:8:5,
                            atomlagekoord[3]:8:5);
end;

procedure write_dls_symeq_karten(var f: text; dls_symeq: dls_symeq_karten);

var i,j,anzsymop: integer;
    symop: st$symop_array;

begin
with dls_symeq do for i:= 1 to strukturdaten.anzatomlagen do
  for j:= 1 to anzsymeq[i] do with symeq[i,j] do begin
    write(f,'SYMEQ  ',erstername,' ',zweitername,' ');
    welchsymop(atona.dessen_atomart^.deren_atomlage^.grundatom,atona,
               rginfo,anzsymop,symop);
    write_symmetrieoperator(f,symop[1]); writeln(f);
    end;
end;

procedure write_dls_tetcon_karten(var f: text; dls_tetcon: dls_tetcon_karten);

var i,j: integer;

begin
with dls_tetcon do for i:= 1 to anztetcon do with tetcon[i] do begin
  write(f,'TETCON       '); for j:= 0 to 8 do write(f,n[j],' '); writeln(f);
  end;
end;

procedure bestimme_geg_distanzen(atom1,atom2: kt$atom;
                                 geg_dist: dt$geg_distanz;
                                 var d,v: real; var b: boolean);

var l,kz_a1, kz_a2: integer;
    el_a1, el_a2: kt$element;

begin
b:= false;
d:= 0.0; v:= 0.0;
el_a1:= atom1.dessen_atomart^.deren_atomlage^.element;
el_a2:= atom2.dessen_atomart^.deren_atomlage^.element;
kz_a1:= koordinationszahl_des_atoms(atom1);
kz_a2:= koordinationszahl_des_atoms(atom2);
l:= 0;
repeat l:= l+1;
  with geg_dist[l] do begin
    if ((el_a1=el1) and (kz_a1=kz1) and (el_a2=el2) and (kz_a2=kz2))
    or ((el_a1=el2) and (kz_a1=kz2) and (el_a2=el1) and (kz_a2=kz1)) then begin
      d:= distanz; v:= dvar;
      b:= true;
      end;
    end;
  until (geg_dist[l].el1='  ') or b;
if not b then
  writeln(' WARNING  no information for distance ',
                el_a1,'[',kz_a1:1,'] - ',el_a2,'[',kz_a2:1,']');
end;

procedure bestimme_geg_winkel(atom1,atom2,atom3: kt$atom;
                              geg_wink: dt$geg_winkel;
                              var w,wv: real; var b: boolean);

var l,kz_a1, kz_a2, kz_a3: integer;
    el_a1, el_a2, el_a3: kt$element;

begin
w:= 0.0; wv:= 0.0;
b:= false;
el_a1:= atom1.dessen_atomart^.deren_atomlage^.element;
el_a2:= atom2.dessen_atomart^.deren_atomlage^.element;
el_a3:= atom3.dessen_atomart^.deren_atomlage^.element;
kz_a1:= koordinationszahl_des_atoms(atom1);
kz_a2:= koordinationszahl_des_atoms(atom2);
kz_a3:= koordinationszahl_des_atoms(atom3);
l:= 0;
repeat l:= l+1;
  with geg_wink[l] do begin
    if ((el_a1=el1) and (kz_a1=kz1) and (el_a2=el2) and (kz_a2=kz2) and
       (el_a3=el3) and (kz_a3=kz3))
    or ((el_a1=el3) and (kz_a1=kz3) and (el_a2=el2) and (kz_a2=kz2) and
       (el_a3=el1) and (kz_a3=kz1)) then begin
      w:= winkel; wv:= wvar;
      b:= true;
      end;
    end;
  until (geg_wink[l].el1='  ') or b;
if not b then
  writeln(' WARNING  no information for angle ',el_a1,'[',kz_a1:1,'] - ',
                el_a2,'[',kz_a2:1,'] - ',el_a3,'[',kz_a3:1,']');
end;

procedure write_dls_distan_karten(var f: text; geg_dist: dt$geg_distanz;
                                  dls_distan:dls_distan_karten);

var i: integer;
    b: boolean;
    distanz,dvar: real;

begin
with dls_distan do for i:= 1 to anzdistcon do with distcon[i] do begin
  bestimme_geg_distanzen(atom1,atom2,geg_dist,distanz,dvar,b);
  write(f,'DISTAN ',atomname1,' ',atomname2,' ');
  if b then writeln(f,distanz:10:6,(1/dvar)/50:9:4)
       else writeln(f,1.0:10:6,0.0:9:4);
  end;
end;

procedure write_loadat_distan_karten(var f: text; geg_dist: dt$geg_distanz;
                                  dls_distan:dls_distan_karten);

var i: integer;
    b: boolean;
    distanz,dvar: real;

begin
with dls_distan do for i:= 1 to anzdistcon do with distcon[i] do begin
  bestimme_geg_distanzen(atom1,atom2,geg_dist,distanz,dvar,b);
  write(f,'DISTAN ',atomname1,' ',atomname2,' ');
  if b then writeln(f,distanz:10:6,dvar:9:4)
       else writeln(f,1.0:10:6,100.0:9:4);
  end;
end;

procedure berechne_t_t_distanzen(d1,d2,w,d1v,d2v,wv: real; var d, dv: real);

begin
w:= w/57.29577951;
wv:= wv/57.29577951;
d:= sqrt( sqr(d1) + sqr(d2) - 2*d1*d2*cos(w) );
dv:= (d1*d1v + d2*d2v - (d1*d2v + d2*d1v)*cos(w) + d1*d2*sin(w)*wv )/d;
end;

procedure write_dls_angle_karten(var f: text; geg_wink: dt$geg_winkel;
                                 geg_dist: dt$geg_distanz;
                                 dls_angle:dls_angle_karten);

var b,bb: boolean;
    d1,d2,d1v,d2v,w,wv,distanz,dvar: real;
    i: integer;

begin
with dls_angle do for i:= 1 to anzanglcon do with anglcon[i] do begin
  bestimme_geg_distanzen(atom1,atomm,geg_dist,d1,d1v,bb);
  bestimme_geg_distanzen(atom2,atomm,geg_dist,d2,d2v,b);
  if not b then bb:= false;
  bestimme_geg_winkel(atom1,atomm,atom2,geg_wink,w,wv,b);
  if not b then bb:= false;
  if bb then begin
    berechne_t_t_distanzen(d1,d2,w,d1v,d2v,wv,distanz,dvar);
    writeln(f,'DISTAN ',atomname1,' ',atomname2,' ',
              distanz:10:6,(1/dvar)/50:9:4);
    end
  else
    writeln(f,'DISTAN ',atomname1,' ',atomname2,' ',1.0:10:6,0.0:9:4);
  end;
end;

procedure write_loadat_angle_karten(var f: text; geg_wink: dt$geg_winkel;
                                 dls_angle:dls_angle_karten);

var b: boolean;
    winkel,wvar: real;
    i: integer;

begin
with dls_angle do for i:= 1 to anzanglcon do with anglcon[i] do begin
  bestimme_geg_winkel(atom1,atomm,atom2,geg_wink,winkel,wvar,b);
  write(f,'ANGLE  ',atomname1,' ',atomnamem,' ',atomname2,' ');
  if b then writeln(f,winkel:10:6,wvar:9:4) else writeln(f,90.0:10:6,0.0:9:4);
  end;
end;

procedure write_dls_bondis_karten(var t: text; dls_tetcon: dls_tetcon_karten;
                                  geg_dist: dt$geg_distanz;
                                  geg_wink: dt$geg_winkel);

type bondis_type = array[1..20] of record
       e1,e2,e3: kt$element;
       a1,a2,a3: kt$atom;
       end;

var bondis: bondis_type;
    i,j,l,nn,kzz: integer;
    a,b,c,d,e,f,g: real;
    d1,d2,d1v,d2v,w1,w2,w1v,w2v,d11,d12,d11v,d12v: real;
    d1b,d2b,w1b,w2b,bb: boolean;

begin
a:=  1.0000;
b:= -0.0004;
c:=  0.0000;
d:=  145.00;
e:=  1.0000;
f:=  1.0000;
g:=  1.0000;
nn:= 0;
with dls_tetcon do for i:= 1 to anztetcon do with tetcon[i] do
  for j:= 1 to 4 do with bondis[nn+1] do begin
    a1:= a[0];   e1:= a1.dessen_atomart^.deren_atomlage^.element;
    a2:= a[j];   e2:= a2.dessen_atomart^.deren_atomlage^.element;
    a3:= a[j+4]; e3:= a3.dessen_atomart^.deren_atomlage^.element;
    bb:=true;
    for l:= 1 to nn do
      if (e1=bondis[l].e1) and (e2=bondis[l].e2) and (e3=bondis[l].e3) then
        bb:= false;
    if bb then nn:= nn+1;
    end;
for i:= 1 to nn do with bondis[i] do begin
  kzz:= a3.dessen_atomart^.deren_atomlage^.koordinationszahl;
  a3.dessen_atomart^.deren_atomlage^.koordinationszahl:=4;
  bestimme_geg_distanzen(a1,a2,geg_dist,d1,d1v,d1b);
  bestimme_geg_distanzen(a3,a2,geg_dist,d2,d2v,d2b);
  bestimme_geg_winkel(a1,a2,a3,geg_wink,w1,w1v,w1b);
  bestimme_geg_winkel(a2,a1,a2,geg_wink,w2,w2v,w2b);
  a3.dessen_atomart^.deren_atomlage^.koordinationszahl:=kzz;
  if d1b then begin
    a:= d1; e:=1/d1v;
    end;
  if w1b then d:= w1;
  if d1b and w2b then begin
    berechne_t_t_distanzen(d1,d1,w2,d1v,d1v,w2v,d11,d11v);
    f:= 1/d11v;
    end;
  if d1b and d2b and w1b then begin
    berechne_t_t_distanzen(d1,d2,w1,d1v,d2v,w1v,d12,d12v);
    g:= 1/d12v;
    end;
  writeln(t,'BONDIS ',e1,'  ',e2,'  ',e3,'   ',
            a:10:5,b:10:5,c:10:5,d:10:5,(e/50):5:1,(f/50):5:1,(g/50):5:1);
  end;
end;

procedure korrigiere_koordinationszahlen(var b: boolean);

var i,kzn: integer;
    c: char;

begin
b:= true;
with strukturdaten do for i:= 1 to anzatomlagen do with atomlage[i]^ do begin
  kzn:= anzahl_bindungen_des_atoms(grundatom);
  if kzn<>koordinationszahl then begin
    if kzn = 1 then writeln(' ERROR  for atom ',aname,' there is 1 bond ')
      else writeln(' ERROR  for atom ',aname,' there are ',kzn:1,' bonds ');
    writeln('        but the given coordination number is ',
            koordinationszahl:1);
    b:= false;
    end;
  end;
if not b then begin
  write(' change coordination numbers to number of bonds? (y/n):');
  if not eoln(input) then begin
    readln(c);
    if c in ['Y','y','J','j'] then begin
      b:= true;
      with strukturdaten do
      for i:= 1 to anzatomlagen do with atomlage[i]^ do begin
        kzn:= anzahl_bindungen_des_atoms(grundatom);
        if kzn<>koordinationszahl then begin
          write(' ... coordination number for atom position ',aname);
          writeln(' changed to ',kzn:1);
          koordinationszahl:=kzn;
          end;
        end;
      end;
    end;
  end;
end;

procedure berechne_dls_input(var dls_tetcon:  dls_tetcon_karten;
                             var dls_distan: dls_distan_karten;
                             var dls_angle: dls_angle_karten;
                             var dls_symeq: dls_symeq_karten; var b: boolean);

var i,j,k,l,anzsymop: integer;
    symeq0: st$symop_array;
    a1,a2,am,atom0: kt$atom;

begin
korrigiere_koordinationszahlen(b);
if not b then writeln(' ... generation of input file interrupted')
else with strukturdaten do begin
  with dls_symeq do with dls_tetcon do begin
    for i:= 1 to maxanzahlsymeqkarten do anzsymeq[i]:= 0;
    for i:= 1 to 4 do for j:= 0 to 8 do tetcon[i].n[j]:= '      ';
    anztetcon:= 0;
    for i:= 1 to anzatomlagen do if atomlage[i]^.koordinationszahl=4 then begin
      b:= true;
      with atomlage[i]^.atom[1]^ do for j:= 1 to 4 do
        if koordinationszahl_des_atoms(bindung[j]) <> 2 then b:= false;
      if b then begin
        anztetcon:= anztetcon+1;
        with tetcon[anztetcon] do begin
          a[0]:= atomlage[i]^.grundatom;
          for j:= 1 to 4 do begin
            berechne_nte_bindung(a[0],j,a[j]);
            berechne_nte_bindung(a[j],1,a[4+j]);
            if gleiche_atome(a[4+j],a[0]) then
                                       berechne_nte_bindung(a[j],2,a[4+j]);
            end;
          end;
        end;
      end;
    end;
  with dls_distan do begin
    anzdistcon:= 0;
    for i:= 1 to anzatomlagen do
    for j:= 1 to atomlage[i]^.koordinationszahl do begin
      a1:= atomlage[i]^.grundatom;
      berechne_nte_bindung(a1,j,a2);
      b:= true;
      for k:= 1 to anzdistcon do with distcon[k] do
        if sym_aequival_distanzen(a1,a2,atom1,atom2)
        or sym_aequival_distanzen(a1,a2,atom2,atom1) then b:= false;
      if b then if anzdistcon < maxindepdistanzen then begin
        anzdistcon:=  anzdistcon +1;
        distcon[anzdistcon].atom1:= a1;
        distcon[anzdistcon].atom2:= a2;
        end
      else begin
        writeln(' ERROR  too many symmetrically independent distances');
        writeln('        The program is compiled for a maximum of ',
                maxindepdistanzen:1,' independent distances.');
        writeln('        Increase constant "maxindepdistanzen" and compile',
                ' again.');
        end;
      end;
    end;
  with dls_angle do begin
    anzanglcon:= 0;
    for i:= 1 to anzatomlagen do for j:= 1 to atomlage[i]^.koordinationszahl do
    for l:= 1 to j-1 do begin
      am:= atomlage[i]^.grundatom;
      berechne_nte_bindung(am,j,a1);
      berechne_nte_bindung(am,l,a2);
      b:= true;
      for k:= 1 to anzanglcon do with anglcon[k] do
        if sym_aequival_winkel(a1,am,a2,atom1,atomm,atom2)
        or sym_aequival_winkel(a1,am,a2,atom2,atomm,atom1) then b:= false;
      if b then if anzanglcon < maxindepwinkel then begin
        anzanglcon:= anzanglcon +1;
        anglcon[anzanglcon].atom1:= a1;
        anglcon[anzanglcon].atom2:= a2;
        anglcon[anzanglcon].atomm:= am;
        end
      else begin
        writeln(' ERROR  too many symmetrically independent angles');
        writeln('        The program is compiled for a maximum of ',
                maxindepwinkel:1,' independent angles.');
        writeln('        Increase constant "maxindepwinkel" and compile',
                ' again.');
        end;
      end;

    (* bringe atom1 in Lage x,y,z *)
    for i:= 1 to anzanglcon do with anglcon[i] do begin
      atom0:= atom1.dessen_atomart^.deren_atomlage^.grundatom;
      welchsymop(atom1,atom0,rginfo,anzsymop,symeq0);
      alg_symmetrieoperation(symeq0[1],atom1,atom1);
      alg_symmetrieoperation(symeq0[1],atomm,atomm);
      alg_symmetrieoperation(symeq0[1],atom2,atom2);
      end;
    end;
  b:= true;
  end;
end;

procedure erzeuge_atomname(atoco: kt$atom; var neuername: nametyp;
                           var dls_symeq: dls_symeq_karten);

var i: integer;
    b: boolean;
    gruato: kt$atom;

begin
neuername:= atoco.dessen_atomart^.deren_atomlage^.aname;
gruato:= atoco.dessen_atomart^.deren_atomlage^.grundatom;
if not gleiche_atome(atoco,gruato) then with dls_symeq do
  with atoco.dessen_atomart^.deren_atomlage^  do begin
  b:= true;
  for i:= 1 to anzsymeq[atomlagenr] do
    if gleiche_atome(atoco,symeq[atomlagenr,i].atona) then begin
      b:= false;
      neuername:=symeq[atomlagenr,i].zweitername;
      end;
  if b then with symeq[atomlagenr,anzsymeq[atomlagenr] + 1] do  begin
    atona:=atoco;
    erstername:= neuername;
    i:= 1; while erstername[i] <> ' ' do i:= i+1;
    anzsymeq[atomlagenr]:=anzsymeq[atomlagenr]+1;
    neuername[i]:='*';
    neuername[i+1]:=chr(anzsymeq[atomlagenr]+ord('A')-1);
    zweitername:= neuername;
    end;
  end;
end;

procedure loesche_redundante_karten(var dls_tetcon:  dls_tetcon_karten;
                                    var dls_distan: dls_distan_karten;
                                    var dls_angle: dls_angle_karten);

var i,j,k,l,m: integer;
    b: boolean;

begin
k:= 0;
with dls_distan do with dls_tetcon do for i:= 1 to anzdistcon do begin
  b:= true;
  for j:= 1 to anztetcon do with distcon[i] do with tetcon[j] do
  for l:= 1 to 4 do
    if sym_aequival_distanzen(atom1,atom2,a[0],a[l])
    or sym_aequival_distanzen(atom1,atom2,a[l],a[0]) then b:= false;
  if b then begin
    k:= k+1;
    distcon[k]:= distcon[i];
    end;
  end;
dls_distan.anzdistcon:= k;
k:= 0;
with dls_angle do with dls_tetcon do for i:= 1 to anzanglcon do begin
  b:= true;
  for j:= 1 to anztetcon do with anglcon[i] do with tetcon[j] do
  for l:= 1 to 4 do begin
    if sym_aequival_winkel(atom1,atomm,atom2,a[0],a[l],a[l+4])
    or sym_aequival_winkel(atom1,atomm,atom2,a[l+4],a[l],a[0]) then b:= false;
    for m:= 1 to l-1 do
    if sym_aequival_winkel(atom1,atomm,atom2,a[l],a[0],a[m])
    or sym_aequival_winkel(atom1,atomm,atom2,a[m],a[0],a[l]) then b:= false;
    end;
  if b then begin
    k:= k+1;
    anglcon[k]:= anglcon[i];
    end;
  end;
dls_angle.anzanglcon:= k;
end;

begin
read_bindungs_groessen(distdat,geg_dist,geg_wink);
berechne_dls_input(dls_tetcon,dls_distan,dls_angle,dls_symeq,b);
if b then begin
  if num=1 then begin
    loesche_redundante_karten(dls_tetcon,dls_distan,dls_angle);
    with dls_tetcon do for i:= 1 to anztetcon do with tetcon[i] do
      for j:= 0 to 8 do erzeuge_atomname(a[j],n[j],dls_symeq);
    end;
  with dls_distan do for i:= 1 to anzdistcon do with distcon[i] do begin
    erzeuge_atomname(atom1,atomname1,dls_symeq);
    erzeuge_atomname(atom2,atomname2,dls_symeq);
    end;
  if (num=1) or (num=2) then
  with dls_angle do for i:= 1 to anzanglcon do with anglcon[i] do begin
    erzeuge_atomname(atom1,atomname1,dls_symeq);
    erzeuge_atomname(atom2,atomname2,dls_symeq);
    end;
  if num=3 then
  with dls_angle do for i:= 1 to anzanglcon do with anglcon[i] do begin
    erzeuge_atomname(atom1,atomname1,dls_symeq);
    erzeuge_atomname(atomm,atomnamem,dls_symeq);
    erzeuge_atomname(atom2,atomname2,dls_symeq);
    end;
  if (num=1) or (num=2) then with strukturdaten do begin
    writeln(f,'TITLE ',name);
    if num=1 then
      writeln(f,'DLS-76   1    10   2 0         2                1');
    if num=2 then
      writeln(f,'DLS-76   1    10     0         2                1');
    if kristallsystem='TRIG' then kristallsystem:='HEX ';
    with zellpar do writeln(f,'CELL   ',kristallsystem,'         ',
              laenge[1]:10:4,laenge[2]:10:4,laenge[3]:10:4,
              winkel[1]:10:4,winkel[2]:10:4,winkel[3]:10:4);
    write_dls_atom_karten(f,strukturdaten);
    write_dls_symeq_karten(f,dls_symeq);
    if num=1 then write_dls_bondis_karten(f,dls_tetcon,geg_dist,geg_wink);
    write_dls_distan_karten(f,geg_dist,dls_distan);
    write_dls_angle_karten(f,geg_wink,geg_dist,dls_angle);
    end;
  if num=1 then write_dls_tetcon_karten(f,dls_tetcon);
  if num=3 then begin
    rewrite(f);
    write_loadat_atom_karten(f,strukturdaten);
    write_dls_symeq_karten(f,dls_symeq);
    write_loadat_distan_karten(f,geg_dist,dls_distan);
    write_loadat_angle_karten(f,geg_wink,dls_angle);
    end;
  writeln(f,'END');
  writeln(f,'FINISH');
  end;
b:= false;
v[1]:=1; v[2]:=0; v[3]:=0;
if polare_richtung(v,rginfo) then begin
  writeln(output,' WARNING  in this space group, the origin is not defined ',
                 'in the x direction ');
  writeln(output,'          The x coordinate of one atom must be fixed with a',
                 ' NOREF card.');
  end;
v[1]:=0; v[2]:=1; v[3]:=0;
if polare_richtung(v,rginfo) then begin
  writeln(output,' WARNING  in this space group, the origin is not defined ',
                 'in the y direction ');
  writeln(output,'          The y coordinate of one atom must be fixed with a',
                 ' NOREF card.');
  end;
v[1]:=0; v[2]:=0; v[3]:=1;
if polare_richtung(v,rginfo) then begin
  writeln(output,' WARNING  in this space group, the origin is not defined ',
                 'in the z direction ');
  writeln(output,'          The z coordinate of one atom must be fixed with a',
                 ' NOREF card.');
  end;
end;

procedure m$initialisiere(var strukturdaten: kt$strukturdaten);

begin
strukturdaten.raumgruppe:='                              ';
berechnet[struktur]:= false;
berechnet[distanzen]:= false;
berechnet[bindungen]:= false;
strudatnew_offen:= false;
rewrite(list);
writeln(' ****************************************************************',
'**************');
writeln(' *                                                               ',
'             *');
writeln(' *   Program  KRIBER        Version 1.0    (January 1991)        ',
'             *');
writeln(' *   ===============                                             ',
'             *');
writeln(' *                                                               ',
'             *');
writeln(' *   an interactive PASCAL program to                            ',
'             *');
writeln(' *    - calculate distances and angles                           ',
'             *');
writeln(' *    - generate input files for the programs DLS-76 and LOADAT (',
'XRS-82)      *');
writeln(' *    - calculate coordination squences and loop configurations  ',
'             *');
writeln(' *                                                               ',
'             *');
writeln(' *   Author:   Roland Bialek, Institut fuer Kristallografie und P',
'etrografie   *');
writeln(' *             ETH-Z, CH-8092 Zuerich, Switzerland               ',
'             *');
writeln(' *                                                               ',
'             *');
writeln(' *    ===> type help to get a summary of the commands            ',
'             *');
writeln(' *              quit or exit to leave the program                ',
'             *');
writeln(' *                                                               ',
'             *');
writeln(' ****************************************************************',
'**************');
end;

procedure m$write_hilfe;

begin
writeln;
writeln(' *******************************************************************');
writeln(' *                                                                 *');
writeln(' *          S U M M A R Y    O F   A L L   C O M M A N D S         *');
writeln(' *                                                                 *');
writeln(' *******************************************************************');
writeln;
writeln(' read      crystal structure data                        ===> reacs ');
writeln(' read      data from first entry in file STRUDAT         ===> reafcs');
writeln(' read      data from next entry in file STRUDAT          ===> reancs');
writeln(' write     structure data on file STRUDATNEW             ===> wrie  ');
writeln(' display   all entries in file STRUDAT                   ===> dise  ');
writeln('                                                                    ');
writeln(' display   crystal structure                             ===> discs ');
writeln(' write     crystal structure on file LIST                ===> wrics ');
writeln(' add       atom position                                 ===> adda  ');
writeln(' delete    atom position                                 ===> dela  ');
writeln(' display   space group                                   ===> diss  ');
writeln(' write     space group on file LIST                      ===> wris  ');
writeln(' display   coordinates                                   ===> disc  ');
writeln(' write     coordinates on file LIST                      ===> wric  ');
writeln('                                                                    ');
writeln('        Type <ret> to continue                                      ');
readln;
writeln('                                                                    ');
writeln(' calculate distances                                     ===> cald  ');
writeln(' display   all distances                                 ===> disad ');
writeln(' display   distances from one atom                       ===> disd  ');
writeln(' display   distances and angles                          ===> disda ');
writeln(' display   distances and interatomic vectors             ===> disdv ');
writeln(' write     all distances on file LIST                    ===> wriad ');
writeln(' write     distances from one atom on file LIST          ===> wrid  ');
writeln(' write     distances and angles on file LIST             ===> wrida ');
writeln(' write     distances and interatomic vectors on file LIST===> wridv ');
writeln('                                                                    ');
writeln(' create    all bonds using shortest distances            ===> creab ');
writeln(' create    a bond                                        ===> creb  ');
writeln(' delete    a bond                                        ===> delb  ');
writeln(' delete    all bonds                                     ===> delab ');
writeln(' display   bonds from one atom                           ===> disb  ');
writeln(' display   all bonds                                     ===> disab ');
writeln(' write     bonds from one atom on file LIST              ===> wrib  ');
writeln(' write     all bonds on file LIST                        ===> wriab ');
writeln('                                                                    ');
writeln('                                                                    ');
writeln('        Type <ret> to continue                                      ');
readln;
writeln('                                                                    ');
writeln(' add       O atoms (oxygen bridges)                      ===> addo  ');
writeln(' delete    oxygen bridges, connect non O-atoms directly  ===> delo  ');
writeln('                                                                    ');
writeln(' calculate coordination sequences                        ===> calcs ');
writeln(' calculate loop configuration                            ===> callc ');
writeln(' determine topology of the framework                     ===> dett  ');
writeln('                                                                    ');
writeln(' write     input file for DLS-76 on file DLSINPUT        ===> wriid ');
writeln(' write     input file for LOADAT on file LOADATINPUT     ===> wriil ');
writeln('                                                                    ');
writeln('                                                                    ');
writeln(' display   a summary of the commands                     ===> help  ');
writeln(' quit or exit the program                                ===> quit  ');
writeln('                                                      or ===> exit  ');
writeln('                                                                    ');
writeln('                                                                    ');
writeln(' *******************************************************************');
writeln;
writeln;
writeln;
writeln;
(* Fuer Kontrollzwecke  cons : kontrolliere Raumgruppe                      *)
(* Fuer Kontrollzwecke  conb : schreibe alle Bindungen                      *)
(* Fuer Kontrollzwecke  calda: berechne Distanzen algebraisch               *)
end;

procedure m$read_strukturdaten;

var i: integer;
    eintrag_gefunden: boolean;

begin
reset(strudat);
writeln(' Enter compound identification code to read data from the file');
write(  ' STRUDAT or <ret> to enter the data interactively:');
if eof(input) then begin
  reset(input);
  read_strukturdaten(input,strukturdaten,true);
  strukturdaten.abk:= '???       ';
  berechnet[struktur]:= true;
  berechnet[distanzen]:= false;
  berechnet[bindungen]:= false;
  end
else begin
  for i:= 1 to 10 do strukturdaten.abk[i]:= ' ';
  i:= 0;
  while (not eoln(input)) and (i<10) do begin
    i:= i+1;
    read(strukturdaten.abk[i]);
    end;
  if strukturdaten.abk[10]<>' ' then
    writeln(' ERROR  compound identification code too long');
  readln(input);
  reset(strudat);
  suche_strukturdaten_eintrag(strudat, strukturdaten.abk,eintrag_gefunden);
  if eintrag_gefunden then begin
    read_strukturdaten(strudat,strukturdaten,false);
    berechnet[struktur]:= true;
    berechnet[distanzen]:= false;
    berechnet[bindungen]:= false;
    end;
  end;
end;

procedure m$read_erste_strukturdaten;

var b: boolean;

begin
reset(strudat);
suche_naechsten_eintrag(strudat,strukturdaten.abk,b);
if b then begin
  writeln(output,' ... entry ',strukturdaten.abk,' found');
  read_strukturdaten(strudat,strukturdaten,false);
  berechnet[struktur]:= true;
  berechnet[distanzen]:= false;
  berechnet[bindungen]:= false;
  end
else writeln(output,' ERROR  no entry found on file STRUDAT');
end;

procedure m$read_naechste_strukturdaten;

var b: boolean;

begin
suche_naechsten_eintrag(strudat,strukturdaten.abk,b);
if b then begin
  writeln(output,' ... entry ',strukturdaten.abk,' found');
  read_strukturdaten(strudat,strukturdaten,false);
  berechnet[struktur]:= true;
  berechnet[distanzen]:= false;
  berechnet[bindungen]:= false;
  end
else writeln(output,' ERROR  no entry found on file STRUDAT');
end;

procedure m$write_strukturdaten(var f: text; strukturdaten: kt$strukturdaten);

begin
if not berechnet[struktur] then m$read_strukturdaten;
write_strukturdaten(f,strukturdaten);
end;

procedure m$contr_st$raumgruppe;

var v: vektor;

begin
readln_raumgruppename(input,strukturdaten.raumgruppe);
read_raumgruppe(symdat,strukturdaten.raumgruppe,rginfo);
strukturdaten.kristallsystem:= rginfo.kristallsystem;
write_st$raumgruppe(output,rginfo);
v[1]:=1; v[2]:=0; v[3]:=0;
if polare_richtung(v,rginfo) then begin
  write(output,' possible shift in ');
  write_vektor(output,v); writeln(output);
  end;
v[1]:=0; v[2]:=1; v[3]:=0;
if polare_richtung(v,rginfo) then begin
  write(output,' possible shift in ');
  write_vektor(output,v); writeln(output);
  end;
v[1]:=0; v[2]:=0; v[3]:=1;
if polare_richtung(v,rginfo) then begin
  write(output,' possible shift in ');
  write_vektor(output,v); writeln(output);
  end;
end;

procedure m$write_koordinaten(var f: text);

var i: integer;

begin
if not berechnet[struktur] then m$read_strukturdaten;
with strukturdaten do
  for i:= 1 to anzatomlagen do write_atomlage_koordinaten(f,atomlage[i]^);
end;

procedure m$berechne_distanzen(anzdistanzen: integer);

var i: integer;

begin
if not berechnet[struktur] then m$read_strukturdaten;
for i:= 1 to strukturdaten.anzatomlagen do
  berechne_distanzen(strukturdaten.atomlage[i]^.grundatom,strukturdaten,
                     anzdistanzen,8);
berechnet[distanzen]:= true;
end;

procedure m$write_distanzen(var f: text);

var i: integer;
    a: kt$atom;

begin
if not berechnet[distanzen] then m$berechne_distanzen(maxanzahldistanzen);
write(' display distances from atom:'); readln_atom(input,a);
if a.dessen_atomart<>nil then write_distanzen(f,a)
else writeln(' ERROR  cannot display distances');
end;

procedure m$write_alle_distanzen(var f: text);

var i: integer;

begin
if not berechnet[distanzen] then m$berechne_distanzen(maxanzahldistanzen);
with strukturdaten do for i:= 1 to anzatomlagen do
  write_distanzen(f,atomlage[i]^.grundatom);
end;

procedure m$write_distanzen_und_winkel(var f: text);

var i: integer;

begin
if not berechnet[distanzen] then m$berechne_distanzen(maxanzahldistanzen);
with strukturdaten do for i:= 1 to anzatomlagen do
  write_distanzen_und_winkel(f,atomlage[i]^.grundatom);
end;

procedure m$write_distanzen_und_vektoren(var f: text);

var i: integer;

begin
if not berechnet[distanzen] then m$berechne_distanzen(maxanzahldistanzen);
with strukturdaten do for i:= 1 to anzatomlagen do
  write_distanzen_und_vektoren(f,atomlage[i]^.grundatom);
end;

procedure m$berechne_bindungen;

begin
if not berechnet[distanzen] then m$berechne_distanzen(maxanzahldistanzen);
berechne_alle_bindungen(strukturdaten);
berechnet[bindungen]:=true;
end;

procedure m$write_bindungen(var f: text);

var i: integer;
    a: kt$atom;

begin
if not berechnet[bindungen] then m$berechne_bindungen;
write(' display bonds from atom:'); readln_atom(input,a);
if a.dessen_atomart<>nil then write_bindungen(f,a)
else writeln(' ERROR  cannot display bonds');
end;

procedure m$write_alle_bindungen(var f: text);

var i: integer;

begin
if not berechnet[bindungen] then m$berechne_bindungen;
with strukturdaten do for i:= 1 to anzatomlagen do
  write_bindungen(f,atomlage[i]^.grundatom);
end;

procedure m$write_bindungs_matrix(var f: text);

var i: integer;

begin
with strukturdaten do
  for i:= 1 to anzatomlagen do write_bindungs_matrix(f,atomlage[i]^);
end;

procedure m$DLSinput;

var c: char;
    n: integer;

begin
if not berechnet[bindungen] then m$berechne_bindungen;
rewrite(dlsinp(* ### BK ut *));
write(' DLSinput with TETCON cards? (y/n):');
if not eoln(input) then begin
  readln(c);
  if c in ['Y','y','J','j'] then n:= 1;
  if c in ['N','n'] then n:= 2;
  end;
write_input_file(dlsinp(* ### BK ut *),n,strukturdaten,distdat);
end;

procedure m$LOADATinput;

begin
rewrite(loadainp(* ### BK loadatinput*));
if not berechnet[bindungen] then m$berechne_bindungen;
write_input_file(loadainp(* ### BK loadatinput *),3,strukturdaten,distdat);
end;

procedure m$berechne_o_positionen;

var status: boolean;

begin
if not berechnet[bindungen] then m$berechne_bindungen;
baue_o_atome_ein(strukturdaten,status);
berechnet[distanzen]:= false;
end;

procedure m$write_strukturdaten_karte;

begin
if not strudatnew_offen then begin
  rewrite(strudatn (* ### BK ew *) );
  strudatnew_offen := true;
  end;
write_strukturdaten_karte(strudatn (* ### BK ew *) ,strukturdaten);
end;

procedure m$loesche_atomlage;

var pala: pkt$atomlage;

begin
write(' Atom position to be deleted:');
if not eof(input) then begin
  read_atomlage_bez(input,pala); readln;
  if pala=nil then writeln(' ERROR  cannot delete this atom')
  else begin
    delete_atomlage(pala^,strukturdaten);
    berechnet[distanzen]:=false;
    with pala^.atom[1]^ do while bindung[1].dessen_atomart<>nil do
      loesche_bindung(pala^.grundatom,bindung[1]);
    end;
  end;
end;

procedure m$erzeuge_atomlage;

var neue_atomlage: pkt$atomlage;
    b: boolean;

begin
with strukturdaten do begin
  writeln(' Add atom position no. ',(anzatomlagen+1):1);
  writeln(' label (element symbol + number)   coordinates',
          '   coordination number');
  new(neue_atomlage);
  write(' (',(anzatomlagen+1):2,')  :');
  read_atomlage(input,neue_atomlage);
  with neue_atomlage^ do frac_vektor(atomlagekoord,atomlagekoord);
  lade_atomlage(neue_atomlage,strukturdaten,b);
  berechnet[distanzen]:=false;
  end;
end;

procedure m$erzeuge_bindung;

var a1,a2: kt$atom;
    d: real;
    n: integer;

begin
write(' Create new bond:             first atom:'); readln_atom(input,a1);
if a1.dessen_atomart<>nil then begin
  a2.dessen_atomart:=nil;
  write(' second atom or distance no. (from disd):');
  while (not eoln(input)) and (input^ = ' ') do get(input);
  if not eoln(input) then begin
    if input^='(' then get(input);
    if input^ in ['0','1'..'9'] then begin
      readln(n);
      berechne_nte_distanz(a1,n,d,a2);
      end
    else readln_atom(input,a2);
    end;
  if a2.dessen_atomart<>nil then erzeuge_bindung(a1,a2)
  else writeln(' ERROR  cannot create this bond');
  end
else writeln(' ERROR  cannot create this bond');
berechnet[bindungen]:= true;
end;

procedure m$loesche_bindung;

var a1,a2: kt$atom;
    n: integer;

begin
write(' Delete bond:             first atom:'); readln_atom(input,a1);
if a1.dessen_atomart<>nil then begin
  a2.dessen_atomart:=nil;
  write(' second atom or bond no. (from disb):');
  while (not eoln(input)) and (input^ = ' ') do get(input);
  if not eoln(input) then begin
    if input^='(' then get(input);
    if input^ in ['0','1'..'9'] then begin
      readln(n);
      if (n>0) and (n<=maxanzahlbindungen) then berechne_nte_bindung(a1,n,a2)
                                           else a2.dessen_atomart:=nil;
      if a2.dessen_atomart=nil then begin
        writeln(' ERROR  there is no bond no. ',n:1,' for atom ');
        write_atom_name(output,a1);
        end;
      end
    else readln_atom(input,a2);
    end;
  if a2.dessen_atomart<>nil then loesche_bindung(a1,a2)
  else writeln(' ERROR  cannot delete this bond');
  end
else writeln(' ERROR  cannot delete this bond')
end;

procedure m$loesche_alle_bindung;

begin
loesche_alle_bindungen(strukturdaten);
end;

procedure m$verknuepfe_t_atome_direkt;

begin
if not berechnet[bindungen] then m$berechne_bindungen;
verknuepfe_t_atome_direkt;
end;

procedure m$berechne_koord_seq;

var i: integer;
    z: tt$koord_sequenz;

begin
if not berechnet[bindungen] then m$berechne_bindungen;
writeln(output);
writeln(output,' Coordination sequences of the T-atoms:');
with strukturdaten do for i:= 1 to anzatomlagen do begin
  berechne_koord_seq(atomlage[i]^.grundatom,11,z);
  write(output,' ',atomlage[i]^.aname); write_koord_seq(output,z);
  end
end;

procedure m$bestimme_topologie;

begin
if not berechnet[bindungen] then m$berechne_bindungen;
bestimme_topologie;
end;

procedure m$berechne_loop_konfig;

var i: integer;

begin
if not berechnet[bindungen] then m$berechne_bindungen;
with strukturdaten do for i:= 1 to anzatomlagen do begin
  writeln(output);
  write(output,' Loop configuration for ');
  write_atom_name(output,atomlage[i]^.grundatom); writeln(output);
  berechne_loop_konfig(atomlage[i]^.grundatom);
  end
end;

procedure m$write_entry_liste;

var i: integer;

begin
reset(strudat);
writeln(' Entries on file STRUDAT');
while not eof(strudat) do begin
  if strudat^='*' then begin
    i:= 0;
    write(' ');
    while not eoln(strudat) do begin
      write(strudat^); get(strudat);
      i:= i+1;
      end;
    writeln;
    if i>9 then writeln(' ERROR  compound identification code too long');
    end;
  readln(strudat);
  end;
end;

procedure m$read_befehl(var befehl: mt$befehl);

var i: integer;

begin
for i:= 1 to 8 do befehl[i]:=' ';
write(' ===>');
i:= 0;
if eof(input) then reset(input)
else begin
  while (not eoln(input)) and (input^ = ' ') do get(input);
  while (not eoln(input)) and (i<8) do begin i:= i+1; read(input,befehl[i]) end;
  readln(input);
  if befehl[8]<>' ' then writeln(' ERROR  command too long');
  for i:= 1 to 8 do if befehl[i] in ['A'..'Z'] then
    befehl[i]:=chr(ord(befehl[i])+ord('a')-ord('A'));
  end;
end;

begin (* main *)
m$initialisiere(strukturdaten);
repeat
  m$read_befehl(befehl);
       if befehl= '        ' then
  else if befehl= 'calcs   ' then m$berechne_koord_seq
  else if befehl= 'cald    ' then m$berechne_distanzen(maxanzahldistanzen)
  else if befehl= 'calda   ' then berechne_distanzen_algebraisch(rginfo)
  else if befehl= 'creab   ' then m$berechne_bindungen
  else if befehl= 'callc   ' then m$berechne_loop_konfig
  else if befehl= 'addo    ' then m$berechne_o_positionen
  else if befehl= 'adda    ' then m$erzeuge_atomlage
  else if befehl= 'creb    ' then m$erzeuge_bindung
  else if befehl= 'conb    ' then m$write_bindungs_matrix(output)
  else if befehl= 'cons    ' then m$contr_st$raumgruppe
  else if befehl= 'dela    ' then m$loesche_atomlage
  else if befehl= 'delab   ' then m$loesche_alle_bindung
  else if befehl= 'delb    ' then m$loesche_bindung
  else if befehl= 'delo    ' then m$verknuepfe_t_atome_direkt
  else if befehl= 'dett    ' then m$bestimme_topologie
  else if befehl= 'disab   ' then m$write_alle_bindungen(output)
  else if befehl= 'disb    ' then m$write_bindungen(output)
  else if befehl= 'discs   ' then m$write_strukturdaten(output,strukturdaten)
  else if befehl= 'disad   ' then m$write_alle_distanzen(output)
  else if befehl= 'disd    ' then m$write_distanzen(output)
  else if befehl= 'disda   ' then m$write_distanzen_und_winkel(output)
  else if befehl= 'disdv   ' then m$write_distanzen_und_vektoren(output)
  else if befehl= 'disc    ' then m$write_koordinaten(output)
  else if befehl= 'dise    ' then m$write_entry_liste
  else if befehl= 'diss    ' then write_st$raumgruppe(output,rginfo)
  else if befehl= 'help    ' then m$write_hilfe
  else if befehl= 'quit    ' then writeln
  else if befehl= 'exit    ' then writeln
  else if befehl= 'reacs   ' then m$read_strukturdaten
  else if befehl= 'reafcs  ' then m$read_erste_strukturdaten
  else if befehl= 'reancs  ' then m$read_naechste_strukturdaten
  else if befehl= 'wrie    ' then m$write_strukturdaten_karte
  else if befehl= 'wrics   ' then m$write_strukturdaten(list,strukturdaten)
  else if befehl= 'wrida   ' then m$write_distanzen_und_winkel(list)
  else if befehl= 'wridv   ' then m$write_distanzen_und_vektoren(list)
  else if befehl= 'wris    ' then write_st$raumgruppe(list,rginfo)
  else if befehl= 'wric    ' then m$write_koordinaten(list)
  else if befehl= 'wriad   ' then m$write_alle_distanzen(list)
  else if befehl= 'wrid    ' then m$write_distanzen(list)
  else if befehl= 'wriab   ' then m$write_alle_bindungen(list)
  else if befehl= 'wrib    ' then m$write_bindungen(list)
  else if befehl= 'wriid   ' then m$DLSinput
  else if befehl= 'wriil   ' then m$LOADATinput
  else writeln(output,' ERROR  unknown command ',befehl);
  writeln;
  until (befehl='quit    ') or (befehl='exit    ');
writeln(' ******************************************************************',
'************');
writeln(' *   end of program  KRIBER                                        ',
'           *');
writeln(' ******************************************************************',
'************');
end.
